package com.cuit.basin.base.biz.config;

import com.github.xiaoymin.knife4j.spring.annotations.EnableKnife4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.core.env.Profiles;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.contexts.SecurityContext;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.*;

@EnableSwagger2
@Configuration
@EnableKnife4j
public class Swagger3config {
    @Bean
    public Docket createRestApi(Environment environment) {

        //设置要显示swagger UI的环境
        Profiles profiles = Profiles.of("dev", "test");

        //通过环境判断
        boolean swaggerflag = environment.acceptsProfiles(profiles);
        return new Docket(DocumentationType.OAS_30)
                .apiInfo(apiInfo())
                .enable(swaggerflag)
                // 支持的通讯协议集合
                .protocols(new LinkedHashSet<>(Arrays.asList("https", "http")))
                .securityContexts(Arrays.asList(securityContext()))
                // ApiKey的name需与SecurityReference的reference保持一致
                .securitySchemes(Arrays.asList(new ApiKey("token", "token", "header")))
                //.securitySchemes(securitySchemes())// 授权信息设置，必要的header token等认证信息
                //.securityContexts(securityContexts())// 授权信息全局应用
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.cuit.sriver.sriversys.controller"))
                .paths(PathSelectors.any())
                .build();
    }
//                .host("http://121.37.165.19:8083")

    private ApiInfo apiInfo() {
        return new ApiInfo("S流域系统接口说明",
                "S流域系统接口RESTful API说明",
                "1.0",
                "http://www.cuit.edu.cn",
                new Contact("3S团队", "http://www.cuitgis.com", "cuitgis@qq.com"),
                "Apache 2.0",
                "http://www.apache.org/licenses/LICENSE-2.0",
                new ArrayList());
    }

    private SecurityContext securityContext() {
        return SecurityContext.builder()
                .securityReferences(defaultAuth())
                //.forPaths(PathSelectors.regex("/*.*"))
                .build();
    }

    private List<SecurityReference> defaultAuth() {
        AuthorizationScope authorizationScope
                = new AuthorizationScope("global", "accessEverything");
        AuthorizationScope[] authorizationScopes = new AuthorizationScope[1];
        authorizationScopes[0] = authorizationScope;
        return Collections.singletonList(new SecurityReference("token", authorizationScopes));
    }

    /**
     * 授权信息全局应用
     */
    private List<SecurityContext> securityContexts() {
        return Collections.singletonList(
                SecurityContext.builder().securityReferences(Collections.singletonList(new SecurityReference("token", new AuthorizationScope[]{new AuthorizationScope("global", "")}))).build());
    }

}
