package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.service.AccuracyEvaluationService;

import com.cuit.basin.base.biz.vo.AccuracyEvaluationView;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

@Api(tags = "05精度评估")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/accuracyevaluation")
public class AccuracyEvaluationController {
    @Autowired
    private AccuracyEvaluationService accuracyEvaluationService;

    /**
     * @author: PYJ
     * @date: 2022-12-4 9:19
     * @version: 1.0
     */
    @ApiOperation("查询精度评估信息")
    @RequestMapping(value = "/queryaccuracyevaluationinfo", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stcd", value = "站点编号(S1001)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mid", value = "mid（1/2）", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "dataTime", value = "起报时间（2022-12-03 00:00:00）", required = true, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<AccuracyEvaluationView> queryAccuracyEvaluationInfo(String stcd, Integer mid,
                                                                         @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime dataTime) {
        try {
            AccuracyEvaluationView view = accuracyEvaluationService.queryAccuracyEvaluationInfo(stcd, mid, dataTime);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("条件查询水文预测列表信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
