package com.cuit.basin.base.biz.controller;

import com.cuit.basin.base.biz.pojo.HydroLocation;
import com.cuit.basin.base.biz.service.CommonFunctionService;
import com.cuit.basin.base.biz.service.HydroDataQueryService;
import com.cuit.basin.base.biz.service.RTMonitoringBaseService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Api(tags = "00公共功能")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/commonfunction")
public class CommonFunctionController {
    @Autowired
    CommonFunctionService commonFunctionService;
    @Resource
    RTMonitoringBaseService rtMonitoringBaseService;
    @Autowired
    HydroDataQueryService hydroDataQueryService;

    @ApiOperation("01查询最新消息")
    @RequestMapping(value = "/querylatestmessage", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "typeid", value = "类型id（0：全部 1：通知 2：系统）", required = true, dataTypeClass = Integer.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 10)
    public RetResult<MessageInfoView> queryLatestMessage(Integer typeid) {
        try {
            MessageInfoView messageInfoView = commonFunctionService.queryMessageInfo(typeid);
            return RetResponse.makeOKRsp(messageInfoView);
        } catch (Exception e) {
            log.error("01查询最新消息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("02关键字提示")
    @RequestMapping(value = "/querykeywords", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "key", value = "关键字（如输入s）", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 20)
    public RetResult<Object> queryKeyWords(String key) {
        try {
            Object res = rtMonitoringBaseService.queryKeyWords(key);
            return RetResponse.makeOKRsp(res);
        } catch (Exception e) {
            log.error("02关键字提示出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("03按关键字查询")
    @RequestMapping(value = "/queryresultbykeywords", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "keyword", value = "关键字(如S1001)", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 30)
    public RetResult<HydroLocation> queryResultByKeywords(String keyword) {
        try {
            HydroLocation res = rtMonitoringBaseService.queryResultByKeyWord(keyword);
            return RetResponse.makeOKRsp(res);
        } catch (Exception e) {
            log.error("03按关键字查询出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("04查询水文站点列表")
    @RequestMapping(value = "/queryhydrostationlist", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "key", value = "站点名称或编号", required = false, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 40)
    public RetResult<HydroStationInfoView> queryHydroStationList(String key) {
        try {
            HydroStationInfoView hydroStationInfoView = commonFunctionService.queryHydroStationList(key);
            return RetResponse.makeOKRsp(hydroStationInfoView);
        } catch (Exception e) {
            log.error("04查询水文站点列表出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("05查询流域排序")
    @RequestMapping(value = "/querywatershedorder", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 50)
    public RetResult<WatershedInfoView> queryWatershedOrder() {
        try {
            WatershedInfoView watershedInfoView = commonFunctionService.queryWaterShedOrder();
            return RetResponse.makeOKRsp(watershedInfoView);
        } catch (Exception e) {
            log.error("05查询流域排序出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("06查询水文站排序")
    @RequestMapping(value = "/queryhydrostationorder", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "orderby", value = "排序（可选：流量变化、水位变化）", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 60)
    public RetResult<HydroStationDataView> queryHydroStationOrder(String orderby) {
        try {
            HydroStationDataView hydroStationDataView = hydroDataQueryService.queryLatestHydroStationDataOrder(orderby);
            return RetResponse.makeOKRsp(hydroStationDataView);
        } catch (Exception e) {
            log.error("06查询水文站排序出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("07查询模型类型字典")
    @RequestMapping(value = "/queryHydrologicalForecastModelDic", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 70)
    public RetResult<ModelDicView> queryHydrologicalForecastModelDic() {
        try {
            return RetResponse.makeOKRsp(commonFunctionService.queryHydrologicalForecastModelDic());
        } catch (Exception e) {
            log.error("查询预报模型信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("08查询流域列表")
    @RequestMapping(value = "/querybasinlist", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 80)
    public RetResult<WatershedDataView> querybasinlist() {
        try {
            WatershedDataView watershedDataView = commonFunctionService.queryWatershedList();
            return RetResponse.makeOKRsp(watershedDataView);
        } catch (Exception e) {
            log.error("04查询水文站点列表出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
