package com.cuit.basin.base.biz.controller;

import com.cuit.basin.base.biz.service.DictionaryFetchService;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.cuit.basin.framework.log.annotation.AopLog;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author hpj
 * @describe 字典获取
 */
@Api(tags = "11、字典操作")
@Slf4j
@RestController
@RequestMapping("/dictionary")
@Transactional(rollbackFor = Exception.class)
public class DictionaryFetchController {

    @Autowired
    private DictionaryFetchService dictionaryFetchService;

    @ApiOperation(value = "查询字典类列表")
    @RequestMapping(value = "/fetchDictionary", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "dictionaryType", value = "待查询字典类名称：输入（<部门、权限等级、职位>）", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public <T> RetResult<List<T>> fetchDictionary(String dictionaryType) {
        List dataList;
        try {
            dataList = dictionaryFetchService.fetchDictionaryByName(dictionaryType);
            return RetResponse.makeOKRsp(dataList);
        } catch (ParamsException p) {
            return RetResponse.makeRsp(500, p.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeRsp(500, "服务器错误...");

        }
    }


}
