package com.cuit.basin.base.biz.controller;

import com.cuit.basin.base.biz.entity.HydroStationData;
import com.cuit.basin.base.biz.service.HistoricalCasesService;
import com.cuit.basin.base.biz.vo.HistoricalCasesDetailView;
import com.cuit.basin.base.biz.vo.HistoricalCasesView;
import com.cuit.basin.base.biz.vo.HydrologicalEventView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.PageVo;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDateTime;

@Api(tags = "08历史个例")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/historycases")
public class HistoricalCasesController {

    @Resource
    private HistoricalCasesService historicalCasesService;

    @ApiOperation("多条件查询水文事件信息（时间范围）")
    @RequestMapping(value = "/getHydrologicalEventListByTime", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型（水位超保证/水位超警戒/流量超阈值）【水位超保证】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "stTime", value = "开始时间 yyyy-MM-dd HH:mm:ss【2022-11-01 00:00:00】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间 yyyy-MM-dd HH:mm:ss【2022-12-31 00:00:00】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "keyword", value = "关键词", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 10)
    public RetResult<HydrologicalEventView> getHydrologicalEventListByTime(String type, String stTime, String endTime, String keyword, BasePageQuery query) {
        try {
            HydrologicalEventView ret = historicalCasesService.getHydrologicalEventListByTime(type, stTime, endTime, keyword, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("多条件查询水文事件信息（固定时段）")
    @RequestMapping(value = "/getHydrologicalEventListByPeriod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型（水位超保证/水位超警戒/流量超阈值）【水位超保证】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "时间段（前24小时/前3天/前1周/前1月）【前1月】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "keyword", value = "关键词", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 20)
    public RetResult<HydrologicalEventView> getHydrologicalEventListByPeriod(String type, String period, String keyword, BasePageQuery query) {
        try {
            HydrologicalEventView ret = historicalCasesService.getHydrologicalEventListByPeriod(type, period, keyword, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("批量删除水文事件")
    @RequestMapping(value = "/deleteHydrologicalEvent", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "eids", value = "主键id列表（多个以英文逗号隔开）", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 30)
    public RetResult<Boolean> deleteHydrologicalEvent(String eids) {
        try {
            if (historicalCasesService.deleteHydrologicalEvent(eids)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("删除水文事件出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

//    @ApiOperation("新增历史个例（从水文事件新增）")
//    @RequestMapping(value = "/addHistoryCaseFromEvent", method = {RequestMethod.POST})
////    @ApiImplicitParam(name = "",value = "",required = false,dataTypeClass = String.class)
//    @ApiResponses({
//            @ApiResponse(code = 200, message = "成功"),
//            @ApiResponse(code = 401, message = "访问未经授权"),
//            @ApiResponse(code = 403, message = "访问被禁止的"),
//            @ApiResponse(code = 404, message = "未找到访问接口"),
//            @ApiResponse(code = 500, message = "服务器内部错误"),
//            @ApiResponse(code = 900, message = "失败")
//    })
//    public RetResult<Boolean> addHistoryCaseFromEvent() {
//        try {
//            if (historicalCasesService.addHistoryCaseFromEvent()) {
//                return RetResponse.makeOKRsp();
//            } else {
//                return RetResponse.makeErrRsp("新增历史个例出错！");
//            }
//        } catch (ParamsException e) {
//            log.error("参数错误！" + e.getMessage());
//            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
//        } catch (ErrorMessage e) {
//            log.error(e.getMessage());
//            return RetResponse.makeErrRsp(e.getMessage());
//        } catch (Exception e) {
//            log.error("服务器错误！" + e.getMessage());
//            return RetResponse.makeErrRsp("服务器错误！");
//        }
//    }

    @ApiOperation("多条件查询历史个例信息（时间范围）")
    @RequestMapping(value = "/getHistoricalCasesByTime", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime", value = "开始时间 yyyy-MM-dd HH:mm:ss【2021-11-01 00:00:00】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间 yyyy-MM-dd HH:mm:ss【2021-12-01 00:00:00】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "keyword", value = "关键词", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 40)
    public RetResult<HistoricalCasesView> getHistoricalCasesByTime(String stTime, String endTime, String keyword, BasePageQuery query) {
        try {
            HistoricalCasesView ret = historicalCasesService.getHistoricalCasesByTime(stTime, endTime, keyword, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("多条件查询历史个例信息（固定时段）")
    @RequestMapping(value = "/getHistoricalCasesByPeriod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "period", value = "时间段（前24小时/前3天/前1周/前1月）【前1月】", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "keyword", value = "关键词", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 50)
    public RetResult<HistoricalCasesView> getHistoricalCasesByPeriod(String period, String keyword, BasePageQuery query) {
        try {
            HistoricalCasesView ret = historicalCasesService.getHistoricalCasesByPeriod(period, keyword, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("查询历史个例详情信息")
    @RequestMapping(value = "/getHistoricalCasesDetail", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "cid", value = "历史个例id【1】", required = true, dataTypeClass = Integer.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 60)
    public RetResult<HistoricalCasesDetailView> getHistoricalCasesDetail(Integer cid, BasePageQuery query) {
        try {
            HistoricalCasesDetailView ret = historicalCasesService.getHistoricalCasesDetail(cid, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("新增历史个例信息")
    @RequestMapping(value = "/addHistoricalCase", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name", value = "个例名称【测试个例名称333】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "desc", value = "个例描述【测试个例描述123】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "stTime", value = "开始时间 yyyy-MM-dd HH:mm:ss【2018-11-08 00:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间 yyyy-MM-dd HH:mm:ss【2018-11-08 01:00:00】", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 70)
    public RetResult<Boolean> addHistoricalCase(String name, String desc,
                                                @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime) {
        try {
            if (historicalCasesService.addHistoricalCase(name, desc, stTime, endTime)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("新增历史个例出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照时间范围查询水文站点数据")
    @RequestMapping(value = "/getHydroStationData", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime", value = "开始时间 yyyy-MM-dd HH:mm:ss【2021-11-08 00:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间 yyyy-MM-dd HH:mm:ss【2021-11-08 01:00:00】", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 80)
    public RetResult<PageVo<HydroStationData>> getHydroStationData(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                                   @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime,
                                                                   BasePageQuery query) {
        try {
            PageVo<HydroStationData> ret = historicalCasesService.getHydroStationData(stTime, endTime, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("批量删除历史个例")
    @RequestMapping(value = "/deleteHistoricalCases", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "cids", value = "历史个例id列表（多个以英文逗号隔开）", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 90)
    public RetResult<Boolean> deleteHistoricalCases(String cids) {
        try {
            if (historicalCasesService.deleteHistoricalCases(cids)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("删除历史个例出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

}
