package com.cuit.basin.base.biz.controller;



import com.cuit.basin.base.biz.service.HydrologicalAnalysisService;
import com.cuit.basin.base.biz.vo.HydroAnalysisView;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDate;

@Api(tags = "06水情分析")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/hydrologicalanalysis")
public class HydrologicalAnalysisController {
    @Autowired
    HydrologicalAnalysisService hydrologicalAnalysisService;
    @ApiOperation("01查询同比-环比数据列表")
    @RequestMapping(value = "/queryAnalysisData", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationNo", value = "站号（S1001）", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "queryType", value = "查询类型(同比，环比)", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "timePeriod", value = "时段(年，季，月，旬，周)", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "开始时间（2022-11-07）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间（2022-11-14）", required = false, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 10)
    public RetResult<HydroAnalysisView> queryAnalysisData(String stationNo, String queryType, String timePeriod, @DateTimeFormat(pattern = Constant.PATTERN_DATE)LocalDate startTime,   @DateTimeFormat(pattern = Constant.PATTERN_DATE)LocalDate endTime) {
        try {
            HydroAnalysisView hydroAnalysisView = hydrologicalAnalysisService.queryAnalysisData(stationNo, queryType,timePeriod, startTime, endTime);
            return RetResponse.makeOKRsp(hydroAnalysisView);
        } catch (ParamsException e){
            log.error("0601查询同比环比分析参数输入错误！" + e.getMessage());
            return RetResponse.makeRsp(-1, e.getMessage());
        } catch (Exception e) {
            log.error("0601查询同比环比分析信息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("02查询同比-环比增长率")
    @RequestMapping(value = "/queryAnylysisData", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationNo", value = "站号（S1001）", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "queryType", value = "查询类型(同比，环比)", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "timePeriod", value = "时段(年，季，月，旬，周)", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "开始时间（2022-11-07）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间（2022-11-14）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "dataType", value = "数值类型(最高水位，最大流量，平均水位，平均流量，最低水位，最小流量)", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "characterType", value = "统计类型(最大值，最小值，平均值)", required = true, dataTypeClass = String.class, paramType = "query"),
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 20)
    public RetResult<Double> queryAnalysisRatio(String stationNo, String queryType, String timePeriod, @DateTimeFormat(pattern = Constant.PATTERN_DATE)LocalDate startTime, @DateTimeFormat(pattern = Constant.PATTERN_DATE)LocalDate endTime, String dataType, String characterType) {
        try {
            Double ratio = hydrologicalAnalysisService.queryAnalysisRatio(stationNo, queryType,timePeriod, startTime, endTime, dataType, characterType);
            return RetResponse.makeOKRsp(ratio);
        } catch (ParamsException e){
            log.error("0601查询同比环比分析参数输入错误！" + e.getMessage());
            return RetResponse.makeRsp(-1, e.getMessage());
        } catch (Exception e) {
            log.error("0601查询同比环比分析信息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
