package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.service.HydrologicalCorrectionService;
import com.cuit.basin.base.biz.vo.HydroForeCurrDataView;
import com.cuit.basin.base.biz.vo.HydroForeCurrReferView;
import com.cuit.basin.base.biz.vo.ManualCorrectionCurveView;
import com.cuit.basin.base.biz.vo.ModuleFittedDataView;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "04预报订正")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/hydrologicalcorrection")
public class HydrologicalCorrectionController {

    @Autowired
    private HydrologicalCorrectionService correctionService;
    /**
     * 1、模型曲线前半段（预测变成历史的数据）（条件：预报时效）
     * 2、后半段（最新起报时间往后的预测曲线[所有时效]）
     * 3、时况曲线（系统本地时间往前推（时效））
     * 4、初始预报曲线（条件（1）快速订正（2）公式订正）
     */

    /**
     * @author: zwx
     * @date: 2022/11/16 20:28
     * @version: 1.0
     */
    @ApiOperation("查询模型参考曲线")
    @RequestMapping(value = "/queryforecastreferencecurve", method = {RequestMethod.GET})
    @ApiOperationSupport(order = 10)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationNo", value = "站号（S1001）", required = true, dataTypeClass = String.class, paramType = "query", example = "S1001"),
            @ApiImplicitParam(name = "validTime", value = "预报时效(1)", required = true, dataTypeClass = String.class, paramType = "query", example = "1"),
            @ApiImplicitParam(name = "timePeriod", value = "时间周期(小时)(24)", required = true, dataTypeClass = String.class, paramType = "query", example = "48")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<HydroForeCurrReferView> queryForecastReferenceCurve(String stationNo, Integer validTime, Integer timePeriod) {
        try {
            HydroForeCurrReferView view = correctionService.queryForecastReferenceCurve(stationNo, validTime, timePeriod);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("查询历史曲线时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @author: zwx
     * @date: 2022/11/16 20:28
     * @version: 1.0
     */
    @ApiOperation("查询实况模型拟合曲线")
    @RequestMapping(value = "/querymodulefitteddatacurve", method = {RequestMethod.GET})
    @ApiOperationSupport(order = 10)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationNo", value = "站号（S1001）", required = true, dataTypeClass = String.class, paramType = "query", example = "S1001"),
            @ApiImplicitParam(name = "timePeriod", value = "历史拟合周期(d)", required = true, dataTypeClass = String.class, paramType = "query", example = "1")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<ModuleFittedDataView> queryModuleFittedDataCurve(String stationNo, Integer timePeriod) {
        try {
            ModuleFittedDataView view = correctionService.queryModuleFittedDataCurve(stationNo, timePeriod);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("查询实况模型拟合曲线时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

//    /**
//     * @author: zwx
//     * @date: 2022/11/16 20:30
//     * @version: 1.0
//    */
//    @ApiOperation("获取最新起报时间的预测曲线")
//    @RequestMapping(value = "/queryforecastcurve", method = {RequestMethod.GET})
//    @ApiOperationSupport(order = 11)
//    @ApiResponses({
//            @ApiResponse(code = 200, message = "成功"),
//            @ApiResponse(code = 401, message = "访问未经授权"),
//            @ApiResponse(code = 403, message = "访问被禁止的"),
//            @ApiResponse(code = 404, message = "未找到访问接口"),
//            @ApiResponse(code = 500, message = "服务器内部错误"),
//            @ApiResponse(code = 900, message = "失败")
//    })
//    public RetResult<HydroForeCurrDataView> queryForecastCurve() {
//        try {
//            return RetResponse.makeOKRsp();
//        } catch (Exception e) {
//            log.error("获取最新起报时间的预测曲线时服务器出现错误！" + e.getMessage());
//            return RetResponse.makeRsp(500, e.getMessage());
//        }
//    }


    /**
     * @author: zwx
     * @date: 2022/11/16 20:25
     * @version: 1.0
     */
    @ApiOperation("预报订正曲线初始化")
    @RequestMapping(value = "/queryhydrologicalcorrectionbyparam", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationNo", value = "站号（S1001）", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "correctType", value = "订正类型(快速订正/公式订正)", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "correctStr", value = "订正参数(最大、最小、平均/公式字符(1,0.3+2,0.4))", required = true, dataTypeClass = String.class, paramType = "query")})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<HydroForeCurrDataView> queryHydrologicalCorrectionByParam(String stationNo, String correctType, String correctStr) {
        try {
            HydroForeCurrDataView view = correctionService.queryHydrologicalCorrectionByParam(stationNo, correctType, correctStr);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("预报订正曲线初始化时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: zwx
     * @date: 2022/11/16 20:00
     * @version: 1.0
     */
    @ApiOperation("保存手动订正预报曲线")
    @RequestMapping(value = "/savemanualcorrectionforecastcurve", method = {RequestMethod.POST})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> saveManualCorrectionForecastCurve(@RequestBody ManualCorrectionCurveView view) {
        try {
        if (correctionService.saveManualCorrectionForecastCurve(view)) {
            return RetResponse.makeOKRsp(true);
        }
        return RetResponse.makeErrRsp("保存手动订正预报曲线失败");
        } catch (Exception e) {
            log.error("保存手动订正预报曲线信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


}
