package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.query.HydrologicalForecastListQuery;
import com.cuit.basin.base.biz.query.HydrologicalForecastQuery;
import com.cuit.basin.base.biz.query.RiverCrossSectionQuery;
import com.cuit.basin.base.biz.service.HydrologicalForecastService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

@Api(tags = "03水文预测")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/hydrologicalforecast")
public class HydrologicalForecastController {

    @Autowired
    private HydrologicalForecastService forecastService;

    @ApiOperation("条件查询水文预测列表信息")
    @RequestMapping(value = "/queryHydrologicalForecastListByParam", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<HydroStationForeView> queryHydrologicalForecastListByParam(HydrologicalForecastListQuery query) {
        try {
            HydroStationForeView hydroStationForeView = forecastService.queryHydrologicalForecastListByParam(query);
            return RetResponse.makeOKRsp(hydroStationForeView);
        } catch (Exception e) {
            log.error("条件查询水文预测列表信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("点击展开站点详情")
    @RequestMapping(value = "/queryHydrologicalForecastModule", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stcd", value = "站点编号(S1001)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "dataTime", value = "预报时间(2022-11-17 00:00:00)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "time", value = "预报时效(1,2,3,...,11,12)", required = true, dataTypeClass = Integer.class)})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<SingleStationForeDataView> queryHydrologicalForecastModule(String stcd, @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") LocalDateTime dataTime, Integer time) {
        try {
            SingleStationForeDataView singleStationForeDataView = forecastService.queryHydroSingleStationModelForeInfo(stcd, dataTime, time);
            return RetResponse.makeOKRsp(singleStationForeDataView);
        } catch (Exception e) {
            log.error("查询预报模型信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    @ApiOperation("条件查询站点水文预报详情")
    @RequestMapping(value = "/queryHydrologicalForecastView", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<HydroForecastDetailView> queryHydrologicalForecastView(HydrologicalForecastQuery query) {

        try {
            HydroForecastDetailView hydroForecastDetailView;
            hydroForecastDetailView = forecastService.queryHydrologicalForecastDetails(query);
            return RetResponse.makeOKRsp(hydroForecastDetailView);
        } catch (Exception e) {
            log.error("条件查询站点水文预报详情时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("条件查询河流断面详情")
    @RequestMapping(value = "/queryRiverCrossSectionView", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<HydroForeSectionDetailView> queryRiverCrossSectionView(RiverCrossSectionQuery query) {
        HydroForeSectionDetailView riverCrossSectionView;
        try {
            riverCrossSectionView = forecastService.queryRiverCrossSectionForeDetails(query);
            return RetResponse.makeOKRsp(riverCrossSectionView);
        } catch (Exception e) {
            log.error("查询预报模型信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
