package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.entity.DicBulletinStutus;
import com.cuit.basin.base.biz.entity.DicBulletinType;
import com.cuit.basin.base.biz.entity.HydPiBulletinP;
import com.cuit.basin.base.biz.pojo.*;
import com.cuit.basin.base.biz.service.HydrologicalBulletinService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.ret.PageVo;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.List;

@Api(tags = "07水情服务")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/hydrologicalservice")
public class HydrologicalServiceController {
    @Autowired
    HydrologicalBulletinService hydrologicalBulletinService;

    /**
     * @author: zwx
     * @date: 2022/12/10 15:26
     * @version: 1.0
     */
    @ApiOperation("条件分页查询公报列表")
    @RequestMapping(value = "/querygazettelistbyparams", method = {RequestMethod.GET})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<HydPiBulletinPInfo>> queryGazetteListByParams(GazetteListQuery query) {
        try {
            PageVo<HydPiBulletinPInfo> pageVo = hydrologicalBulletinService.queryGazetteListByParams(query);
            return RetResponse.makeOKRsp(pageVo);
        } catch (Exception e) {
            log.error("条件分页查询公报列表服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: zwx
     * @date: 2022/12/10 16:15
     * @version: 1.0
     */
    @ApiOperation("查询公报信息")
    @RequestMapping(value = "/querygazettebyid", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "bId", value = "公报Id", required = true, dataTypeClass = String.class, paramType = "query")

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<HydPiBulletinP> queryGazetteById(Integer bId) {
        try {
            HydPiBulletinP hydPiBulletinP = hydrologicalBulletinService.queryGazetteById(bId);
            return RetResponse.makeOKRsp(hydPiBulletinP);
        } catch (Exception e) {
            log.error("查询公报信息时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: zwx
     * @date: 2022/12/10 16:15
     * @version: 1.0
     */
    @ApiOperation("批量删除公报")
    @RequestMapping(value = "/batchdeletegazettebyids", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "bIds", value = "公报Id(多个,隔开)", required = true, dataTypeClass = String.class, paramType = "query")

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> batchDeleteGazetteByIds(String bIds) {
        try {if (hydrologicalBulletinService.batchDeleteGazetteByIds(bIds)){
            return RetResponse.makeOKRsp(true);
        }
            return RetResponse.makeErrRsp("批量删除公报失败");
        } catch (Exception e) {
            log.error("批量删除公报时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: zwx
     * @date: 2022/12/10 16:15
     * @version: 1.0
     */
    @ApiOperation("批量下载公报")
    @RequestMapping(value = "/batchdownloadgazettebyids", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "bIds", value = "公报Id(多个,隔开)", required = true, dataTypeClass = String.class, paramType = "query")

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> batchDownLoadGazetteByIds(String bIds) {
        try {
            List<String> urls = hydrologicalBulletinService.batchDownLoadGazetteByIds(bIds);
            return RetResponse.makeOKRsp(urls);
        } catch (Exception e) {
            log.error("批量下载公报时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ASUS
     * @date: 2022-12-13 15:14
     * @version: 1.0
     */
    @ApiOperation("获取雨情公报生成条件")
    @RequestMapping(value = "/getgazetterainparamsview", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime",value = "开始时间",required = true,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "endTime",value = "结束时间",required = true,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "situation",value = "超警情况",required = false,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "bsId",value = "流域id",required = false,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "keywords",value = "关键字",required = false,dataTypeClass = String.class,paramType = "query" )
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<GazetteRainParamsView> getGazetteRainParamsView(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                                     @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime,
                                                                     String situation, Integer bsId, String keywords) {
        try {
            GazetteRainParamsView view = hydrologicalBulletinService.getGazetteRainParamsView(stTime, endTime, situation, bsId, keywords);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("获取公报雨情生成条件时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @author: ASUS
     * @date: 2022-12-17 20:04
     * @version: 1.0
    */
    @ApiOperation("获取特征值字典")
    @RequestMapping(value = "/getdiceigenvalue", method = {RequestMethod.GET})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<EigenvalueView>getDicEigenvalue(){
        try {
            EigenvalueView view = hydrologicalBulletinService.getDicEigenvalue();
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("获取公报雨情生成条件时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @author: ASUS
     * @date: 2022-12-13 15:14
     * @version: 1.0
     */
    @ApiOperation("获取水情公报生成条件")
    @RequestMapping(value = "/getgazetteregimenparamsview", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime",value = "开始时间",required = true,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "endTime",value = "结束时间",required = true,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "situation",value = "超警情况",required = false,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "bsId",value = "流域id",required = false,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "keywords",value = "关键字",required = false,dataTypeClass = String.class,paramType = "query" )
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<GazetteRegimenParamsView> getGazetteRegimenParamsView(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                                           @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime,
                                                                           String situation, Integer bsId, String keywords) {
        try {
            GazetteRegimenParamsView view = hydrologicalBulletinService.getGazetteRegimenParamsView(stTime, endTime, situation, bsId, keywords);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("获取公报雨情生成条件时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @author: ASUS
     * @date: 2022-12-13 15:14
     * @version: 1.0
     */
    @ApiOperation("获取模型预报公报生成条件")
    @RequestMapping(value = "/getgazettemodeforecastparamsview", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime",value = "开始时间",required = true,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "endTime",value = "结束时间",required = true,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "situation",value = "超警情况",required = false,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "bsId",value = "流域id",required = false,dataTypeClass = String.class,paramType = "query" ),
            @ApiImplicitParam(name = "keywords",value = "关键字",required = false,dataTypeClass = String.class,paramType = "query" )
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<GazetteModeForecastParamsView> getGazetteModeForecastParamsView(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                                                               @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime,
                                                                                               String situation, Integer bsId, String keywords) {
        try {
            GazetteModeForecastParamsView view = hydrologicalBulletinService.getGazetteModeForecastParamsView(stTime, endTime, situation, bsId, keywords);
            return RetResponse.makeOKRsp(view);
        } catch (Exception e) {
            log.error("获取公报雨情生成条件时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @author: zwx
     * @date: 2022/12/10 19:03
     * @version: 1.0
     */
    @ApiOperation("预览公报")
    @RequestMapping(value = "/producegazette", method = {RequestMethod.POST})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> produceGazette(@RequestBody ProduceGazetteParam param) {
//        try {
            String content = hydrologicalBulletinService.produceGazette(param);
            return RetResponse.makeOKRsp(content);
//        } catch (Exception e) {
//            log.error("预览公报时服务器出现错误！" + e.getMessage());
//            return RetResponse.makeRsp(500, e.getMessage());
//        }
    }

    /**
     * @author: zwx
     * @date: 2022/12/10 16:15
     * @version: 1.0
     */
    @ApiOperation("导出公报")
    @RequestMapping(value = "/savegazette", method = {RequestMethod.PUT})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> saveGazette(HydPiBulletinPParams params) {
//        try {
            String content = hydrologicalBulletinService.saveGazette(params);
            return RetResponse.makeOKRsp(content);
//        } catch (Exception e) {
//            log.error("导出公报时服务器出现错误！" + e.getMessage());
//            return RetResponse.makeRsp(500, e.getMessage());
//        }
    }

    /**
     * @author: zwx
     * @date: 2022/12/10 16:15
     * @version: 1.0
     */
    @ApiOperation("获取公报类型")
    @RequestMapping(value = "/getgazettetype", method = {RequestMethod.GET})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<DicBulletinType> > getGazetteType() {
        try {
            List<DicBulletinType>  types = hydrologicalBulletinService.getGazetteType();
            return RetResponse.makeOKRsp(types);
        } catch (Exception e) {
            log.error("获取公报类型时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("获取公报状态类型")
    @RequestMapping(value = "/getgazettestutus", method = {RequestMethod.GET})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<DicBulletinStutus>> getGazetteStutus() {
        try {
            List<DicBulletinStutus> stutus = hydrologicalBulletinService.getGazetteStutus();
            return RetResponse.makeOKRsp(stutus);
        } catch (Exception e) {
            log.error("获取公报状态类型时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("获取流域列表")
    @RequestMapping(value = "/getbsdatalist", method = {RequestMethod.GET})

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<DicBSData>> getDicBSDataList() {
        try {
            List<DicBSData> stutus = hydrologicalBulletinService.getDicBSDataList();
            return RetResponse.makeOKRsp(stutus);
        } catch (Exception e) {
            log.error("获取流域列表时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: zwx
     * @date: 2022/12/10 16:15
     * @version: 1.0
     */
    @ApiOperation("审核公报")
    @RequestMapping(value = "/auditgazette", method = {RequestMethod.PUT})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bId", value = "公报Id", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "stutusId", value = "状态Id", required = true, dataTypeClass = String.class, paramType = "query")
    })

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> auditGazette(String bId, Integer stutusId) {
        try {
            if (hydrologicalBulletinService.auditGazette(bId, stutusId)){
            return RetResponse.makeOKRsp(true);
        }
            return RetResponse.makeErrRsp("审核公报失败");
        } catch (Exception e) {
            log.error("审核公报时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
