package com.cuit.basin.base.biz.controller;

import com.cuit.basin.base.biz.entity.sys.User;
import com.cuit.basin.base.biz.service.LoginUserService;
import com.cuit.basin.framework.common.annotation.AuthToken;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.cuit.basin.framework.log.annotation.AopLog;
import com.cuit.basin.framework.log.aop.OperationEnum;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author wzw
 */
@Api(tags = "1、用户信息管理")
@Slf4j
@RestController
@Transactional(rollbackFor = Exception.class)
public class LoginUserController {
    public static final String REQUEST_CURRENT_KEY = "REQUEST_CURRENT_KEY";
    @Autowired
    private LoginUserService loginUserService;

    /**
     * @author: ZengwuWang
     * @date: 2022/5/7 13:18
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "获取Token", opType = OperationEnum.OPERATION_LOGIN)
    @ApiOperation("获取Token")
    @RequestMapping(value = "/getToken", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginname", value = "登录账号", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> getToken(String loginname, String password) {
        String tk = null;

        User user = null;
        try {
            user = loginUserService.checkUser(loginname, password);
            if (user != null) {
                try {
                    log.debug(user.getLoginname() + "登录验证成功！");
                    tk = loginUserService.getAppIdToken("DataService", user.getLoginname());
                } catch (ErrorMessage e) {
                    log.error("获取Appid[{}] token出现错误:{}", loginname, e.getMessage());
                    return RetResponse.makeErrRsp("服务器出错！");
                } catch (Exception e) {
                    log.error("获取Appid的Token时服务器出现错误！" + e.getMessage());
                    return RetResponse.makeErrRsp("服务器出错！");
                }
                return RetResponse.makeOKRsp(tk);
            } else {
                return RetResponse.makeErrRsp("拒绝访问！");
            }
        } catch (ErrorMessage e) {
            log.error("身份验证异常,验证账户[" + loginname + "] + 出现错误:" + e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("查询密码的时候服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: Administrator
     * @date: 2022/3/22 22:50
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "校验用户", opType = OperationEnum.OPERATION_LOGIN)
    @ApiOperation("校验用户")
    @RequestMapping(value = "/login", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginname", value = "账号", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<User> auth(@RequestParam String loginname, @RequestParam String password) {
        User user = null;
        try {
            user = loginUserService.checkUser(loginname, password);
        } catch (ErrorMessage e) {
            log.error("身份验证异常,验证账户[" + loginname + "] + 出现错误:" + e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("查询密码的时候服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
        return RetResponse.makeOKRsp(user);
    }

    /**
     * @author: hpj
     * @date: 2022/9/15 20:38
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "获取小程序密钥", opType = OperationEnum.OPERATION_LOGIN)
    @ApiOperation("获取小程序密钥")
    @RequestMapping(value = "/getAppSecret", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> getAppSecret() {
        return RetResponse.makeOKRsp("e22c61330e34b30e3af37f21834a2e9a");
    }


    /**
     * @author: Administrator
     * @date: 2022/3/22 23:33
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "重置密码", opType = OperationEnum.OPERATION_LOGIN)
    @AuthToken
    @ApiOperation("重置密码")
    @RequestMapping(value = "/setnewpassword", method = {RequestMethod.POST})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginname", value = "登录名", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "newpassword", value = "新密码", required = true, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<Boolean> setPassword(@RequestParam String loginname, @RequestParam String newpassword) {
        try {
            if (loginUserService.setUserPassword(loginname, newpassword)) {
                return RetResponse.makeOKRsp(true);
            } else {
                return RetResponse.makeErrRsp("重置密码失败！");
            }
        } catch (ParamsException p) {
            log.error("重置密码的时候服务器出现错误！" + p.getMessage());
            return RetResponse.makeRsp(500, p.getMessage());
        } catch (Exception e) {
            log.error("重置密码的时候服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: Administrator
     * @date: 2022/3/22 22:50
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "退出登录", opType = OperationEnum.OPERATION_LOGIN)
    @AuthToken
    @ApiOperation("退出登录")
    @RequestMapping(value = "/loginout", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<Boolean> loginOut(HttpServletRequest request, HttpServletResponse response) {
        String attribute = (String) request.getAttribute(REQUEST_CURRENT_KEY);
        boolean b = loginUserService.loginOut(attribute);
        return RetResponse.makeOKRsp(b);
    }


    /**
     * @author: ZengwuWang
     * @date: 2022/3/22 23:46
     * @version: 1.0
     */

    @ApiOperation("锁定用户")
    @RequestMapping(value = "/lock", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "loginname", value = "登录名", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<Boolean> lockUser(String loginname) {
        try {
            if (loginUserService.lockUser(loginname, true)) {
                return RetResponse.makeOKRsp(true);
            } else {
                return RetResponse.makeErrRsp("锁定用户失败！");
            }
        } catch (Exception e) {
            log.error("锁定用户时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ZengwuWang
     * @date: 2022/3/22 23:46
     * @version: 1.0
     */
    @AuthToken
    @ApiOperation("解锁用户")
    @RequestMapping(value = "/unlock", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "loginname", value = "登录名", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<Boolean> unlockUser(String loginname) {
        try {
            if (loginUserService.lockUser(loginname, false)) {
                return RetResponse.makeOKRsp(true);
            } else {
                return RetResponse.makeErrRsp("解锁用户失败！");
            }
        } catch (Exception e) {
            log.error("解锁用户时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ZengwuWang
     * @date: 2022/3/22 23:51
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "删除用户", opType = OperationEnum.OPERATION_DELETE)
    @ApiOperation("删除用户")
    @RequestMapping(value = "/deleteuser", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "loginname", value = "登录名", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<Boolean> deleteUser(@RequestParam String loginname) {
        try {
            if (loginUserService.deleteUser(loginname)) {
                return RetResponse.makeOKRsp(true);
            } else {
                return RetResponse.makeErrRsp("删除用户失败！");
            }
        } catch (Exception e) {
            log.error("删除用户时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ZengwuWang
     * @date: 2022/3/23 0:49
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "新增用户", opType = OperationEnum.OPERATION_ADD)
    @ApiOperation("新增用户")
    @RequestMapping(value = "/adduser", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 900, message = "失败"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误")
    })
    public RetResult<Boolean> addUser(User user) {
        try {
            if (loginUserService.addUser(user)) {
                return RetResponse.makeOKRsp(true);
            } else {
                return RetResponse.makeErrRsp("新增用户失败！");
            }
        } catch (Exception e) {
            log.error("新增用户时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: Xiao Yinrui
     * @date: 2022/7/20 15:47
     * @version: 1.0
     */
    @AopLog(module = "4", operation = "更新用户", opType = 2)
    @ApiOperation("更新用户")
    @RequestMapping(value = "/updateUser", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> updateUser(User user) {
        try {
            if (loginUserService.updateUser(user)) {
                return RetResponse.makeOKRsp(true);
            } else {
                return RetResponse.makeErrRsp("更新用户失败！");
            }
        } catch (Exception e) {
            log.error("更新用户时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
