package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.entity.ModelParam;
import com.cuit.basin.base.biz.service.ModelManagerService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author Xiao Yinrui
 */
@Api(tags = "10模型管理")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/modelmanager")
public class ModelManagerController {

    @Resource
    private ModelManagerService modelManagerService;

    @ApiOperation("查询模型列表信息")
    @RequestMapping(value = "/getModelDicList", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "status", value = "运行状态（1运行中 2停止）", required = false, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 10)
    public RetResult<ModelDicView> getModelDicList(Integer status) {
        ModelDicView ret = modelManagerService.getModelDicList(status);
        return RetResponse.makeOKRsp(ret);
    }

    @ApiOperation("新增模型基础信息")
    @RequestMapping(value = "/addModelInfo", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mname", value = "模型名称", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mdes", value = "模型描述", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "path", value = "存放路径", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "dfproc", value = "数据预报处理", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mfdata", value = "模型前置数据", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mprog", value = "模型程序", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mfprog", value = "模型前置程序", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "daproc", value = "数据后处理", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 20)
    public RetResult<Boolean> addModelInfo(String mname, String mdes, String path, String dfproc, String mfdata, String mprog, String mfprog, String daproc) {
        try {
            if (modelManagerService.addModelInfo(mname, mdes, path, dfproc, mfdata, mprog, mfprog, daproc)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("新增模型基础信息出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("编辑模型基础信息")
    @RequestMapping(value = "/editModelInfo", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mid", value = "模型id", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mname", value = "模型名称", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mdes", value = "模型描述", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "path", value = "存放路径", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "dfproc", value = "数据预报处理", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mfdata", value = "模型前置数据", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mprog", value = "模型程序", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mfprog", value = "模型前置程序", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "daproc", value = "数据后处理", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 30)
    public RetResult<Boolean> editModelInfo(Integer mid, String mname, String mdes, String path, String dfproc, String mfdata, String mprog, String mfprog, String daproc) {
        try {
            if (modelManagerService.editModelInfo(mid, mname, mdes, path, dfproc, mfdata, mprog, mfprog, daproc)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("编辑模型基础信息出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("批量删除模型信息")
    @RequestMapping(value = "/deleteModelInfo", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "mids", value = "模型id字符串列表（多个英文逗号隔开）", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 40)
    public RetResult<Boolean> deleteModelInfo(String mids) {
        try {
            if (modelManagerService.deleteModelInfo(mids)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("删除模型信息出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("立即启动某一个模型")
    @RequestMapping(value = "/startModelByMid", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "mid", value = "模型id", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 50)
    public RetResult<Boolean> startModelByMid(Integer mid) {
        try {
            if (modelManagerService.startModelByMid(mid)) {
                return RetResponse.makeRsp(200, "模型启动成功！");
            } else {
                return RetResponse.makeErrRsp("手动启动模型出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("停止某一个模型")
    @RequestMapping(value = "/stopModelByMid", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "mid", value = "模型id", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 60)
    public RetResult<Boolean> stopModelByMid(Integer mid) {
        try {
            if (modelManagerService.stopModelByMid(mid)) {
                return RetResponse.makeRsp(200, "模型停止成功！");
            } else {
                return RetResponse.makeErrRsp("手动停止模型出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("查询模型定时运行参数")
    @RequestMapping(value = "/getModelScheduler", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "mid", value = "模型id", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 70)
    public RetResult<CronHourMinView> getModelScheduler(Integer mid) {
        try {
            CronHourMinView ret = modelManagerService.getModelScheduler(mid);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("设置模型定时运行参数")
    @RequestMapping(value = "/scheduleModel", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 80)
    public RetResult<String> scheduleModel(CronHourMinView cronV) {
        try {
            String ret = modelManagerService.scheduleModel(cronV);
            if ("修改成功".equals(ret)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp(ret);
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("多条件查询模型运行日志")
    @RequestMapping(value = "/getModelLogList", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime", value = "开始时间 yyyy-MM-dd HH:mm:ss", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间 yyyy-MM-dd HH:mm:ss", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mids", value = "模型id字符串列表（多个用英文逗号隔开）", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "method", value = "运行方式（手动运行/定时调度）", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 90)
    public RetResult<ModelLogView> getModelLogList(String stTime, String endTime, String mids, String method, BasePageQuery query) {
        try {
            ModelLogView ret = modelManagerService.getModelLogList(stTime, endTime, mids, method, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("批量删除模型运行日志")
    @RequestMapping(value = "/deleteModelLogs", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "ids", value = "模型日志id字符串列表（多个英文逗号隔开）", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 100)
    public RetResult<Boolean> deleteModelLogs(String ids) {
        try {
            if (modelManagerService.deleteModelLogs(ids)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("删除模型运行日志出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("多条件查询模型运行校准信息")
    @RequestMapping(value = "/getModelParamList", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "mid", value = "模型id", required = true, dataTypeClass = Integer.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 110)
    public RetResult<ModelParamView> getModelParamList(Integer mid, BasePageQuery query) {
        try {
            ModelParamView ret = modelManagerService.getModelParamList(mid, query);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("模型参数运行校准（同时编辑某一个模型的参数）")
    @RequestMapping(value = "/editModelParamAndCalibration", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 120)
    public RetResult<Boolean> editModelParamAndCalibration(@RequestBody List<ModelParam> mParam) {
        try {
            if (modelManagerService.editModelParamAndCalibration(mParam)) {
                return RetResponse.makeRsp(200, "模型参数已修改，并开始重新运行校准。");
            } else {
                return RetResponse.makeErrRsp("编辑模型参数出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("批量删除模型运行校准信息")
    @RequestMapping(value = "/deleteModelParam", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "ids", value = "id字符串列表（多个英文逗号隔开）", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 130)
    public RetResult<Boolean> deleteModelParam(String ids) {
        try {
            if (modelManagerService.deleteModelParam(ids)) {
                return RetResponse.makeOKRsp();
            } else {
                return RetResponse.makeErrRsp("删除模型参数和校准信息出错！");
            }
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }
}
