package com.cuit.basin.base.biz.controller;
import com.cuit.basin.base.biz.entity.GeoserverData;
import com.cuit.basin.base.biz.pojo.LakeDataUnion;
import com.cuit.basin.base.biz.service.HydroDataQueryService;
import com.cuit.basin.base.biz.service.MeteDataQueryService;
import com.cuit.basin.base.biz.service.RTMonitoringBaseService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDateTime;

@Api(tags = "01实时监测基础数据")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/rtmonitoringbase")
public class RTMonitoringBaseController {
    @Resource
    RTMonitoringBaseService rtMonitoringBaseService;
    @Autowired
    HydroDataQueryService hydroDataQueryService;
    @Autowired
    MeteDataQueryService meteDataQueryService;

    @ApiOperation("01查询最新水文站点数据")
    @RequestMapping(value = "/queryhydrostaiondata", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 10)
    public RetResult<HydroStationDataView> queryHydroStaionData() {
        try {
            HydroStationDataView hydroStationDataView = hydroDataQueryService.queryLatestHydroStationData();
            return RetResponse.makeOKRsp(hydroStationDataView);
        } catch (Exception e) {
            log.error("01查询最新水文站点数据出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("02查询最新气象站点数据")
    @RequestMapping(value = "/querymetestaiondata", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 20)
    public RetResult<MeteStationDataView> queryMeteStaionData() {
        try {
            return RetResponse.makeOKRsp(new MeteStationDataView());
        } catch (Exception e) {
            log.error("02查询最新气象站点数据出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("03查询湖泊图层")
    @RequestMapping(value = "/querylakelayer", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 30)
    public RetResult<LakeDataUnion> queryLakeLayer() {
        try {
            LakeDataUnion lakeDataUnion = rtMonitoringBaseService.queryLakeLayer();
            return RetResponse.makeOKRsp(lakeDataUnion);
        } catch (Exception e) {
            log.error("03查询湖泊图层出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("04查询最新冰川图层列表")
    @RequestMapping(value = "/querylatestglacierlayer", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 40)
    public RetResult<GlacierView> queryLatestGlacierLayer() {
        try {
            GlacierView glacierView = rtMonitoringBaseService.queryLatestGlacierLayer();
            return RetResponse.makeOKRsp(glacierView);
        } catch (Exception e) {
            log.error("04查询最新冰川图层列表出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("05查询最新积雪图层列表")
    @RequestMapping(value = "/querylatestsnowlayer", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 50)
    public RetResult<SnowView> queryLatestSnowLayer() {
        try {
            SnowView snowView = rtMonitoringBaseService.queryLatestSnowLayer();
            return RetResponse.makeOKRsp(snowView);
        } catch (Exception e) {
            log.error("05查询最新积雪图层列表出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("06查询实况融合")
    @RequestMapping(value = "/queryrtintergrationdata", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "datatime", value = "数据时间", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "ele", value = "要素（温度、湿度、风速、气压、降水）", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 60)
    public RetResult<GeoserverData> queryRtIntergrationData(LocalDateTime datatime, String ele) {
        try {
            GeoserverData geoserverData = meteDataQueryService.queryRtIntergrationData(datatime, ele);
            return RetResponse.makeOKRsp(geoserverData);
        } catch (Exception e) {
            log.error("06查询实况融合出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    @ApiOperation("07查询预报格网")
    @RequestMapping(value = "/queryforecastinggriddata", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "datatime", value = "起报时间", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "ele", value = "要素（温度、降水、风速、风向）", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 70)
    public RetResult<GeoserverDataView> queryForecastingGridData(LocalDateTime datatime, String ele) {
        try {
            return RetResponse.makeOKRsp(new GeoserverDataView());
        } catch (Exception e) {
            log.error("07查询预报格网出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }
}
