package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.entity.HydroStationInfo;
import com.cuit.basin.base.biz.entity.MeteStationData;
import com.cuit.basin.base.biz.entity.MeteStationForcastingData;
import com.cuit.basin.base.biz.service.RTMonitoringDetailsService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.PageVo;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

@Api(tags = "02实时监测-详情面板")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/rtmonitoringdetails")
public class RTMonitoringDetailsController {
    @Resource
    private RTMonitoringDetailsService rtMonitoringDetailsService;

    @ApiOperation("按固定时段查询历史水位流量")
    @RequestMapping(value = "/queryhistoryhydrodatabyperiod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）【前1月】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 10)
    public RetResult<HydroHistoryView> queryHistoryHydroDataByPeriod(String stationidc, String period) {
        try {
            HydroHistoryView hydroDataHistory = rtMonitoringDetailsService.queryHistoryHydroDataByPeriod(stationidc, period);
            return RetResponse.makeOKRsp(hydroDataHistory);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按时间范围查询历史水位流量")
    @RequestMapping(value = "/queryhistoryhydrodatabytimerange", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "starttime", value = "开始时间【2022-11-01 00:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endtime", value = "结束时间【2022-11-29 00:00:00】", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 20)
    public RetResult<HydroHistoryView> queryHistoryHydroDataByTimeRange(String stationidc,
                                                                        @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime starttime,
                                                                        @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endtime) {
        try {
            HydroHistoryView hydroDataHistory = rtMonitoringDetailsService.queryHistoryHydroDataByTimeRange(stationidc, starttime, endtime);
            return RetResponse.makeOKRsp(hydroDataHistory);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按固定时段查询历史断面水位")
    @RequestMapping(value = "/queryhistorysectiondatabyperiod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）【前1月】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 30)
    public RetResult<SectionLevelView> queryHistorySectionDataByPeriod(String stationidc, String period) {
        try {
            SectionLevelView sectionLevelView = rtMonitoringDetailsService.queryHistorySectionDataByPeriod(stationidc, period);
            return RetResponse.makeOKRsp(sectionLevelView);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按时间范围查询历史断面水位")
    @RequestMapping(value = "/queryhistorysectiondatabytimerange", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "starttime", value = "开始时间【2022-11-01 00:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endtime", value = "结束时间【2022-11-29 00:00:00】", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 40)
    public RetResult<SectionLevelView> queryHistorySectionDataByTimeRange(String stationidc,
                                                                          @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime starttime,
                                                                          @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endtime) {
        try {
            SectionLevelView sectionLevelView = rtMonitoringDetailsService.queryHistorySectionDataByTimeRange(stationidc, starttime, endtime);
            return RetResponse.makeOKRsp(sectionLevelView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("查询历史同期对比的所有年份")
    @RequestMapping(value = "/queryHistoryComparisonYears", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 70)
    public RetResult<List<Integer>> queryHistoryComparisonYears() {
        try {
            List<Integer> retList = rtMonitoringDetailsService.queryHistoryComparisonYears();
            return RetResponse.makeOKRsp(retList);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按固定时段查询历史同期对比")
    @RequestMapping(value = "/queryhistorycomparisonbyperiod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）【前1月】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "year", value = "更多年份（多个用逗号隔开）【2022,2021,2020】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 80)
    public RetResult<DataCompareView> queryHistoryComparisonByPeriod(String stationidc, String period, String year) {
        try {
            DataCompareView dataCompareView = rtMonitoringDetailsService.queryHistoryComparisonByPeriodMore(stationidc, period, year);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按时间范围查询历史同期对比")
    @RequestMapping(value = "/queryhistorycomparisonbytimerange", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号盘【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "starttime", value = "开始时间【2021-11-07 20:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endtime", value = "结束时间【2022-11-25 20:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "year", value = "更多年份（多个以逗号隔开）【2022,2021,2020】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 90)
    public RetResult<DataCompareView> queryHistoryComparisonByTimeRange(String stationidc,
                                                                        @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime starttime,
                                                                        @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endtime,
                                                                        String year) {
        try {
            DataCompareView dataCompareView = rtMonitoringDetailsService.queryHistoryComparisonByTimeRangeMore(stationidc, starttime, endtime, year);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按固定时段查询多站对比")
    @RequestMapping(value = "/querystationcomparisonbyperiod", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）", required = true, dataTypeClass = String.class)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号（多个,隔开）【S1001，S1002】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）【前1月】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 100)
    public RetResult<DataCompareView> queryStationComparisonByPeriod(String stationidc, String period) {
        try {
            DataCompareView dataCompareView = rtMonitoringDetailsService.queryStationComparisonByPeriod(stationidc, period);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按时间范围查询多站对比")
    @RequestMapping(value = "/querystationcomparisonbytimerange", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号（多个,隔开）【S1001，S1002】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "starttime", value = "开始时间【2021-11-07 20:00:00】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endtime", value = "结束时间【2022-11-25 20:00:00】", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 110)
    public RetResult<DataCompareView> queryStationComparisonByTimeRange(String stationidc,
                                                                        @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime starttime,
                                                                        @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endtime) {
        try {
            DataCompareView dataCompareView = rtMonitoringDetailsService.queryStationComparisonByTimeRange(stationidc, starttime, endtime);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按年查询历史值对比")
    @RequestMapping(value = "/queryhistorycomparisonbyyear", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "currentyear", value = "开始年 yyyy【2022】", required = true, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "comparisonyear", value = "对比年  yyyy【2021】", required = true, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "querytype", value = "查询类型（确认/距平）【确认】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 120)
    public RetResult<DataHisValueCompareView> queryHistoryComparisonByYear(String stationidc, Integer currentyear, Integer comparisonyear, String querytype) {
        try {
            DataHisValueCompareView dataCompareView = rtMonitoringDetailsService.queryHistoryComparisonByYear(stationidc, currentyear, comparisonyear, querytype);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按月查询历史值对比")
    @RequestMapping(value = "/queryhistorycomparisonbymonth", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "currentmonth", value = "开始月 yyyy-MM【2022-05】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "comparisonmonth", value = "对比月 yyyy-MM【2021-05】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "querytype", value = "查询类型（确认/距平）【查询】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 130)
    public RetResult<DataHisValueCompareView> queryHistoryComparisonByMonth(String stationidc, String currentmonth, String comparisonmonth, String querytype) {
        try {
            DataHisValueCompareView dataCompareView = rtMonitoringDetailsService.queryHistoryComparisonByMonth(stationidc, currentmonth, comparisonmonth, querytype);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按日查询历史值对比")
    @RequestMapping(value = "/queryhistorycomparisonbyday", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "currentday", value = "开始日 yyyy-MM-dd【2022-11-08】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "comparisonday", value = "对比日 yyyy-MM-dd【2021-11-08】", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "querytype", value = "查询类型（确认/距平）【确认】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 140)
    public RetResult<DataHisValueCompareView> queryHistoryComparisonByDay(String stationidc,
                                                                          @DateTimeFormat(pattern = Constant.PATTERN_DATE) LocalDate currentday,
                                                                          @DateTimeFormat(pattern = Constant.PATTERN_DATE) LocalDate comparisonday,
                                                                          String querytype) {
        try {
            DataHisValueCompareView dataCompareView = rtMonitoringDetailsService.queryHistoryComparisonByDay(stationidc, currentday, comparisonday, querytype);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("查询相似年对比")
    @RequestMapping(value = "/queryhistorysimilaritycomparison", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "stationidc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 160)
    public RetResult<DataSimilarView> queryHistorySimilarityComparison(String stationidc) {
        try {
            DataSimilarView dataCompareView = rtMonitoringDetailsService.queryHistorySimilarityComparison(stationidc);
            return RetResponse.makeOKRsp(dataCompareView);
        } catch (Exception e) {
            log.error("查询时服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, "查询时服务器出现错误！");
        }
    }

    @ApiOperation("按条件分页查询报汛特征值")
    @RequestMapping(value = "/queryMonthCharacteristicValue", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "startTime", value = "开始年月 yyyy-MM【2022-05】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "endTime", value = "结束年月 yyyy-MM【2022-01】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 170)
    public RetResult<HydroMonthCharacterValueView> queryMonthCharacteristicValue(String stationIdc, String startTime, String endTime, BasePageQuery query) {
        try {
            HydroMonthCharacterValueView view = rtMonitoringDetailsService.queryMonthCharacteristicValue(stationIdc, startTime, endTime, query);
            return RetResponse.makeOKRsp(view);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按条件分页查询报汛特征值的历史对比")
    @RequestMapping(value = "/queryMonthCharacteristicValueCompare", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号【S1001】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "startTime", value = "开始年月 yyyy-MM【2022-01】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "endTime", value = "结束年月 yyyy-MM【2022-08】", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "dataType", value = "数据类型（最大/最小/平均）【最大】", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 180)
    public RetResult<HydroMonthCharacterValueCompareView> queryMonthCharacteristicValueCompare(String stationIdc, String startTime, String endTime, String dataType, BasePageQuery query) {
        try {
            HydroMonthCharacterValueCompareView view = rtMonitoringDetailsService.queryMonthCharacteristicValueCompare(stationIdc, startTime, endTime, dataType, query);
            return RetResponse.makeOKRsp(view);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }


    @ApiOperation("多条件查询特征值排序信息")
    @RequestMapping(value = "/queryCharacteristicValueOrder", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号（S1001）", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "对比周期（年/月/日）", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "datatype", value = "数据类型（最高水位/最低水位/平均水位/最高流量/最低流量/平均流量）", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "startTime", value = "开始时间(年:2021/月:2021-01/2021-01-01)", required = true, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间(年:2022/月:2022-12/2022-12-30)", required = true, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "orderType", value = "排序方式（时间排序/数据排序）", required = true, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "querytype", value = "查询类型（确认/距平）", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 190)
    public RetResult<CharacteristicValueOrderView> queryCharacteristicValueOrder(String stationIdc, String datatype, String period, String startTime, String endTime, String orderType, String querytype) {
        try {
            CharacteristicValueOrderView ret = rtMonitoringDetailsService.queryCharacteristicValueOrder(stationIdc, datatype, period, startTime, endTime, orderType, querytype);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按条件查询径流组成信息")
    @RequestMapping(value = "/queryHydroComposition", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号（S1001）", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "mid", value = "模型ID(1)", required = true, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "dataTime", value = "起报时间(2022-05-01 12:00:00)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "validTime", value = "预报时效（3）", required = true, dataTypeClass = Integer.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 220)
    public RetResult<HydroCompositionView> queryHydroComposition(String stationIdc, Integer mid, @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime dataTime, Integer validTime) {
        try {
            HydroCompositionView ret = rtMonitoringDetailsService.queryHydroCompositionByPeriod(stationIdc, mid, dataTime, validTime);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照固定时段查询雨量过程信息")
    @RequestMapping(value = "/queryRainfallProcessListByPeriod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号(S1001)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 250)
    public RetResult<RainfallProcessView> queryRainfallProcessListByPeriod(String stationIdc, String period) {
        try {
            RainfallProcessView ret = rtMonitoringDetailsService.queryRainfallProcessListByPeriod(stationIdc, period);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照时间范围查询雨量过程信息")
    @RequestMapping(value = "/queryRainfallProcessListByTimeRange", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号(S1001)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "stTime", value = "开始时间(2022-05-01 11:00:00)", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间(2021-05-01 23:00:00)", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 260)
    public RetResult<RainfallProcessView> queryRainfallProcessListByTimeRange(String stationIdc,
                                                                              @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                                              @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime) {
        try {
            RainfallProcessView ret = rtMonitoringDetailsService.queryRainfallProcessListByTimeRange(stationIdc, stTime, endTime);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照固定时段和流域查询流域站点面雨量")
    @RequestMapping(value = "/queryWatershedStationArealRainfallListByPeriod", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stationIdc", value = "站点编号（S1001）", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "period", value = "固定时段（前12小时、前24小时、前3天、前1周、前1月）", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "bsidList", value = "流域id字符串列表（用英文逗号分隔，不填则表示所有流域）", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 270)
    public RetResult<WatershedStationArealRainfallView> queryWatershedStationArealRainfallListByPeriod(String period, String bsidList) {
        try {
            WatershedStationArealRainfallView ret = rtMonitoringDetailsService.queryWatershedStationArealRainfallListByPeriod(period, bsidList);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照时间范围和流域查询流域站点面雨量")
    @RequestMapping(value = "/queryWatershedStationArealRainfallList", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime", value = "开始时间(2021-05-01 00:00:00)", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间(2022-05-01 23:30:00)", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "bsidList", value = "流域id字符串列表（用英文逗号分隔，不填则表示所有流域,查询流域列表接口在公共功能模块）", required = false, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 280)
    public RetResult<WatershedStationArealRainfallView> queryWatershedStationArealRainfallList(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime stTime,
                                                                                               @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime,
                                                                                               String bsidList) {
        try {
            WatershedStationArealRainfallView ret = rtMonitoringDetailsService.queryWatershedStationArealRainfallList(stTime, endTime, bsidList);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照时间和流域查询气象站点实况数据")
    @RequestMapping(value = "/queryMeteStationDataByTimeAndBsid", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dataTime", value = "数据时间（）", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "bsid", value = "流域id(1)", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 290)
    public RetResult<PageVo<MeteStationData>> queryMeteStationDataByTimeAndBsid(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime time, String bsid) {
        try {
            PageVo<MeteStationData> ret = rtMonitoringDetailsService.queryMeteStationDataByTimeAndBsid(time, bsid);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("按照时间和流域查询气象站点预报数据")
    @RequestMapping(value = "/queryMeteStationForecastDataByTimeAndBsid", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dataTime", value = "起报时间 yyyy-MM-dd HH:mm:ss", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "validTime", value = "预报时效", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "bsid", value = "流域id(1)", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "type", value = "类型（实况/预报）", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 290)
    public RetResult<PageVo<MeteStationForcastingData>> queryMeteStationForecastDataByTimeAndBsid(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime dataTime, Integer validTime
            , String bsid, String type) {
        try {
            PageVo<MeteStationForcastingData> ret = rtMonitoringDetailsService.queryMeteStationForecastDataByTimeAndBsid(dataTime, validTime, bsid, type);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("查询流域气象预报")
    @RequestMapping(value = "/queryWatershedMeteForecastList", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dataTime", value = "起报时间(2022-11-11 08:00:00)", required = true, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "validTime", value = "预报时效(3)", required = true, dataTypeClass = LocalDateTime.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 300)
    public RetResult<WatershedMeteForecastView> queryWatershedMeteForecastList(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime dataTime, Integer validTime) {
        try {
            WatershedMeteForecastView ret = rtMonitoringDetailsService.queryWatershedMeteForecastList(dataTime, validTime);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    @ApiOperation("查询测站信息")
    @RequestMapping(value = "/queryHydroStationInfo", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "stationIdc", value = "站点编号(S1001)", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @ApiOperationSupport(order = 310)
    public RetResult<HydroStationInfo> queryHydroStationInfo(String stationIdc) {
        try {
            HydroStationInfo ret = rtMonitoringDetailsService.queryHydroStationInfo(stationIdc);
            return RetResponse.makeOKRsp(ret);
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }
}
