package com.cuit.basin.base.biz.controller;

import com.cuit.basin.base.biz.dao.TbSysDatauserMapper;
import com.cuit.basin.base.biz.pojo.QueryPersonalCenterInfo;
import com.cuit.basin.base.biz.pojo.SysLogPojo;
import com.cuit.basin.base.biz.pojo.UpdatePersonalCenterInfo;
import com.cuit.basin.base.biz.pojo.UserDto;
import com.cuit.basin.base.biz.query.SysLogQuery;
import com.cuit.basin.base.biz.query.UserQuery;
import com.cuit.basin.base.biz.service.UserManagerService;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.cuit.basin.framework.common.utils.other.RedisUtil;
import com.cuit.basin.framework.common.utils.other.RequestUtil;
import com.cuit.basin.framework.log.annotation.AopLog;
import com.cuit.basin.framework.log.aop.OperationEnum;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @author: llw
 * @date: 2022/12/3 19:23
 * @description:
 */
@Api(tags = "个人管理中心")
@Slf4j
@RestController
public class UserManagerController {
    @Autowired
    UserManagerService userManagerService;


    private RedisUtil redisUtil = new RedisUtil();

    @Autowired
    TbSysDatauserMapper tbSysDatauserMapper;

    /*
     * @author: llw
     * @date: 2022/12/1 17:53
     * @param null
     * @return: sysLogPojoList
     * @description: 分页查询所有日志
     */
    @AopLog(module = "5", operation = "分页查询所有日志", opType = OperationEnum.OPERATION_QUERY)
    @ApiOperation("07 分页查询所有日志")
    @RequestMapping(value = "/queryAllSysLog", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Map> queryAllSysLog() {
        Map refMap = null;
        try {
            refMap = userManagerService.queryAllSysLog();
            return RetResponse.makeOKRsp(refMap);
        } catch (ErrorMessage error) {
            log.error("查询数据时出错" + error.getMessage());
            return RetResponse.makeErrRsp("分页查询数据时出错!");
        } catch (Exception e) {
            log.error("服务器错误！" + "/n");
            e.printStackTrace();
            return RetResponse.makeErrRsp("查询数据时服务器出错!");
        }
    }

    /*
     * @author: llw
     * @date: 2022/12/1 17:53
     * @param null
     * @return: map
     * @description: 按条件分页查询日志
     */
    @AopLog(module = "5", operation = "按条件分页查询所有日志", opType = OperationEnum.OPERATION_QUERY)
    @ApiOperation("08 按条件分页查询日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "stTime", value = "开始时间", required = false, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间", required = false, dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "ip", value = "访问ip", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "module", value = "模块类型", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "opType", value = "操作类型  0-增 1-删 2-改 3-查 4-审核", required = false, dataTypeClass = Integer.class)
    })
    @RequestMapping(value = "/querySysLogByParams", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Map> querySysLogByParams(SysLogQuery sysLogQuery) {
        Map refMap = null;
        try {
            refMap = userManagerService.querySysLogByParams(sysLogQuery);
            return RetResponse.makeOKRsp(refMap);
        } catch (ErrorMessage error) {
            log.error("查询数据时出错" + error.getMessage());
            return RetResponse.makeErrRsp("分页查询数据时出错!");
        } catch (Exception e) {
            log.error("服务器错误！" + "/n");
            e.printStackTrace();
            return RetResponse.makeErrRsp("查询数据时服务器出错!");
        }
    }


    /*
     * @author: llw
     * @date: 2022/12/4 16:19
     * @param ids
     * @return:
     * @description: 批量删除日志
     */
    @AopLog(module = "5", operation = "批量删除日志", opType = OperationEnum.OPERATION_DELETE)
    @ApiOperation("09 批量删除日志")
    @RequestMapping(value = "/batchDeleteSysLogByQuery", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "ids", value = "日志id字符串，多个以英文逗号分开", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<Boolean> batchDeleteSysLogByQuery(String ids) {
        Boolean isDelect;
        try {
            isDelect = userManagerService.batchDeleteSysLogByQuery(ids);
            return RetResponse.makeOKRsp(isDelect);
        } catch (ParamsException p) {
            log.error(p.getMsg());
            return RetResponse.makeErrRsp("删除时参数异常：" + p.getMsg());
        } catch (Exception e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp("服务器异常......");
        }
    }


    /*
     * @author: llw
     * @date: 2022/12/4 17:56
     * @param null
     * @return:
     * @description: 多条件查询系统用户列表
     */
    @AopLog(module = "5", operation = "多条件查询系统用户列表", opType = OperationEnum.OPERATION_QUERY)
    @ApiOperation("10 多条件查询系统用户列表")
    @RequestMapping(value = "/querySysUserByParam", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name", value = "用户姓名", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "dptId", value = "部门或单位", required = false, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "positionId", value = "所任职务", required = false, dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "permissionId", value = "权限等级", required = false, dataTypeClass = Integer.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<Map> querySysUserByParam(UserQuery userQuery) {
        Map refMap = null;
        try {
            refMap = userManagerService.querySysUserByParam(userQuery);
            return RetResponse.makeOKRsp(refMap);
        } catch (ParamsException p) {
            return RetResponse.makeErrRsp("多条件查询系统用户列表失败！");
        } catch (Exception e) {
            return RetResponse.makeErrRsp("服务器异常......");
        }
    }


    /*
     * @author: llw
     * @date: 2022/12/4 19:56
     * @param null
     * @return:
     * @description: 批量删除系统用户
     */
    @AopLog(module = "5", operation = "批量删除系统用户", opType = OperationEnum.OPERATION_DELETE)
    @ApiOperation("11 批量删除系统用户")
    @RequestMapping(value = "/batchDelectUserByIds", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "ids", value = "id字符串，多个以英文逗号分开", required = true, dataTypeClass = String.class)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<Boolean> batchDelectUserByIds(String ids) {
        boolean isdelect;
        try {
            isdelect = userManagerService.batchDelectUserByIds(ids);
            return RetResponse.makeOKRsp(isdelect);
        } catch (ParamsException p) {
            return RetResponse.makeErrRsp("删除时参数异常！");
        } catch (Exception e) {
            return RetResponse.makeErrRsp("服务器异常......");
        }
    }


    /*
     * @author: llw
     * @date: 2022/12/4 19:56
     * @param null
     * @return:
     * @description: 禁用启用系统用户
     */
    @AopLog(module = "5", operation = "启用/禁用用户", opType = OperationEnum.OPERATION_DELETE)
    @ApiOperation("12 启用/禁用用户")
    @RequestMapping(value = "/batchDelectUserByIds", method = {RequestMethod.PUT})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "请输入用户id", required = true, dataTypeClass = String.class),
            @ApiImplicitParam(name = "status", value = "启用：1，禁用：0", required = true, dataTypeClass = String.class)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<Boolean> enableOrDisable(String id, Integer status) {
        try {
            userManagerService.enableOrDisable(id, status);
            return RetResponse.makeOKRsp(true);
        } catch (ParamsException p) {
            return RetResponse.makeErrRsp("启用/禁用参数异常！");
        } catch (Exception e) {
            return RetResponse.makeErrRsp("服务器异常......");
        }
    }

    /*
     * @author: llw
     * @date: 2022/12/5 14:38
     * @param UserDto
     * @return: isSuccess
     * @description: 编辑用户信息
     */
    @AopLog(module = "5", operation = "编辑用户信息", opType = OperationEnum.OPERATION_MODIFY)
    @ApiOperation("13 编辑用户信息(通过用户id)")
    @RequestMapping(value = "/editUserByParams", method = {RequestMethod.PUT})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginname", value = "登录名", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "name", value = "姓名", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "dptName", value = "部门或单位名", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "positionName", value = "所任职务", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "permission_level_name", value = "权限等级", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "tel", value = "电话号码", required = false, dataTypeClass = String.class),
            @ApiImplicitParam(name = "password", value = "密码", required = false, dataTypeClass = String.class),
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<Boolean> editUserByParams(UserDto userDto, Integer id) {
        Boolean isSuccess;
        try {
            isSuccess = userManagerService.editUserByParams(userDto, id);
            return RetResponse.makeOKRsp(isSuccess);
        } catch (ErrorMessage error) {
            log.error("编辑用户信息时出错...." + error.getMessage());
            return RetResponse.makeErrRsp("编辑用户信息时，用户名已重复....");
        } catch (Exception e) {
            log.error("服务器出错！");
            e.printStackTrace();
            return RetResponse.makeErrRsp("编辑用户时服务器出错");
        }
    }

    @ApiOperation("01查询个人中心信息")
    @RequestMapping(value = "/queryPersonalCenterInfo", method = RequestMethod.GET)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @AopLog(module = "5", operation = "查询个人中心信息", opType = OperationEnum.OPERATION_QUERY)
    public RetResult<QueryPersonalCenterInfo> queryPersonalCenterInfo(HttpServletRequest request) {
        String account = (String) redisUtil.get(RequestUtil.releaseTokenFromRequest(request));
        try {
            QueryPersonalCenterInfo queryPersonalCenterInfo = userManagerService.queryPersonalCenterInfo(account);
            return RetResponse.makeOKRsp(queryPersonalCenterInfo);
        } catch (Exception e) {
            log.error("01查询个人中心信息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    @ApiOperation("02修改个人中心信息")
    @RequestMapping(value = "/updatePersonalCenterInfo", method = RequestMethod.PUT)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @AopLog(module = "5", operation = "修改个人中心信息", opType = OperationEnum.OPERATION_MODIFY)
    public RetResult<Boolean> updatePersonalCenterInfo(HttpServletRequest request, UpdatePersonalCenterInfo updatePersonalCenterInfo) {
        String account = (String) redisUtil.get(RequestUtil.releaseTokenFromRequest(request));
        try {
            userManagerService.updatePersonalCenterInfo(account, updatePersonalCenterInfo);
            return RetResponse.makeOKRsp(true);
        } catch (Exception e) {
            log.error("02修改个人中心信息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    @ApiOperation("03修改密码")
    @RequestMapping(value = "/updatePassword", method = RequestMethod.PUT)
    @ApiImplicitParam(name = "newPassword", value = "请输入新密码", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    @AopLog(module = "5", operation = "修改个人中心信息", opType = OperationEnum.OPERATION_MODIFY)
    public RetResult<Boolean> updatePassword(HttpServletRequest request, String newPassword) {
        String account = (String) redisUtil.get(RequestUtil.releaseTokenFromRequest(request));
        try {
            userManagerService.updatePassword(account, newPassword);
            return RetResponse.makeOKRsp(true);
        } catch (Exception e) {
            log.error("03修改密码信息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @param
     * @return
     * @description : 展示个人日志
     * @author tianjunbiao
     * @date 2022/12/4 20:08
     */

    @ApiOperation("04展示个人日志")
    @RequestMapping(value = "/queryPersonalLogs", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<List<SysLogPojo>> queryPersonalLogs(HttpServletRequest request) {
        String account = (String) redisUtil.get(RequestUtil.releaseTokenFromRequest(request));
        try {
            List<SysLogPojo> sysLogPojos = userManagerService.queryPersonalLogs(account);
            return RetResponse.makeOKRsp(sysLogPojos);
        } catch (ErrorMessage error) {
            log.error("展示个人日志时出错" + error.getMessage());
            return RetResponse.makeErrRsp("04展示个人日志时出错!");
        } catch (Exception e) {
            log.error("服务器错误！" + "/n");
            e.printStackTrace();
            return RetResponse.makeErrRsp("查询数据时服务器出错!");
        }

    }


    /**
     * @param
     * @return
     * @description : 按条件分页查询个人日志
     * @author tianjunbiao
     * @date 2022/12/4 20:08
     */

    @ApiOperation("05 按条件分页查询个人日志")
    @RequestMapping(value = "/querysysLogByParams", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")

    })

    public RetResult<Map> querySysLogByParams(HttpServletRequest request, SysLogQuery sysLogQuery) {
        String account = (String) redisUtil.get(RequestUtil.releaseTokenFromRequest(request));
        Map refMap = null;
        try {
            refMap = userManagerService.querySysLogByParams(account, sysLogQuery);
            return RetResponse.makeOKRsp(refMap);
        } catch (ErrorMessage error) {
            log.error("05按条件分页查询个人日志时出错" + error.getMessage());
            return RetResponse.makeErrRsp("05按条件分页查询个人日志时出错!");
        } catch (Exception e) {
            log.error("服务器错误！" + "/n");
            e.printStackTrace();
            return RetResponse.makeErrRsp("查询数据时服务器出错!");
        }
    }


    /**
     * @param
     * @return
     * @description : 查询所有用户信息
     * @author tianjunbiao
     * @date 2022/12/4 21:08
     */
    @ApiOperation("06 查询所有用户信息")
    @RequestMapping(value = "/queryAllUsersInfo", method = RequestMethod.GET)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })

    public RetResult<Map> queryAllUsersInfo() {
        Map refMap = null;
        try {
            refMap = userManagerService.queryAllUsersInfo();
            return RetResponse.makeOKRsp(refMap);
        } catch (Exception e) {
            log.error("01查询所有用户信息出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /*
     * @description: 下拉展示菜单接口(部门或单位)
     * @author: LLW
     * @date: 2022/12/8 16:13
     * @param:
     * @return: list：部门或单位列表
     **/

    @ApiOperation("14 下拉展示菜单接口(部门或单位)")
    @RequestMapping(value = "/DropdownMenuOfDptName", method = RequestMethod.GET)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> DropdownMenuOfDptName() {
        List<String> list;
        list = userManagerService.DropdownMenuOfDptName();
        return RetResponse.makeOKRsp(list);
    }

    /*
     * @description: 下拉展示菜单接口(所任职务)
     * @author: LLW
     * @date: 2022/12/8 17:01
     * @param:
     * @return: list：职务列表
     **/


    @ApiOperation("15 下拉展示菜单接口(部门或单位)")
    @RequestMapping(value = "/DropdownMenuOfPositionName", method = RequestMethod.GET)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> DropdownMenuOfPositionName() {
        List<String> list;
        list = userManagerService.DropdownMenuOfPositionName();
        return RetResponse.makeOKRsp(list);
    }

    /*
     * @description:
     * @author: LLW
     * @date: 2022/12/8 17:06
     * @param:
     * @return: list:权限 等级列表
     **/
    @ApiOperation("16 下拉展示菜单接口(权限等级)")
    @RequestMapping(value = "/DropdownMenuOfPermissonName", method = RequestMethod.GET)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> DropdownMenuOfPermissonName() {
        List<String> list;
        list = userManagerService.DropdownMenuOfPermissonName();
        return RetResponse.makeOKRsp(list);
    }
}
