package com.cuit.basin.base.biz.controller;


import com.cuit.basin.base.biz.entity.HydBiGlacierD;
import com.cuit.basin.base.biz.entity.TfReport;
import com.cuit.basin.base.biz.service.WaterResourceDBService;
import com.cuit.basin.base.biz.vo.GlacierDatasetView;
import com.cuit.basin.base.biz.vo.SnowDatasetView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.ret.PageVo;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.List;

@Api(tags = "09水资源库")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/waterresourcedb")
public class WaterResourceDBController {
    @Autowired
    private WaterResourceDBService waterResourceDBService;

    /**
     * @author: zwx
     * @date: 2022/11/30 16:16
     * @version: 1.0
     */
    @ApiOperation("条件分页查询冰川数据集")
    @RequestMapping(value = "/queryglacierdatasetbyparam", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startYear", value = "开始年份（2019）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "endYear", value = "结束年份（2021）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "keyword", value = "关键字", required = false, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<GlacierDatasetView>> queryGlacierDataSetByParam(Integer startYear, Integer endYear, String keyword, BasePageQuery pageQuery) {
        try {
            PageVo<GlacierDatasetView> pageVo = waterResourceDBService.queryGlacierDataSetByParam(startYear, endYear, keyword, pageQuery);
            return RetResponse.makeOKRsp(pageVo);
        } catch (Exception e) {
            log.error("条件分页查询冰川数据集服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ASUS
     * @date: 2022-12-6 12:09
     * @version: 1.0
     */
    @ApiOperation("查询数据集对应冰川数据")
    @RequestMapping(value = "/queryglacierfatabyglacierids", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "setId", value = "数据集Id（1/2）", required = true, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "glacierIds", value = "冰川ids(多个,隔开)(1,2,3)", required = false, dataTypeClass = String.class, paramType = "query")

    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<HydBiGlacierD>> queryGlacierDataByGlacierIds(Integer setId, String glacierIds) {
        try {
            List<HydBiGlacierD> hydBiGlacierDList = waterResourceDBService.queryGlacierDataByGlacierIds(setId, glacierIds);
            return RetResponse.makeOKRsp(hydBiGlacierDList);
        } catch (Exception e) {
            log.error("查询数据集对应冰川数据服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ASUS
     * @date: 2022-12-6 12:20
     * @version: 1.0
     */
    @ApiOperation("条件查询冰川的变化曲线")
    @RequestMapping(value = "/queryglacierdatachangecurvebyglacierid", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startYear", value = "开始年份（2019）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "endYear", value = "结束年份（2021）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "glacierId", value = "冰川id", required = true, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<HydBiGlacierD>> queryGlacierDataChangeCurveByGlacierId(String glacierId, Integer startYear, Integer endYear) {
        try {
            List<HydBiGlacierD> hydBiGlacierDList = waterResourceDBService.queryGlacierDataChangeCurveByGlacierId(glacierId, startYear, endYear);
            return RetResponse.makeOKRsp(hydBiGlacierDList);
        } catch (Exception e) {
            log.error("条件查询冰川的变化曲线服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }


    /**
     * @author: ASUS
     * @date: 2022-12-6 16:38
     * @version: 1.0
     */
    @ApiOperation("条件分页查询研究报告")
    @RequestMapping(value = "/queryreportbyparam", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "year", value = "年份", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "dataType", value = "数据类型（冰川/积雪）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "keyword", value = "关键字", required = false, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<TfReport>> queryReportByParam(Integer year, String dataType, String keyword, BasePageQuery pageQuery) {
        try {
            PageVo<TfReport> pageVo = waterResourceDBService.queryReportByParam(year, dataType, keyword, pageQuery);
            return RetResponse.makeOKRsp(pageVo);
        } catch (Exception e) {
            log.error("条件分页查询研究报告服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: ASUS
     * @date: 2022-12-6 18:20
     * @version: 1.0
     */
    @ApiOperation("下载研究报告")
    @RequestMapping(value = "/downloadreportbyid", method = {RequestMethod.POST})
    @ApiImplicitParam(name = "reportIds", value = "报告Id", required = true, dataTypeClass = String.class, paramType = "query")

    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> downloadReportByIds(String reportIds) {
        try {
            List<String> urls = waterResourceDBService.downloadReportByIds(reportIds);
            return RetResponse.makeOKRsp(urls);
        } catch (Exception e) {
            log.error("下载研究报告服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

    /**
     * @author: zwx
     * @date: 2022/11/30 16:16
     * @version: 1.0
     */
    @ApiOperation("条件分页查询积雪数据集")
    @RequestMapping(value = "/querysnowdatasettbyparam", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startTime", value = "开始时间（2015-01-01 00:00:00）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间（2015-02-01 00:00:00）", required = false, dataTypeClass = String.class, paramType = "query"),
            @ApiImplicitParam(name = "keyword", value = "关键字", required = false, dataTypeClass = String.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<SnowDatasetView>> querySnowDataSettByParam(@DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime startTime,
                                                                       @DateTimeFormat(pattern = Constant.PATTERN_DATETIME) LocalDateTime endTime, String keyword, BasePageQuery pageQuery) {
        try {
            PageVo<SnowDatasetView> pageVo = waterResourceDBService.querySnowDataSettByParam(startTime, endTime, keyword, pageQuery);
            return RetResponse.makeOKRsp(pageVo);
        } catch (Exception e) {
            log.error("条件分页查询积雪数据集服务器出现错误！" + e.getMessage());
            return RetResponse.makeRsp(500, e.getMessage());
        }
    }

}
