package com.cuit.basin.base.biz.controller.model;

import com.cuit.basin.base.biz.controller.model.vo.InitTaskParams;
import com.cuit.basin.base.biz.controller.model.vo.ModelTaskInfo;
import com.cuit.basin.base.biz.entity.ModelLog;
import com.cuit.basin.base.biz.service.model.ModelOperationTaskService;
import com.cuit.basin.base.biz.service.model.ModelProgressService;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import com.cuit.basin.framework.kafka.core.api.KafkaProducerApi;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Map;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.basin.base.biz.controller.model
 * @Author: HouPJ
 * @Date: 2022/12/23 9:27
 * @Description: TODO
 */
@Api(tags = "0000、模型操作")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/modeloperation")
public class ModelOperationTaskController {

    @Resource
    private ModelOperationTaskService modelOperationTaskService;
    @Resource
    private ModelProgressService modelProgressService;
    @Resource
    private KafkaProducerApi producerServer;

    @ApiOperation("初始化模型运行任务")
    @RequestMapping(value = "/initModelTask", method = {RequestMethod.GET})
    public RetResult<ModelTaskInfo> initModelTask(InitTaskParams initTaskParams) {
        return RetResponse.makeOKRsp(modelOperationTaskService.initModelTask(initTaskParams));
    }

    @ApiOperation("(主动)获取模型运行进度")
    @RequestMapping(value = "/obtainTaskProgress", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "modelTaskId", value = "模型运行任务id", required = true, dataTypeClass = String.class)
    public RetResult<ModelTaskInfo> obtainTaskProgress(String modelTaskId) {
        return RetResponse.makeOKRsp(modelOperationTaskService.obtainTaskProgress(modelTaskId));
    }

    @ApiOperation("获取运行中模型列表")
    @RequestMapping(value = "/obtainModelTask", method = {RequestMethod.GET})
    public RetResult<Map<String, ModelLog>> obtainModelTask() {
        return RetResponse.makeOKRsp(modelProgressService.obtainModelTask());
    }

    @ApiOperation("停止某一运行中模型")
    @RequestMapping(value = "/stopModelTask", method = {RequestMethod.GET})
    public RetResult<Boolean> stopModelTask(String taskId) {
        producerServer.sendMessageWithCallBack(KafkaTopic.TOPIC_MODEL_STOP,taskId);
        modelOperationTaskService.stopModelWithError(taskId);
        return RetResponse.makeOKRsp(true);
    }

}
