package com.cuit.basin.base.biz.controller.monitor;

import com.cuit.basin.base.biz.controller.monitor.vo.MonitorScreenInfo;
import com.cuit.basin.base.biz.controller.monitor.vo.ServerMonitorInfo;
import com.cuit.basin.base.biz.pojo.monitor.BaseAttributeReq;
import com.cuit.basin.base.biz.pojo.monitor.MonitorLogDisplayReq;
import com.cuit.basin.base.biz.service.monitor.RTMonitorScreenService;
import com.cuit.basin.base.websocket.server.WebSocketServer;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.RetResponse;
import com.cuit.basin.framework.common.ret.RetResult;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * @BelongsProject: sriversys
 * @BelongsPackage: com.cuit.basin.base.biz.controller
 * @Author: HouPJ
 * @Date: 2022/12/17 14:41
 * @Description: TODO
 */
@Api(tags = "000 实时监控")
@Slf4j
@RestController
@RequestMapping("/watershedsystem/rtmonitorscreen")
public class RTMonitorScreenController {

    @Resource
    private WebSocketServer webSocketServer;
    @Autowired
    private RTMonitorScreenService monitorScreenService;

    public RetResult<Integer> getHydrologicalEventListByTime() {
        try {
            return RetResponse.makeOKRsp(webSocketServer.getConcurrentClientCount());
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    /**
     * @author: hpj
     * @date: 2022/12/17 19:31
     * @version: 1.0
     */
    @ApiOperation("获取服务器基础监控信息")
    @RequestMapping(value = "/getServerMonitorInfo", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<ServerMonitorInfo> getServerMonitorInfo() {

        try {
            MonitorScreenInfo monitorScreenInfo = new MonitorScreenInfo();
            monitorScreenInfo.copyInfo();
            return RetResponse.makeOKRsp(ServerMonitorInfo.builder().cpu(monitorScreenInfo.getCpu())
                    .jvm(monitorScreenInfo.getJvm())
                    .mem(monitorScreenInfo.getMem())
                    .sys(monitorScreenInfo.getSys())
                    .sysFiles(monitorScreenInfo.getSysFiles()).build());
        } catch (ParamsException e) {
            log.error("参数错误！" + e.getMessage());
            return RetResponse.makeErrRsp("参数错误！" + e.getMessage());
        } catch (ErrorMessage e) {
            log.error(e.getMessage());
            return RetResponse.makeErrRsp(e.getMessage());
        } catch (Exception e) {
            log.error("服务器错误！" + e.getMessage());
            return RetResponse.makeErrRsp("服务器错误！");
        }
    }

    /**
     * @author: hpj
     * @date: 2022/12/17 19:31
     * @version: 1.0
     */
    @ApiOperation("获取服务器实时日志信息")
    @RequestMapping(value = "/getServerRTLog", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "moduleId", value = "模块id 7-模型管理 8-水文数据 9-气象数据 ", required = true, dataTypeClass = String.class)
    public RetResult<List<MonitorLogDisplayReq>> getServerRealTimeLog(Integer moduleId) {
        return RetResponse.makeOKRsp(monitorScreenService.obtainRTMonitorLogs(moduleId));
    }

    /**
     * @author: hpj
     * @date: 2022/12/17 19:31
     * @version: 1.0
     */
    @ApiOperation("获取服务器属性数据信息")
    @RequestMapping(value = "/getServerRTAttribute", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "moduleId", value = "模块id 7-模型管理 8-水文数据 9-气象数据 ", required = true, dataTypeClass = String.class)
    public RetResult<BaseAttributeReq> getServerRTAttribute(Integer moduleId) {
        return RetResponse.makeOKRsp(monitorScreenService.getServerRTAttribute(moduleId));
    }


}
