package com.cuit.basin.base.biz.controller.monitor.vo;

import com.cuit.basin.base.biz.controller.monitor.util.Arith;
import com.cuit.basin.base.biz.controller.monitor.util.DateUtils;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.lang.management.ManagementFactory;

/**
 * @BelongsProject: sriversys
 * @BelongsPackage: com.cuit.sriver.sriversys.server.websocket.vo
 * @Author: HouPJ
 * @Date: 2022/12/17 15:36
 * @Description: TODO
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel(value = "jvm基础信息", description = "")
public class MonitorJvmInfo {

    /**
     * 当前JVM占用的内存总数(M)
     */
    @ApiModelProperty(value = "当前JVM占用的内存总数(M)")
    private double total;
    /**
     * JVM最大可用内存总数(M)
     */
    @ApiModelProperty(value = "JVM最大可用内存总数(M)")
    private double max;
    /**
     * JVM空闲内存(M)
     */
    @ApiModelProperty(value = "JVM空闲内存(M)")
    private double free;
    /**
     * JDK版本
     */
    @ApiModelProperty(value = "JDK版本")
    private String version;
    /**
     * JDK路径
     */
    @ApiModelProperty(value = "JDK路径")
    private String home;

    public double getTotal() {
        return Arith.div(total, (1024 * 1024), 2);
    }

    public void setTotal(double total) {
        this.total = total;
    }

    public double getMax() {
        return Arith.div(max, (1024 * 1024), 2);
    }

    public void setMax(double max) {
        this.max = max;
    }

    public double getFree() {
        return Arith.div(free, (1024 * 1024), 2);
    }

    public void setFree(double free) {
        this.free = free;
    }

    public double getUsed() {
        return Arith.div(total - free, (1024 * 1024), 2);
    }

    public double getUsage() {
        return Arith.mul(Arith.div(total - free, total, 4), 100);
    }

    public String getName() {
        return ManagementFactory.getRuntimeMXBean().getVmName();
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getHome() {
        return home;
    }

    public void setHome(String home) {
        this.home = home;
    }

    public String getStartTime() {
        return DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, DateUtils.getServerStartDate());
    }

    public String getRunTime() {
        return DateUtils.getDatePoor(DateUtils.getNowDate(), DateUtils.getServerStartDate());
    }
}
