package com.cuit.basin.base.biz.dao;

import com.cuit.basin.base.biz.entity.sys.AppidToKen;
import com.cuit.basin.base.biz.entity.sys.LoginUser;
import com.cuit.basin.base.biz.pojo.UserInfo;
import com.cuit.basin.base.biz.query.SysUserQuery;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;


@Mapper
public interface LoginUserMapper {
    /**
     * 根据登录名提取用户信息
     *
     * @param loginname 登录名
     * @return
     */
    LoginUser getUserByLoginName(String loginname);

    /**
     * 设置错误次数
     *
     * @param loginname 登录名
     * @return
     */
    int setLoginError(String loginname);

    /**
     * 清空错误次数
     *
     * @param loginname 登录名
     * @return
     */
    int clearLoginError(String loginname);

    /**
     * 设置用户密码
     *
     * @param loginname 用户名
     * @param password  密码
     * @return
     */
    int setUserPassword(String loginname, String password);

    /**
     * 删除用户
     *
     * @param loginname 用户名
     * @return 删除条数
     */
    int deleteUser(String loginname);


    /**
     * 设置禁用标志
     *
     * @param loginname 用户名
     * @return 影响条数
     */
    int setUserLock(String loginname);

    /**
     * 解除禁用标志
     *
     * @param loginname 用户名
     * @return 影响条数
     */
    int setUserUnLock(String loginname);

    /**
     * 新增用户
     *
     * @param loginUser 用户
     * @return 影响条数
     */
    int addLoginUser(LoginUser loginUser);

    /**
     * 更新用户
     *
     * @param loginUser
     * @return
     */
    int updateLoginUser(LoginUser loginUser);

    /**
     * 获取授权AppID
     *
     * @param appid 应用ID或用户
     * @return
     */
    AppidToKen getAppidToKen(String appid);

    /**
     * 根据登录名查找用户id
     *
     * @param account
     * @return
     */
    Integer queryUserIdByAccount(String account);

    /**
     * 获取所有用户
     *
     * @return
     */
    List<LoginUser> getAll();

    /**
     * 多条件查询用户列表信息
     *
     * @param query
     * @return
     */
    List<UserInfo> selectByParams(@Param("query") SysUserQuery query);

    /**
     * 根据职位名称查询部门id
     *
     * @param dptName
     * @return
     */
    Integer queryDptIdByName(String dptName);


    /**
     * 根据职位名称查询职位id
     *
     * @param positionName
     * @return
     */
    Integer queryPositionIdByName(String positionName);

    /**
     * 根据权限等级名称获取id
     *
     * @param permissionLevelName
     * @return
     */
    Integer queryPermissionIdByName(String permissionLevelName);

    /**
     * 根据id获取名称
     *
     * @param dptId
     * @return
     */
    String getDptId(Integer dptId);

    /**
     * 根据id获取名称
     *
     * @param permissionId
     * @return
     */
    String getPermissionName(Integer permissionId);

    /**
     * 根据id获取名称
     *
     * @param positionId
     * @return
     */
    String getPositionId(Integer positionId);

    /**
     * 根据账户查询用户权限等级
     *
     * @param account
     * @return
     */
    Integer queryUserLevelByAccount(String account);

    /**
     * 根据登录名查询牧民名称
     *
     * @param account
     * @return
     */
    String queryHerdsManByAccount(String account);


    /**
     * 根据账户查询lp_id
     *
     * @param account
     * @return
     */
    List<Long> queryLpIdByAccount(String account);

    /**
     * 根据牧民id获取牧民用户id
     *
     * @param herdsmanId
     * @return
     */
    Integer queryUserIdByHerdsManId(Long herdsmanId);

    String getUserAccountById(@Param("userId") Integer userId);
}
