package com.cuit.basin.base.biz.mq.hydrology;

import cn.hutool.core.util.ObjectUtil;
import com.cuit.basin.base.websocket.server.WebSocketServer;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.sriver.sriversys.mq.model
 * @Author: HouPJ
 * @Date: 2022/12/22 16:27
 * @Description: 水文数据相关消息消费者
 */
@Component
@Slf4j
public class HydrologyMQConsummer {

    @Resource
    private WebSocketServer webSocket;

    /**
     * 监听水文实时日志处理消息
     * @param message
     * @param ack
     */
    @KafkaListener(topics = {KafkaTopic.TOPIC_DATA_ETL_HYDRO}, groupId = "defaultGroup")
    public void listenModelLog(String message, Acknowledgment ack) {
        log.info("接收到主题为{}的消息，size:{}",KafkaTopic.TOPIC_DATA_ETL_HYDRO,message.length());
        if (ObjectUtil.isEmpty(message)) {
            return;
        }
        //利用websocket通知前端刷新页面
        log.info("消息为" + message + "开始利用websocket通知前端.....");
        try{
            webSocket.sendGroupInfo(message);
        }catch (Exception e) {
            log.error("websocket 消息发送失败！");
        }
        // 手动提交偏移量
        try{
            ack.acknowledge();
        }catch (Exception e){
            log.error("消费水文实时日志处理消息异常，errMsg:{}", e.getMessage(),e);
        }
    }
}
