package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.entity.HydroStationData;
import com.cuit.basin.base.biz.vo.HistoricalCasesDetailView;
import com.cuit.basin.base.biz.vo.HistoricalCasesView;
import com.cuit.basin.base.biz.vo.HydrologicalEventView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.ret.PageVo;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * 历史个例板块Service
 *
 * @author Xiao Yinrui
 */
@Service
public interface HistoricalCasesService {

    /**
     * 多条件查询水文事件列表信息（时间范围）
     *
     * @param type    类型
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param keyword 关键词
     * @param query   查询类
     * @return HydrologicalEventView
     */
    HydrologicalEventView getHydrologicalEventListByTime(String type, String stTime, String endTime, String keyword, BasePageQuery query);

    /**
     * 多条件查询水文事件列表信息（固定时段）
     *
     * @param type    类型
     * @param period  时间段
     * @param keyword 关键词
     * @param query   查询类
     * @return HydrologicalEventView
     */
    HydrologicalEventView getHydrologicalEventListByPeriod(String type, String period, String keyword, BasePageQuery query);

    /**
     * 新增历史个例（从水文事件新增）
     *
     * @return boolean
     */
    boolean addHistoryCaseFromEvent();

    /**
     * 删除水文事件
     *
     * @param eids 主键id列表
     * @return boolean
     */
    boolean deleteHydrologicalEvent(String eids);

    /**
     * 多条件查询历史个例信息（时间范围）
     *
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param keyword 关键词
     * @param query   查询类
     * @return HistoricalCasesView
     */
    HistoricalCasesView getHistoricalCasesByTime(String stTime, String endTime, String keyword, BasePageQuery query);

    /**
     * 多条件查询历史个例信息（固定时段）
     *
     * @param period  时间段
     * @param keyword 关键词
     * @param query   查询类
     * @return HistoricalCasesView
     */
    HistoricalCasesView getHistoricalCasesByPeriod(String period, String keyword, BasePageQuery query);

    /**
     * 查询历史个例详情信息
     *
     * @param cid   历史个例id
     * @param query 查询类
     * @return HistoricalCasesDetailView
     */
    HistoricalCasesDetailView getHistoricalCasesDetail(Integer cid, BasePageQuery query);

    /**
     * 新增历史个例信息
     *
     * @param name    历史个例名
     * @param desc    历史个例描述
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @return boolean
     */
    boolean addHistoricalCase(String name, String desc, LocalDateTime stTime, LocalDateTime endTime);

    /**
     * 按照时间范围查询水文站点数据
     *
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param query   查询类
     * @return PageVo<HydroStationData>
     */
    PageVo<HydroStationData> getHydroStationData(LocalDateTime stTime, LocalDateTime endTime, BasePageQuery query);

    /**
     * 批量删除历史个例
     *
     * @param cids 历史个例id列表
     * @return boolean
     */
    boolean deleteHistoricalCases(String cids);
}
