package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.pojo.*;
import com.cuit.basin.base.biz.entity.DicBulletinStutus;
import com.cuit.basin.base.biz.entity.DicBulletinType;
import com.cuit.basin.base.biz.entity.HydPiBulletinP;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.ret.PageVo;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 水情服务公报-sevice层
 */
@Service
public interface HydrologicalBulletinService {


    /**
     * 条件分页查询公报列表
     *
     * @param query 查询参数类
     * @return PageVo<HydPiBulletinPInfo>
     */
    PageVo<HydPiBulletinPInfo> queryGazetteListByParams(GazetteListQuery query);

    /**
     * 查询公报信息
     *
     * @param bId 公报id
     * @return HydPiBulletinP
     */
    HydPiBulletinP queryGazetteById(Integer bId);

    /**
     * 批量删除公报
     *
     * @param bIds 公报Id(多个,隔开)
     * @return Boolean
     */
    Boolean batchDeleteGazetteByIds(String bIds);

    /**
     * 保存公报信息
     *
     * @param params
     * @return String
     */
    String saveGazette(HydPiBulletinPParams params);



    /**
     * 预览公报
     *
     * @param param 条件
     * @return String
     */
    String produceGazette(ProduceGazetteParam param);

    /**
     * 获取公报雨情生成条件
     *
     * @param stTime 开始时间
     * @param endTime 结束时间
     * @param situation 情况
     * @param bsId 流域id
     * @param keywords 关键字
     * @return GazetteRainParamsView
     */
    GazetteRainParamsView getGazetteRainParamsView(LocalDateTime stTime, LocalDateTime endTime, String situation, Integer bsId, String keywords);

    /**
     * 获取公报水情生成条件
     *
     * @param stTime 开始时间
     * @param endTime 结束时间
     * @param situation 情况
     * @param bsId 流域id
     * @param keywords 关键字
     * @return GazetteRegimenParamsView
     */
    GazetteRegimenParamsView getGazetteRegimenParamsView(LocalDateTime stTime, LocalDateTime endTime, String situation, Integer bsId, String keywords);

    /**
     * 获取模型预报公报生成条件
     *
     * @param stTime 开始时间
     * @param endTime 结束时间
     * @param situation 情况
     * @param bsId 流域id
     * @param keywords 关键字
     * @return GazetteModeForecastParamsView
     */
    GazetteModeForecastParamsView getGazetteModeForecastParamsView(LocalDateTime stTime, LocalDateTime endTime, String situation, Integer bsId, String keywords);

    /**
     * 获取特征值字典
     *
     * @return EigenvalueView
     */
    EigenvalueView getDicEigenvalue();

    /**
     * 获取公报类型
     *
     * @return List<DicBulletinType>
     */
    List<DicBulletinType> getGazetteType();

    /**
     * 获取公报状态类型
     *
     * @return List<DicBulletinStutus>
     */
    List<DicBulletinStutus> getGazetteStutus();

    /**
     * 审核公报
     *
     * @param bId
     * @param stutusId
     * @return Boolean
     */
    Boolean auditGazette(String bId, Integer stutusId);

    /**
     * 获取流域列表
     *
     * @return
     */
    List<DicBSData> getDicBSDataList();


    /**
     * 批量下载
     *
     * @param bIds List<String>
     * @return List<String>
     */
    List<String> batchDownLoadGazetteByIds(String bIds);
}
