package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.vo.HydroForeCurrDataView;
import com.cuit.basin.base.biz.vo.HydroForeCurrReferView;
import com.cuit.basin.base.biz.vo.ManualCorrectionCurveView;
import com.cuit.basin.base.biz.vo.ModuleFittedDataView;
import org.springframework.stereotype.Service;

@Service
public interface HydrologicalCorrectionService {
    /**
     * 查询模型参考曲线
     *
     * @param stationNo  站号
     * @param validTime  预报时效
     * @param timePeriod
     * @return HydroForeCurrReferView
     */
    HydroForeCurrReferView queryForecastReferenceCurve(String stationNo, Integer validTime, Integer timePeriod);

    /**
     * 预报订正曲线初始化
     *
     * @param stationNo   站号
     * @param correctType 订正类型(快速订正/公式订正)
     * @param correctStr  订正参数(最大、最小、平均/公式字符)
     * @return HydroForeCurrDataView
     */
    HydroForeCurrDataView queryHydrologicalCorrectionByParam(String stationNo, String correctType, String correctStr);

    /**
     * 保存手动订正预报曲线
     *
     * @param view 订正数据
     * @return Boolean
     */
    Boolean saveManualCorrectionForecastCurve(ManualCorrectionCurveView view);

    /**
     * 查询模型拟合曲线
     *
     * @param stationNo  站号
     * @param timePeriod 历史拟合周期(h)
     * @return ModuleFittedDataView
     */
    ModuleFittedDataView queryModuleFittedDataCurve(String stationNo, Integer timePeriod);
}
