package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.entity.sys.LoginUser;
import com.cuit.basin.base.biz.entity.sys.User;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public interface LoginUserService {
    /**
     * 获取app id的token
     *
     * @param appid
     * @return
     */
    String getAppIdToken(String appid);

    /**
     * 根据账号和appid 获取token
     *
     * @param appid
     * @param account
     * @return
     */
    String getAppIdToken(String appid, String account);

    /**
     * 获取用户登录表的所有信息
     *
     * @return
     */
    List<LoginUser> getAll();

    /**
     * 校验用户 返回用户类
     *
     * @param longinname
     * @param password
     * @return
     */
    User checkUser(String longinname, String password);

    /**
     * 重置用户密码
     *
     * @param loginname
     * @param password
     * @return
     */
    boolean setUserPassword(String loginname, String password);

    /**
     * 删除用户
     *
     * @param loginname
     * @return
     */
    boolean deleteUser(String loginname);

    /**
     * 锁定用户
     *
     * @param loginname
     * @param lockflag
     * @return
     */
    boolean lockUser(String loginname, boolean lockflag);

    /**
     * 添加用户
     *
     * @param user
     * @return
     */
    boolean addUser(User user);

    /**
     * 判断token是否有效
     *
     * @param token
     * @return
     */
    boolean isValidToken(String token);

    /**
     * 更新用户信息
     *
     * @param user
     * @return
     */
    boolean updateUser(User user);

    /**
     * 退出登录
     *
     * @param token
     * @return
     */
    boolean loginOut(String token);
}
