package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.entity.ModelParam;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.base.BasePageQuery;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

/**
 * 模型管理 Service
 *
 * @author Xiao Yinrui
 */
@Service
public interface ModelManagerService {
    /**
     * 查询模型列表信息
     *
     * @param status 运行状态
     * @return ModelDicView
     */
    ModelDicView getModelDicList(Integer status);

    /**
     * 新增模型基础信息
     *
     * @param mname  模型名
     * @param mdes   模型描述
     * @param path   模型路径
     * @param dfproc 数据预报处理
     * @param mfdata 模型前置处理
     * @param mprog 模型程序
     * @param mfprog 模型前置程序
     * @param daproc 数据后处理
     * @return boolean
     */
    boolean addModelInfo(String mname, String mdes, String path, String dfproc, String mfdata, String mprog, String mfprog, String daproc);

    /**
     * 编辑模型基础信息
     *
     * @param mid    模型id
     * @param mname  模型名
     * @param mdes   模型描述
     * @param path   模型路径
     * @param dfproc 数据预报处理
     * @param mfdata 模型前置处理
     * @param mprog 模型程序
     * @param mfprog 模型前置程序
     * @param daproc 数据后处理
     * @return boolean
     */
    boolean editModelInfo(Integer mid, String mname, String mdes, String path, String dfproc, String mfdata, String mprog, String mfprog, String daproc);

    /**
     * 批量删除模型信息
     *
     * @param mids 模型id字符串
     * @return boolean
     */
    boolean deleteModelInfo(String mids);

    /**
     * 手动启动某一个模型
     *
     * @param mid 模型id
     * @return boolean
     */
    boolean startModelByMid(Integer mid);

    /**
     * 停止某一个模型
     *
     * @param mid 模型uid
     * @return boolean
     */
    boolean stopModelByMid(Integer mid);

    /**
     * 查询模型定时运行参数
     *
     * @param mid 模型id
     * @return CronHourMinView
     */
    CronHourMinView getModelScheduler(Integer mid);

    /**
     * 设置模型定时运行参数
     *
     * @param cronV 定时类
     * @return String
     */
    String scheduleModel(CronHourMinView cronV);

    /**
     * 多条件查询模型运行日志
     *
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param mids    模型id字符串列表
     * @param method  运行方式
     * @param query   查询类
     * @return ModelLogView
     */
    ModelLogView getModelLogList(String stTime, String endTime, String mids, String method, BasePageQuery query);

    /**
     * 批量删除模型运行日志
     *
     * @param ids id字符串列表
     * @return boolean
     */
    boolean deleteModelLogs(String ids);

    /**
     * 多条件查询模型参数和校准信息
     *
     * @param mid   模型id
     * @param query 查询类
     * @return ModelParamView
     */
    ModelParamView getModelParamList(Integer mid, BasePageQuery query) throws NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException;

    /**
     * 模型参数运行校准（同时编辑某一个模型的参数）
     *
     * @param mParam 模型参数列表
     * @return boolean
     */
    boolean editModelParamAndCalibration(List<ModelParam> mParam);

    /**
     * 批量删除模型参数和校准信息
     *
     * @param ids id字符串列表
     * @return boolean
     */
    boolean deleteModelParam(String ids);
}
