package com.cuit.basin.base.biz.service;


import com.baomidou.mybatisplus.extension.service.IService;
import com.cuit.basin.base.biz.entity.SysLog;
import com.cuit.basin.base.biz.pojo.QueryPersonalCenterInfo;
import com.cuit.basin.base.biz.pojo.SysLogPojo;
import com.cuit.basin.base.biz.pojo.UpdatePersonalCenterInfo;
import com.cuit.basin.base.biz.pojo.UserDto;
import com.cuit.basin.base.biz.query.SysLogQuery;
import com.cuit.basin.base.biz.query.UserQuery;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;


@Service
public interface UserManagerService extends IService<SysLog> {


    /*
     * @author: llw
     * @date: 2022/12/2 15:15
     * @param null
     * @return: SysLogPojo
     * @description: 分页查询所有日志
     */
    Map queryAllSysLog();


    /*
     * @author: llw
     * @date: 2022/12/2 15:20
     * @param null
     * @return:
     * @description: 按条件分页查询日志
     */
    Map querySysLogByParams(SysLogQuery sysLogQuery);

    /*
     * @author: llw
     * @date: 2022/12/2 18:59
     * @param null
     * @return:
     * @description: 批量删除日志
     */
    Boolean batchDeleteSysLogByQuery(String ids);


    /*
     * @author: llw
     * @date: 2022/12/4 18:44
     * @param null
     * @return:
     * @description: 多条件查询用户信息列表
     */
    Map querySysUserByParam(UserQuery userQuery);


    /*
     * @author: llw
     * @date: 2022/12/4 19:53
     * @param null
     * @return:
     * @description: 批量删除用户
     */
    boolean batchDelectUserByIds(String ids);

    /*
     /*
     * @author: llw
     * @date: 2022/12/4 22:07
      * @param id
     * @param status
     * @return: void
     * @description:启用/禁用用户
     */
    void enableOrDisable(String id, Integer status);



    /*
     * @author: llw
     * @date: 2022/12/5 14:38
     * @param UserDto
     * @return: isSuccess
     * @description: 编辑用户信息
     */

    Boolean editUserByParams(UserDto userDto, Integer id);


    /**
     * @param
     * @return
     * @description : 查询个人中心信息
     * @author tianjunbiao
     * @date 2022/12/4 15:15
     */
    QueryPersonalCenterInfo queryPersonalCenterInfo(String account);

    /**
     * @param
     * @return
     * @description : 修改个人中心信息
     * @author tianjunbiao
     * @date 2022/12/4 16:03
     */
    void updatePersonalCenterInfo(String account, UpdatePersonalCenterInfo updatePersonalCenterInfo);

    /**
     * @param
     * @return
     * @description : 修改密码
     * @author tianjunbiao
     * @date 2022/12/4 17:19
     */
    void updatePassword(String account, String newPassword);

    /**
     * @param
     * @return
     * @description : 按条件分页查询个人日志
     * @author tianjunbiao
     * @date 2022/12/4 20:11
     */
    Map querySysLogByParams(String account, SysLogQuery sysLogQuery);

    /**
     * @param
     * @return
     * @description : 展示个人日志
     * @author tianjunbiao
     * @date 2022/12/4 20:36
     */
    List<SysLogPojo> queryPersonalLogs(String account);

    /**
     * @param
     * @return
     * @description : 查询所有用户信息
     * @author tianjunbiao
     * @date 2022/12/3 20:40
     */
    Map queryAllUsersInfo();


    /**
     * @param
     * @return
     * @description : 新增用户信息
     * @author tianjunbiao
     * @date 2022/12/5 11:30
     */
//    void addUser(UserDto userDto);

    /*
     * @description: 下拉展示菜单接口(部门或单位)
     * @author: LLW
     * @date: 2022/12/8 16:13
     * @param:
     * @return:
     **/

    List<String> DropdownMenuOfDptName();

    /*
     * @description: 下拉展示菜单接口(所任职务)
     * @author: LLW
     * @date: 2022/12/8 17:01
     * @param:
     * @return: list：职务列表
     **/


    List<String> DropdownMenuOfPositionName();


    /*
     * @description: 下拉展示菜单接口(权限等级)
     * @author: LLW
     * @date: 2022/12/8 17:01
     * @param:
     * @return: list：权限等级列表
     **/
    List<String> DropdownMenuOfPermissonName();
}
