package com.cuit.basin.base.biz.service;

import com.cuit.basin.base.biz.entity.HydBiGlacierD;
import com.cuit.basin.base.biz.entity.TfReport;
import com.cuit.basin.base.biz.vo.GlacierDatasetView;
import com.cuit.basin.base.biz.vo.SnowDatasetView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.ret.PageVo;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

@Service
public interface WaterResourceDBService {
    /**
     * 条件分页查询冰川数据集
     *
     * @param startYear 开始年份
     * @param endYear   结束年份
     * @param keyword   关键字
     * @param pageQuery 分页参数
     * @return PageVo<GlacierDatasetView>
     */
    PageVo<GlacierDatasetView> queryGlacierDataSetByParam(Integer startYear, Integer endYear, String keyword, BasePageQuery pageQuery);

    /**
     * 查询数据集对应冰川数据
     *
     * @param setId      数据集Id
     * @param glacierIds 冰川Id
     * @return List<HydBiGlacierD>
     */
    List<HydBiGlacierD> queryGlacierDataByGlacierIds(Integer setId, String glacierIds);

    /**
     * 条件查询冰川的变化曲线
     *
     * @param glacierId 冰川Id
     * @param startYear 开始年份
     * @param endYear   结束年份
     * @return List<HydBiGlacierD>
     */
    List<HydBiGlacierD> queryGlacierDataChangeCurveByGlacierId(String glacierId, Integer startYear, Integer endYear);

    /**
     * 条件分页查询研究报告
     *
     * @param year      年份
     * @param dataType  数据类型
     * @param keyword   关键字
     * @param pageQuery 分页参数
     * @return PageVo<TfReport>
     */
    PageVo<TfReport> queryReportByParam(Integer year, String dataType, String keyword, BasePageQuery pageQuery);

    /**
     * 下载研究报告(批量)
     *
     * @param reportIds 报告ids
     * @return List<String>
     */
    List<String> downloadReportByIds(String reportIds);

    /**
     * 条件分页查询积雪数据集
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param keyword   关键字
     * @param pageQuery 分页参数
     * @return PageVo<SnowDatasetView>
     */
    PageVo<SnowDatasetView> querySnowDataSettByParam(LocalDateTime startTime, LocalDateTime endTime, String keyword, BasePageQuery pageQuery);
}
