package com.cuit.basin.base.biz.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cuit.basin.base.biz.service.CommonFunctionService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.base.biz.dao.*;
import com.cuit.basin.base.biz.entity.HydroStationInfo;
import com.cuit.basin.base.biz.entity.MessageInfo;
import com.cuit.basin.base.biz.entity.ModelDic;
import com.cuit.basin.base.biz.entity.WatershedData;
import com.cuit.basin.base.biz.pojo.HydroStationDataInfo;
import com.cuit.basin.base.biz.pojo.WatershedInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class CommonFunctionServiceImpl implements CommonFunctionService {
    @Resource
    MessageInfoMapper messageInfoMapper;
    @Resource
    HydroStationInfoMapper hydroStationInfoMapper;
    @Resource
    HydroStationDataMapper hydroStationDataMapper;
    @Resource
    WatershedDataMapper watershedDataMapper;
    @Resource
    ModelDicMapper modelDicMapper;

    @Override
    public MessageInfoView queryMessageInfo(Integer typeid) {
        LambdaQueryWrapper<MessageInfo> queryWrapper = new LambdaQueryWrapper<>();
        if (typeid == 0) {
            queryWrapper.orderByDesc(MessageInfo::getLymdh).last("limit 10");
        } else {
            queryWrapper.eq(MessageInfo::getType, typeid).orderByDesc(MessageInfo::getLymdh).last("limit 10");
        }
        List<MessageInfo> messageInfos = messageInfoMapper.selectList(queryWrapper);
        return MessageInfoView.builder()
                .messageInfoList(messageInfos)
                .build();
    }

    @Override
    public HydroStationInfoView queryHydroStationList(String key) {
        LambdaQueryWrapper<HydroStationInfo> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isEmpty(key)) {
            queryWrapper.isNotNull(HydroStationInfo::getStcd);
        } else {
            queryWrapper.likeRight(HydroStationInfo::getStnm, key).or().likeRight(HydroStationInfo::getStcd, key);
        }
        List<HydroStationInfo> hydroStationInfos = hydroStationInfoMapper.selectList(queryWrapper);
        return HydroStationInfoView.builder()
                .hydroStationInfoList(hydroStationInfos)
                .build();
    }

    @Override
    public WatershedInfoView queryWaterShedOrder() {
        List<WatershedInfo> watershedInfos = watershedDataMapper.selectWatershedOrderInfo();
        return WatershedInfoView.builder().watershedOrderList(watershedInfos).build();
    }

    @Override
    public HydroStationDataView queryHydroStationOrder(String orderby) {
        List<HydroStationDataInfo> hydroStationDataInfos;
        if (orderby.equalsIgnoreCase("水位变化")) {
            hydroStationDataInfos = hydroStationDataMapper.queryLatestHydroStationDataOrderByZC();
        } else if (orderby.equalsIgnoreCase("流量变化")) {
            hydroStationDataInfos = hydroStationDataMapper.queryLatestHydroStationDataOrderByZC();
        } else {
            throw new ParamsException();
        }
        return HydroStationDataView.builder().hydroStationDataList(hydroStationDataInfos).build();
    }

    @Override
    public ModelDicView queryHydrologicalForecastModelDic() {
        QueryWrapper<ModelDic> queryWrapper = new QueryWrapper<>();
        List<ModelDic> modelDics = modelDicMapper.selectList(queryWrapper);
        return ModelDicView.builder().modelDicList(modelDics).build();
    }

    @Override
    public WatershedDataView queryWatershedList() {
        LambdaQueryWrapper<WatershedData> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.isNotNull(WatershedData::getBsid);
        List<WatershedData> watershedDataList = watershedDataMapper.selectList(lambdaQueryWrapper);
        return WatershedDataView.builder().watershedDataList(watershedDataList).build();
    }

}
