package com.cuit.basin.base.biz.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.basin.base.biz.dao.HydroDayCharacteristicValueMapper;
import com.cuit.basin.base.biz.entity.HydroDayCharacteristicValue;
import com.cuit.basin.base.biz.service.HydrologicalAnalysisService;
import com.cuit.basin.base.biz.vo.HydroAnalysisView;
import com.cuit.basin.framework.common.exception.ParamsException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.time.LocalDate;
import java.util.List;
import java.util.OptionalDouble;


@Service
@Slf4j
public class HydrologicalAnalysisServiceImpl implements HydrologicalAnalysisService {
    @Autowired
    HydroDayCharacteristicValueMapper hydroDayCharacteristicValueMapper;
    @Override
    /**
     * 查询同比环比数据列表
     */
    public HydroAnalysisView queryAnalysisData(String stationNo, String queryType, String timePeriod, LocalDate startTime, LocalDate endTime) {
      //  LocalDate localDate = LocalDate.now();
        LocalDate localDate = LocalDate.of(2022,11,14);
        LambdaQueryWrapper<HydroDayCharacteristicValue> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
        lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getStcd,stationNo);
        LambdaQueryWrapper<HydroDayCharacteristicValue> lambdaQueryWrapper2 = new LambdaQueryWrapper<>();
        lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getStcd,stationNo);
        int year = localDate.getYear();
        int monthValue = localDate.getMonthValue();
        int dayOfYear = localDate.getDayOfYear();
        List<HydroDayCharacteristicValue> hydroDayCharacteristicValueList1;
        List<HydroDayCharacteristicValue> hydroDayCharacteristicValueList2;
        if(queryType.equalsIgnoreCase("同比")){
            if(!StringUtils.isEmpty(timePeriod)){
                switch(timePeriod){
                    case "年":
                        lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getYr,year);
                        lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getYr,year-1);
                        break;
                    case "季":
                        if(monthValue>=3) {
                            lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getMnth, monthValue,monthValue-2);
                            lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getMnth, monthValue,monthValue-2);
                            lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getYr,year);
                            lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getYr,year-1);
                        }else {
                            throw new ParamsException("本年度不足三个月。");
                        }
                        break;
                    case "月":
                            lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getMnth, monthValue);
                            lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getMnth, monthValue);
                            lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getYr,year);
                            lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getYr,year-1);
                        break;
                    case "旬":
                        if(dayOfYear<10){
                            throw new ParamsException("本年度数据不足10天");
                        }
                        LocalDate startDate1 = localDate.plusDays(-10);
                        lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getDt,startDate1,localDate);
                        LocalDate startDate2 = startDate1.plusYears(-1);
                        LocalDate endDate1 = localDate.plusYears(-1);
                        lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getDt,startDate2,endDate1);
                        break;
                    case "周":
                        if(dayOfYear<7){
                            throw new ParamsException("本年度数据不足7天");
                        }
                        LocalDate startDate3 = localDate.plusDays(-7);
                        lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getDt,startDate3,localDate);
                        LocalDate startDate4 = startDate3.plusYears(-1);
                        LocalDate endDate2 = localDate.plusYears(-1);
                        lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getDt,startDate4,endDate2);
                        break;
                }
            }else {
                lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getDt,startTime,endTime);
                LocalDate startDate5 = startTime.plusYears(-1);
                LocalDate endDate3 = endTime.plusYears(-1);
                lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getDt,startDate5,endDate3);
            }
          hydroDayCharacteristicValueList1 = hydroDayCharacteristicValueMapper.selectList(lambdaQueryWrapper1);
          hydroDayCharacteristicValueList2 = hydroDayCharacteristicValueMapper.selectList(lambdaQueryWrapper2);
        }else if(queryType.equalsIgnoreCase("环比")){
            if(!StringUtils.isEmpty(timePeriod)){
                switch(timePeriod){
                    case "年":
                       throw new ParamsException("环比不支持年");
                    case "季":
                        if(monthValue>=6) {
                            lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getMnth, monthValue,monthValue-2);
                            lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getMnth, monthValue,monthValue-2);
                            lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getYr,year);
                            lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getYr,year);
                        }else {
                            throw new ParamsException("本年度不足6个月。");
                        }
                        break;
                    case "月":
                        if(monthValue>=2) {
                            lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getMnth, monthValue);
                            lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getMnth, monthValue-1);
                            lambdaQueryWrapper1.eq(HydroDayCharacteristicValue::getYr,year);
                            lambdaQueryWrapper2.eq(HydroDayCharacteristicValue::getYr,year);
                        }else{
                            throw new ParamsException("本年度不足2个月。");
                        }
                        break;
                    case "旬":
                        if(dayOfYear<20){
                            throw new ParamsException("本年度数据不足20天");
                        }
                        LocalDate startDate1 = localDate.plusDays(-10);
                        lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getDt,startDate1,localDate);
                        LocalDate startDate2 = localDate.plusDays(-20);
                        lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getDt,startDate2,startDate1);
                        break;
                    case "周":
                        if(dayOfYear<14){
                            throw new ParamsException("本年度数据不足14天");
                        }
                        LocalDate startDate3 = localDate.plusDays(-7);
                        lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getDt,startDate3,localDate);
                        LocalDate startDate4 = localDate.plusDays(-14);
                        lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getDt,startDate4,startDate3);
                        break;
                }
            }else {
                lambdaQueryWrapper1.between(HydroDayCharacteristicValue::getDt,startTime,endTime);
                int dayOfYear1 = startTime.getDayOfYear();
                int dayOfYear2 = endTime.getDayOfYear();
                int days = dayOfYear1 - dayOfYear2;
                LocalDate startTime1 = startTime.plusDays(days);
                lambdaQueryWrapper2.between(HydroDayCharacteristicValue::getDt,startTime1,startTime);
            }
             hydroDayCharacteristicValueList1 = hydroDayCharacteristicValueMapper.selectList(lambdaQueryWrapper1);
             hydroDayCharacteristicValueList2 = hydroDayCharacteristicValueMapper.selectList(lambdaQueryWrapper2);
        }else {
            throw new ParamsException("查询类型只支持：同比、环比");
        }
        return HydroAnalysisView.builder().hydroDayCharacteristicValueList1(hydroDayCharacteristicValueList1).
                hydroDayCharacteristicValueList2(hydroDayCharacteristicValueList2).build();
    }
    @Override
    /**
     * 查询同比环比增长率
     */
    public Double queryAnalysisRatio(String stationNo, String queryType, String timePeriod, LocalDate startTime, LocalDate endTime, String dataType, String characterType) {
        HydroAnalysisView hydroAnalysisView = this.queryAnalysisData(stationNo, queryType, timePeriod, startTime, endTime);
        List<HydroDayCharacteristicValue> hydroDayCharacteristicValueList1 = hydroAnalysisView.getHydroDayCharacteristicValueList1();
        List<HydroDayCharacteristicValue> hydroDayCharacteristicValueList2 = hydroAnalysisView.getHydroDayCharacteristicValueList2();
        Double ratio = 0.0;
        switch (dataType){
            case "最高水位":
                if(characterType.equalsIgnoreCase("最大值")) {
                    OptionalDouble max1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).max();
                    OptionalDouble max2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).max();
                    ratio = max1.getAsDouble()/max2.getAsDouble()-1;
                }else if(characterType.equalsIgnoreCase("平均值")){
                    double sum1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).sum();
                    long count1= hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).count();
                    Double avg1 = sum1/count1;
                    double sum2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).sum();
                    long count2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).count();
                    Double avg2 = sum2/count2;
                    ratio = avg1/avg2-1;
                }else if(characterType.equalsIgnoreCase("最小值")){
                    OptionalDouble min1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).min();
                    OptionalDouble min2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getHtz).min();
                    ratio = min1.getAsDouble()/min2.getAsDouble()-1;
                }else {
                    throw new ParamsException("仅支持最大值、最小值、平均值。");
                }
                break;
            case "最大流量":
                if(characterType.equalsIgnoreCase("最大值")) {
                    OptionalDouble max1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).max();
                    OptionalDouble max2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).max();
                    ratio = max1.getAsDouble()/max2.getAsDouble()-1;
                }else if(characterType.equalsIgnoreCase("平均值")){
                    double sum1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).sum();
                    long count1= hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).count();
                    Double avg1 = sum1/count1;
                    double sum2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).sum();
                    long count2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).count();
                    Double avg2 = sum2/count2;
                    ratio = avg1/avg2-1;
                }else if(characterType.equalsIgnoreCase("最小值")){
                    OptionalDouble min1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).min();
                    OptionalDouble min2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMxq).min();
                    ratio = min1.getAsDouble()/min2.getAsDouble()-1;
                }else {
                    throw new ParamsException("仅支持最大值、最小值、平均值。");
                }
                break;
            case "平均水位":
                if(characterType.equalsIgnoreCase("最大值")) {
                    OptionalDouble max1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).max();
                    OptionalDouble max2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).max();
                    ratio = max1.getAsDouble()/max2.getAsDouble()-1;
                }else if(characterType.equalsIgnoreCase("平均值")){
                    double sum1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).sum();
                    long count1= hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).count();
                    Double avg1 = sum1/count1;
                    double sum2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).sum();
                    long count2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).count();
                    Double avg2 = sum2/count2;
                    ratio = avg1/avg2-1;
                }else if(characterType.equalsIgnoreCase("最小值")){
                    OptionalDouble min1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).min();
                    OptionalDouble min2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvz).min();
                    ratio = min1.getAsDouble()/min2.getAsDouble()-1;
                }else {
                    throw new ParamsException("仅支持最大值、最小值、平均值。");
                }
                break;
            case "平均流量":
                if(characterType.equalsIgnoreCase("最大值")) {
                    OptionalDouble max1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).max();
                    OptionalDouble max2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).max();
                    ratio = max1.getAsDouble()/max2.getAsDouble()-1;
                }else if(characterType.equalsIgnoreCase("平均值")){
                    double sum1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).sum();
                    long count1= hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).count();
                    Double avg1 = sum1/count1;
                    double sum2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).sum();
                    long count2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).count();
                    Double avg2 = sum2/count2;
                    ratio = avg1/avg2-1;
                }else if(characterType.equalsIgnoreCase("最小值")){
                    OptionalDouble min1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).min();
                    OptionalDouble min2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getAvq).min();
                    ratio = min1.getAsDouble()/min2.getAsDouble()-1;
                }else {
                    throw new ParamsException("仅支持最大值、最小值、平均值。");
                }
                break;
            case "最低水位":
                if(characterType.equalsIgnoreCase("最大值")) {
                    OptionalDouble max1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).max();
                    OptionalDouble max2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).max();
                    ratio = max1.getAsDouble()/max2.getAsDouble()-1;
                }else if(characterType.equalsIgnoreCase("平均值")){
                    double sum1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).sum();
                    long count1= hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).count();
                    Double avg1 = sum1/count1;
                    double sum2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).sum();
                    long count2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).count();
                    Double avg2 = sum2/count2;
                    ratio = avg1/avg2-1;
                }else if(characterType.equalsIgnoreCase("最小值")){
                    OptionalDouble min1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).min();
                    OptionalDouble min2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getLtz).min();
                    ratio = min1.getAsDouble()/min2.getAsDouble()-1;
                }else {
                    throw new ParamsException("仅支持最大值、最小值、平均值。");
                }
                break;
            case "最小流量":
                if(characterType.equalsIgnoreCase("最大值")) {
                    OptionalDouble max1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).max();
                    OptionalDouble max2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).max();
                    ratio = max1.getAsDouble()/max2.getAsDouble()-1;
                }else if(characterType.equalsIgnoreCase("平均值")){
                    double sum1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).sum();
                    long count1= hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).count();
                    Double avg1 = sum1/count1;
                    double sum2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).sum();
                    long count2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).count();
                    Double avg2 = sum2/count2;
                    ratio = avg1/avg2-1;
                }else if(characterType.equalsIgnoreCase("最小值")){
                    OptionalDouble min1 = hydroDayCharacteristicValueList1.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).min();
                    OptionalDouble min2 = hydroDayCharacteristicValueList2.stream().mapToDouble(HydroDayCharacteristicValue::getMnq).min();
                    ratio = min1.getAsDouble()/min2.getAsDouble()-1;
                }else {
                    throw new ParamsException("仅支持最大值、最小值、平均值。");
                }
                break;
        }
        DecimalFormat format = new DecimalFormat("#.00");
        String str = format.format(ratio);
        double ratio2 = Double.parseDouble(str);
        return ratio2;
    }
}
