package com.cuit.basin.base.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.basin.base.biz.dao.*;
import com.cuit.basin.base.biz.entity.*;
import com.cuit.basin.base.biz.entity.dic.DicEigenvalue;
import com.cuit.basin.base.biz.pojo.*;
import com.cuit.basin.base.biz.service.HydrologicalBulletinService;
import com.cuit.basin.base.biz.vo.*;
import com.cuit.basin.framework.common.ret.PageVo;
import com.cuit.basin.framework.common.utils.word.WordHtmlUtil;
import com.spire.doc.*;
import com.spire.doc.documents.*;
import org.apache.poi.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.awt.*;
import java.io.*;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
public class HydrologicalBulletinServiceImpl implements HydrologicalBulletinService {
    @Autowired
    private Environment environment;
    @Autowired
    private HydPiBulletinPMapper hydPiBulletinPMapper;
    @Autowired
    private HydroStationInfoMapper hydroStationInfoMapper;
    @Autowired
    private DicBulletinTypeMapper typeMapper;
    @Autowired
    private DicBulletinStutusMapper stutusMapper;
    @Autowired
    private DicEigenvalueMapper eigenvalueMapper;
    @Autowired
    private ThematicMapMapper thematicMapMapper;
    @Autowired
    private MeteStationInfoMapper meteStationInfoMapper;
    @Autowired
    private MeteStationDataMapper meteStationDataMapper;
    @Autowired
    private HydroStationDataMapper hydroStationDataMapper;
    @Autowired
    private HydFiStdataFMapper hydFiStdataFMapper;

    private String head = "";
    private String edd = "";
//    @Override
//    public HydroBulletinView queryPagesHydroBulletin(HydroBulletinQuery query) {
//        LambdaQueryWrapper<HydPiBulletinP> lambdaQueryWrapper = new LambdaQueryWrapper<>();
//        if (query.getAuditstutus() != null) {
//            lambdaQueryWrapper.eq(HydPiBulletinP::getAuditStutus, query.getAuditstutus());
//        }
//        if (query.getBtype() != null) {
//            lambdaQueryWrapper.eq(HydPiBulletinP::getBTYPE, query.getBtype());
//        }
//        if (query.getBsid() != null) {
//            lambdaQueryWrapper.eq(HydPiBulletinP::getBSID, query.getBsid());
//        }
//        if (query.getKeyword() != null && !query.getKeyword().equalsIgnoreCase("")) {
//            lambdaQueryWrapper.like(HydPiBulletinP::getBNM, query.getKeyword());
//        }
//        if (query.getSigner() != null && !query.getSigner().equalsIgnoreCase("")) {
//            lambdaQueryWrapper.eq(HydPiBulletinP::getSIGNER, query.getSigner());
//        }
//        if (query.getStTime() != null && query.getEndTime() != null) {
//            lambdaQueryWrapper.between(HydPiBulletinP::getPUBTIME, query.getStTime(), query.getEndTime());
//        }
//        Page<HydPiBulletinP> page = new Page<>(query.getCurrent(), query.getSize());
//        hydPiBulletinPMapper.selectPage(page, lambdaQueryWrapper);
//        return HydroBulletinView.builder().hydPiBulletinPList(page.getRecords()).current(page.getCurrent())
//                .size(page.getSize())
//                .total(page.getTotal())
//                .build();
//    }

    /**
     * 条件分页查询公报列表
     *
     * @param query 查询参数类
     * @return PageVo<HydPiBulletinPInfo>
     */
    @Override
    public PageVo<HydPiBulletinPInfo> queryGazetteListByParams(GazetteListQuery query) {
        //分页对象，当前面及每页条数
        Page<HydPiBulletinPInfo> page = new Page<>(query.getCurrent(), query.getSize());
        //查询信息
        hydPiBulletinPMapper.getGazetteListByParams(page, query.getBsId(), query.getTypeId(), query.getAuditsTutus()
                , query.getStTime(), query.getEndTime(), query.getKeyWords());
        //将Page转换为PageVO
        return new PageVo<>(page);
    }

    /**
     * 查询公报信息
     *
     * @param bId 公报id
     * @return HydPiBulletinP
     */
    @Override
    public HydPiBulletinP queryGazetteById(Integer bId) {
        QueryWrapper<HydPiBulletinP> hydPiBulletinPQueryWrapper = new QueryWrapper<>();
        HydPiBulletinP hydPiBulletinP = hydPiBulletinPMapper.selectOne(hydPiBulletinPQueryWrapper.eq("BID", bId));
        return hydPiBulletinP;
    }

    /**
     * 批量删除公报
     *
     * @param bIds 公报Id(多个,隔开)
     * @return Boolean
     */
    @Override
    public Boolean batchDeleteGazetteByIds(String bIds) {
        String ids = bIds.replaceAll("，", ",");
        int ret = hydPiBulletinPMapper.delete(new LambdaQueryWrapper<HydPiBulletinP>()
                .apply("FIND_IN_SET(BID, '" + ids + "')"));
        return (ret >= 1);
    }

    /**
     * 保存公报信息
     *
     * @param params
     * @return String
     */
    @Override
    public String saveGazette(HydPiBulletinPParams params) {
        HydPiBulletinP hydPiBulletinP = HydPiBulletinP.builder().auditStutus(2).bNm(params.getBnm()).bType(params.getBtype())
                .publishTags(params.getPublishTags()).createdTime(LocalDateTime.now()).discription(params.getDiscription())
                .content(params.getContent()).signer(params.getSigner()).build();
        DateTimeFormatter dfcn = DateTimeFormatter.ofPattern("yyMMddHH");
        String outLocation = environment.getProperty("outLocation");
        String mapLocation = environment.getProperty("mapLocation");
        String fileName = outLocation + File.separator + "pdf" + File.separator + params.getBnm() + dfcn.format(LocalDateTime.now()) + ".pdf";
        String htmlName = outLocation + File.separator + "pdf" + File.separator + params.getBnm() + dfcn.format(LocalDateTime.now()) + ".html";
        //新建Document对象
        Document document = new Document();
        //添加section
        Section sec = document.addSection();
        String html = head + params.getContent() + edd;
//        WordHtmlUtil.addHtml(params.getContent(),htmlName);
//        document.loadFromFile(htmlName, FileFormat.Html, XHTMLValidationType.None);
        String htmlText = (html);
        //添加段落并写入HTML文本
        sec.addParagraph().appendHTML(htmlText);
        //文档另存为PDF
        document.saveToFile(fileName, FileFormat.PDF);
        String address = fileName.replace(outLocation, mapLocation).replace(File.separator, "/");


//        try {
//            WordHtmlUtil.fileZh(fileName,fileName);
//        } catch (IOException e) {
//            throw new RuntimeException(e);
//        }
        document.dispose();


//        String a = params.getContent();
//        String b = "<html><body>" + JSONObject.parseObject(a).get("html").toString() + "</body></html>";
//        PdfUtil.createdPdfByItextHtml(b, new File("fileName"));
        hydPiBulletinP.setWordUrl(address);
        hydPiBulletinPMapper.insert(hydPiBulletinP);
        return address;
    }

    /**
     * 预览公报
     *
     * @param param 条件
     * @return String
     */
    @Override
    public String produceGazette(ProduceGazetteParam param) {
        int secondTitleNum = 0;
        String stDate = null, endDate = null;//文本通用内容
        DateTimeFormatter dfcn = DateTimeFormatter.ofPattern("yyyy年MM月dd日HH时");
        stDate = dfcn.format(param.getStTime());
        endDate = dfcn.format(param.getEndTime());
        //1.根据选择条件获得雨情数据，拼成文本--------------------------------------------------------------------------------
        List<MeteStationInfo> meteStationInfos = param.getRainParamsView().getMeteStationInfos();//雨情站点
        List<ThematicMap> rainThematicMapList = param.getRainParamsView().getThematicMapList();//雨情专题图
        List<DicEigenvalue> rainEigenvalue = param.getEigenvalueView().getRainEigenvalue().getEigenvalues();//雨情特征值
        String rainTitle = null, rainContent = null;//雨情文本内容
        List<RainGazetteData> rainGazetteData = new ArrayList<>();//符合条件的雨情数据
        if (!ObjectUtils.isEmpty(meteStationInfos)) {
            secondTitleNum++;//二级标题计数器
            //获取二级标题序号
            rainTitle = getSecondTitleNum(secondTitleNum, "雨情");
            List<String> stcdList = meteStationInfos.stream().map(MeteStationInfo::getStcd).collect(Collectors.toList());
            rainGazetteData = meteStationDataMapper.selectByGazetteParam(StringUtil.join(stcdList.toArray(), ","), param.getStTime(), param.getEndTime(), param.getSituation());
            if (!ObjectUtils.isEmpty(rainGazetteData)) {
                Map<String, List<RainGazetteData>> map = rainGazetteData.stream().collect(Collectors.groupingBy(RainGazetteData::getStcd));
                //获取不同站点条件时间段的特征值
                List<GazetteEigenvalue> rainEigenvalueList = new ArrayList<>();
                map.forEach((k, v) -> {
                    //计算数据统计值
                    Double max1hp, min1hp, avg1hp, max10mp, min10mp, avg10mp = null;
                    max1hp = v.stream().mapToDouble(RainGazetteData::getPre1h).max().getAsDouble();
                    min1hp = v.stream().mapToDouble(RainGazetteData::getPre1h).min().getAsDouble();
                    avg1hp = v.stream().mapToDouble(RainGazetteData::getPre1h).average().getAsDouble();
                    max10mp = v.stream().mapToDouble(RainGazetteData::getPre10min).max().getAsDouble();
                    min10mp = v.stream().mapToDouble(RainGazetteData::getPre10min).min().getAsDouble();
                    avg10mp = v.stream().mapToDouble(RainGazetteData::getPre10min).average().getAsDouble();
                    GazetteEigenvalue eigenvalue = GazetteEigenvalue.builder()
                            .stcd(k).stnm(v.get(0).getStnm())
                            .maxq(Double.valueOf(String.format("%.2f", max1hp)))
                            .minq(Double.valueOf(String.format("%.2f", min1hp)))
                            .avgq(Double.valueOf(String.format("%.2f", avg1hp)))
                            .maxz(Double.valueOf(String.format("%.2f", max10mp)))
                            .minz(Double.valueOf(String.format("%.2f", min10mp)))
                            .avgz(Double.valueOf(String.format("%.2f", avg10mp))).build();
                    rainEigenvalueList.add(eigenvalue);
                });
                //获取特征值
                String stationContent = "";
                for (GazetteEigenvalue each1 : rainEigenvalueList) {
                    String rainChangContent = "";
                    for (int i = 0; i < rainEigenvalue.size(); i++) {
                        String startSymbol = ",";
                        if (i == 0) {
                            startSymbol = "";
                        }
                        rainChangContent = rainChangContent + startSymbol + rainEigenvalue.get(i).getEigenvalueName() + "为" + getGazetteEigenvalue(rainEigenvalue.get(i).getEigenvalueName(), each1, "雨情");
                    }
                    stationContent = stationContent + each1.getStcd() + "（" + each1.getStnm() + "）" + rainChangContent + "。\n";
                }
                //若选择超警站点，获取超警信息
                String warnContent = "";
                if (!ObjectUtils.isEmpty(param.getSituation())) {
                    //获取雨情站点额警戒值
                    MeteBiWarntB warntB = meteStationInfoMapper.seletMeteBiWarntB();//雨情阈值
                    for (GazetteEigenvalue eigenvalue : rainEigenvalueList) {
                        GazetteEigenvalue value = new GazetteEigenvalue();
                        value.setMaxz(eigenvalue.getMaxz() - warntB.getPre1hWrt());
                        value.setMinz(eigenvalue.getMaxz() - warntB.getPre1hWrt());
                        value.setAvgz(eigenvalue.getMaxz() - warntB.getPre1hWrt());
                        value.setMaxq(eigenvalue.getMaxz() - warntB.getPre10minWrt());
                        value.setMinq(eigenvalue.getMaxz() - warntB.getPre10minWrt());
                        value.setAvgq(eigenvalue.getMaxz() - warntB.getPre10minWrt());
                    }
                    for (GazetteEigenvalue excess : rainEigenvalueList) {
                        String changeContent = "";
                        for (int i = 0; i < rainEigenvalue.size(); i++) {
                            String startSymbol = ",";
                            if (i == 0) {
                                startSymbol = "";
                            }
                            changeContent = changeContent + startSymbol + rainEigenvalue.get(i).getEigenvalueName()
                                    + "超过阈值" + getGazetteEigenvalue(rainEigenvalue.get(i).getEigenvalueName(), excess, "雨情");
                        }
                        warnContent = warnContent + excess.getStcd() + "（" + excess.getStnm() + "）" + changeContent + "。\n";
                    }
                }
                rainContent = stDate + "至" + endDate + "," + rainGazetteData.get(0).getBsnm() + "流域内。\n特征值统计：\n" + stationContent + "超警情况：\n" + warnContent;
            }
        }

        //2.根据选择条件获得水情数据，拼成文本写word--------------------------------------------------------------------------
//        getWaterText();
        List<HydroStationInfo> waterStationInfos = param.getRegimenParamsView().getHydroStationInfoList();
        List<ThematicMap> waterThematicMapList = param.getRegimenParamsView().getThematicMapList();
        List<DicEigenvalue> waterEigenvalue = param.getEigenvalueView().getWaterEigenvalue().getEigenvalues();//水情特征值
        String waterTitle = null, waterContent = null;//水情文本内容
        List<WaterGazetteData> waterGazetteData = new ArrayList<>();//符合条件的水情数据
        if (!ObjectUtils.isEmpty(waterStationInfos)) {
            List<String> stcdList = waterStationInfos.stream().map(HydroStationInfo::getStcd).collect(Collectors.toList());
            //查询出符合条件的数据
            waterGazetteData = hydroStationDataMapper.selectByGazetteParam(StringUtil.join(stcdList.toArray(), ","), param.getStTime(), param.getEndTime(), param.getSituation());
            //获取不同站点条件时间段的特征值
            List<GazetteEigenvalue> waterEigenvalueList = new ArrayList<>();
            if (!ObjectUtils.isEmpty(waterGazetteData)) {
                secondTitleNum++;//二级标题计数器
                //根据站号分类
                Map<String, List<WaterGazetteData>> map = waterGazetteData.stream().collect(Collectors.groupingBy(WaterGazetteData::getStcd));
                map.forEach((k, v) -> {
                    //计算数据统计值
                    Double maxq, minq, avgq, maxz, minz, avgz = null;
                    maxq = v.stream().mapToDouble(WaterGazetteData::getQ).max().getAsDouble();
                    minq = v.stream().mapToDouble(WaterGazetteData::getQ).min().getAsDouble();
                    avgq = v.stream().mapToDouble(WaterGazetteData::getQ).average().getAsDouble();
                    maxz = v.stream().mapToDouble(WaterGazetteData::getZ).max().getAsDouble();
                    minz = v.stream().mapToDouble(WaterGazetteData::getZ).min().getAsDouble();
                    avgz = v.stream().mapToDouble(WaterGazetteData::getZ).average().getAsDouble();
                    GazetteEigenvalue eigenvalue = GazetteEigenvalue.builder()
                            .stcd(k).stnm(v.get(0).getStnm())
                            .maxq(Double.valueOf(String.format("%.2f", maxq)))
                            .minq(Double.valueOf(String.format("%.2f", minq)))
                            .avgq(Double.valueOf(String.format("%.2f", avgq)))
                            .maxz(Double.valueOf(String.format("%.2f", maxz)))
                            .minz(Double.valueOf(String.format("%.2f", minz)))
                            .avgz(Double.valueOf(String.format("%.2f", avgz))).build();
                    waterEigenvalueList.add(eigenvalue);
                });
                //获取二级标题序号
                waterTitle = getSecondTitleNum(secondTitleNum, "水情");
                String stationContent = "";
                for (GazetteEigenvalue each1 : waterEigenvalueList) {
                    String waterChangContent = "";
                    for (DicEigenvalue each2 : waterEigenvalue) {
                        waterChangContent = waterChangContent + "，小时" + each2.getEigenvalueName() + "为" + getGazetteEigenvalue(each2.getEigenvalueName(), each1, "水情");
                    }
                    stationContent = stationContent + each1.getStcd() + "（" + each1.getStnm() + "）" + waterChangContent + "。\n";
                }
                //若选择超警站点，获取超警信息
                String warnContent = "";
                if (!ObjectUtils.isEmpty(param.getSituation())) {
                    //获取水情站点额警戒值
                    Double warnZ, warnQ;
//                    List<GazetteEigenvalue> waterEigenvalueListExcess = new ArrayList<>();//特征值与警戒线的超警差值
                    for (GazetteEigenvalue eigenvalue : waterEigenvalueList) {
                        for (HydroStationInfo stationInfo : waterStationInfos) {
                            if (stationInfo.getStcd().equals(eigenvalue.getStcd())) {
                                warnZ = stationInfo.getWrz();
                                warnQ = stationInfo.getWrq();
                                eigenvalue.setMaxz(eigenvalue.getMaxz() - warnZ);
                                eigenvalue.setMinz(eigenvalue.getMaxz() - warnZ);
                                eigenvalue.setAvgz(eigenvalue.getMaxz() - warnZ);
                                eigenvalue.setMaxq(eigenvalue.getMaxz() - warnQ);
                                eigenvalue.setMinq(eigenvalue.getMaxz() - warnQ);
                                eigenvalue.setAvgq(eigenvalue.getMaxz() - warnQ);
                            }
                        }
                        String changeContent = "";
                        for (int i = 0; i < waterEigenvalue.size(); i++) {
                            String startSymbol = ",";
                            if (i == 0) {
                                startSymbol = "";
                            }
                            changeContent = changeContent + startSymbol + rainEigenvalue.get(i).getEigenvalueName() + "超过警戒值" + getGazetteEigenvalue(waterEigenvalue.get(i).getEigenvalueName(), eigenvalue, "水情");
                        }
                        warnContent = warnContent + eigenvalue.getStcd() + "（" + eigenvalue.getStnm() + "）" + changeContent + "。\n";
                    }
                }
                waterContent = stDate + "至" + endDate + "," + waterGazetteData.get(0).getBsnm() + "流域内。\n特征值统计：\n" + stationContent + "\n超警情况：\n" + warnContent +"\n";
            }
        }

        //3.根据选择条件获得模型预报数据-------------------------------------------------------------------------------------
        List<HydroStationInfo> modeStationInfoList = param.getModeForecastParamsView().getHydroStationInfoList();
        List<ThematicMap> modeThematicMapList = param.getModeForecastParamsView().getThematicMapList();
        List<DicEigenvalue> modeEigenvalue = param.getEigenvalueView().getModeEigenvalue().getEigenvalues();//模型预报特征值
        String modeTitle = null, modeContent = null;//模型预报文本内容
        List<ModeGazetteData> modeGazetteData = new ArrayList<>();//符合条件的模型预报数据
//        DateTimeFormatter df0 = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        if (!ObjectUtils.isEmpty(modeStationInfoList)) {
            secondTitleNum++;//二级标题计数器
            List<String> stcdList = modeStationInfoList.stream().map(HydroStationInfo::getStcd).collect(Collectors.toList());
            modeGazetteData = hydFiStdataFMapper.selectByGazetteParam(StringUtil.join(stcdList.toArray(), ",")
                    , param.getStTime(), param.getEndTime(), param.getSituation());
            //获取不同站点条件时间段的特征值
            List<GazetteEigenvalue> modeEigenvalueList = new ArrayList<>();
            if (!ObjectUtils.isEmpty(modeGazetteData)) {
                //根据站点分类
                Map<String, List<ModeGazetteData>> map = modeGazetteData.stream().collect(Collectors.groupingBy(ModeGazetteData::getStcd));
                map.forEach((k, v) -> {
                    //再根据模型分类
                    Map<String, List<ModeGazetteData>> maps = v.stream().collect(Collectors.groupingBy(ModeGazetteData::getMname));
                    maps.forEach((k2, v2) -> {
                        //计算数据统计值
                        Double maxq, minq, avgq, maxz, minz, avgz = null;
                        BigDecimal maxfq = v2.stream().map(ModeGazetteData::getFq).max((x1, x2) -> x1.compareTo(x2)).get();
                        BigDecimal minfq = v2.stream().map(ModeGazetteData::getFq).min((x1, x2) -> x1.compareTo(x2)).get();
                        BigDecimal averagefq = v2.stream().map(vo -> ObjectUtils.isEmpty(vo.getFq()) ? new BigDecimal(0) : vo.getFq())
                                .reduce(BigDecimal.ZERO, BigDecimal::add).divide(BigDecimal.valueOf(v.size()), 2, BigDecimal.ROUND_HALF_UP);
                        BigDecimal maxfz = v2.stream().map(ModeGazetteData::getFz).max((x1, x2) -> x1.compareTo(x2)).get();
                        BigDecimal minfz = v2.stream().map(ModeGazetteData::getFz).min((x1, x2) -> x1.compareTo(x2)).get();
                        BigDecimal averagefz = v2.stream().map(vo -> ObjectUtils.isEmpty(vo.getFz()) ? new BigDecimal(0) : vo.getFz())
                                .reduce(BigDecimal.ZERO, BigDecimal::add).divide(BigDecimal.valueOf(v2.size()), 2, BigDecimal.ROUND_HALF_UP);
                        maxq = maxfq.doubleValue();
                        minq = minfq.doubleValue();
                        avgq = averagefq.doubleValue();
                        maxz = maxfz.doubleValue();
                        minz = minfz.doubleValue();
                        avgz = averagefz.doubleValue();
                        GazetteEigenvalue eigenvalue = GazetteEigenvalue.builder()
                                .stcd(k).stnm(v.get(0).getStnm())
                                .mname(k2)
                                .maxq(Double.valueOf(String.format("%.2f", maxq)))
                                .minq(Double.valueOf(String.format("%.2f", minq)))
                                .avgq(Double.valueOf(String.format("%.2f", avgq)))
                                .maxz(Double.valueOf(String.format("%.2f", maxz)))
                                .minz(Double.valueOf(String.format("%.2f", minz)))
                                .avgz(Double.valueOf(String.format("%.2f", avgz))).build();
                        modeEigenvalueList.add(eigenvalue);
                    });
                });
                //获取二级标题序号
                modeTitle = getSecondTitleNum(secondTitleNum, "模型预报");
                List<String> content;
                String stationContent = "";
                for (int i = 0; i < modeEigenvalueList.size(); i++) {
                    String modeEigenvalueContent = "";
//                    changContent = 
                    for (DicEigenvalue each2 : modeEigenvalue) {
                        modeEigenvalueContent = modeEigenvalueContent + "，预测小时" + each2.getEigenvalueName() + "为" + getGazetteEigenvalue(each2.getEigenvalueName(), modeEigenvalueList.get(i), "水情");
                    }
                    String stnm = modeEigenvalueList.get(i).getStnm(), stcd = modeEigenvalueList.get(i).getStcd(), mName = modeEigenvalueList.get(i).getMname();
                    stationContent = stationContent + stcd + "（" + stnm + "）" + "，" + mName + "模型" + modeEigenvalueContent + "。\n";
                }
                //若选择超警站点，获取超警信息
                String warnContent = "\n超警情况：\n";
                if (!ObjectUtils.isEmpty(param.getSituation())) {
                    //获取模型预报站点额警戒值
                    Double warnZ, warnQ;
//                    List<GazetteEigenvalue> waterEigenvalueListExcess = new ArrayList<>();//特征值与警戒线的超警差值
                    for (GazetteEigenvalue eigenvalue : modeEigenvalueList) {
                        for (HydroStationInfo stationInfo : modeStationInfoList) {
                            if (stationInfo.getStcd().equals(eigenvalue.getStcd())) {
                                warnZ = stationInfo.getWrz();
                                warnQ = stationInfo.getWrq();
                                eigenvalue.setMaxz(eigenvalue.getMaxz() - warnZ);
                                eigenvalue.setMinz(eigenvalue.getMaxz() - warnZ);
                                eigenvalue.setAvgz(eigenvalue.getMaxz() - warnZ);
                                eigenvalue.setMaxq(eigenvalue.getMaxz() - warnQ);
                                eigenvalue.setMinq(eigenvalue.getMaxz() - warnQ);
                                eigenvalue.setAvgq(eigenvalue.getMaxz() - warnQ);
                            }
                        }
                        String changeContent = "";
                        for (int i = 0; i < modeEigenvalue.size(); i++) {
                            String startSymbol = ",";
                            if (i == 0) {
                                startSymbol = "";
                            }
                            changeContent = changeContent + startSymbol + rainEigenvalue.get(i).getEigenvalueName() + "超过警戒值" + getGazetteEigenvalue(modeEigenvalue.get(i).getEigenvalueName(), eigenvalue, "水情");
                        }
                        String stnm = eigenvalue.getStnm(), stcd = eigenvalue.getStcd(), mName = eigenvalue.getMname();
                        warnContent = warnContent + stcd + "（" + stnm + "）" + "，" + mName + "模型" + changeContent + "。\n";
                    }
                    modeContent = stDate + "至" + endDate + "," + waterGazetteData.get(0).getBsnm() + "流域内。\n特征值统计：\n" + stationContent + warnContent;
                }else {
                    modeContent = stDate + "至" + endDate + "," + waterGazetteData.get(0).getBsnm() + "流域内。\n特征值统计：\n" + stationContent;
                }
            }
        }

        //4.创建word
        //通过流加载WPS文字文档
//        FileInputStream inputStream = new FileInputStream(new File("E:\\ZMG\\DailyTasks\\青藏高原水文\\公报\\水情简报.docx"));

//        Document document1 = new Document("E:\\ZMG\\DailyTasks\\青藏高原水文\\公报\\水情简报.docx");//模板
        Document document1 = new Document("D:\\SBasinSys\\word\\modeWord.doc");//模板
        //匹配以#{dateTime}开头,并替换
        Pattern replace = Pattern.compile("#\\{dateTime}");
        Pattern replace2 = Pattern.compile("#\\{title}");
        document1.replace(replace, stDate + "-" + endDate);
        document1.replace(replace2, param.getGazetteType());
        //设置格式
        Document document2 = new Document();//预览
        document2 = setDocumentStyle(document2);
        //将雨情文本写入word
        document2 = addSection(stDate, endDate, rainTitle, rainContent, rainThematicMapList, document2, "雨情");
        //将水情文本写入word
        document2 = addSection(stDate, endDate, waterTitle, waterContent, waterThematicMapList, document2, "水情");
        //将模型预测文本写入word
        document2 = addSection(stDate, endDate, modeTitle, modeContent, modeThematicMapList, document2, "模型预报");
        //添加一个eddSection
        Section eddSection = document2.addSection();
        String issueTime;
        //添加个段落至section
        issueTime = LocalDateTime.now().getYear() + "年" + LocalDateTime.now().getMonthValue() + "月" + LocalDateTime.now().getDayOfMonth() + "日";
        Paragraph para1 = eddSection.addParagraph();
        para1.appendText("发布时间:" + issueTime);
        para1.applyStyle("paraStyle");
        //设置段落的对齐方式
        para1.getFormat().setHorizontalAlignment(HorizontalAlignment.Right);
        //设置第一段和第二段的段后间距
        para1.getFormat().setBeforeSpacing(10f);
        para1.getFormat().setAfterSpacing(10f);

        //拼接两个Document
        splicingDocument(document1, document2);
        //生成文件名
//        String fileName =  "E:\\ZMG\\DailyTasks\\青藏高原水文\\公报\\"+param.getGazetteType() +".docx";
        String fileName =  "D:\\SBasinSys\\word\\"+param.getGazetteType() +".doc";
        String fileName2 = "D:\\SBasinSys\\word\\" + param.getGazetteType() + ".html";
        //保存结果文档
        document1.saveToFile(fileName2, FileFormat.Html);
        document1.saveToFile(fileName, FileFormat.Doc);
        String htmlStr1 = WordHtmlUtil.toHtmlString(new File(fileName2));
        //读取.html文件为字符串

//        try {
//            htmlStr = WordHtmlUtil.docToHtml(new File(fileName)).replace("<span class=\"s2\">&lt;","<").replace("&gt;</span>",">");
        String htmlStr2 = htmlStr1
                .replace("<p class=\"imageStyle\" style=\"text-align:center;\"><span style=\"color:#000000;font-size:9.5pt;font-family:'Times New Roman';mso-fareast-font-family:'Times New Roman';mso-bidi-font-family:'Times New Roman';\">&lt;", "<p style=\"text-align: center; width: 150px; height: 150px;\"><")
                .replace("&gt;</span></p>", "></p>");
        String a = "<div style=\"min-height:72pt\" />";
        String htmlStr3 = htmlStr2.substring(htmlStr2.indexOf(a) + a.length(), htmlStr2.indexOf("</div>"));
//        } catch (ParserConfigurationException e) {
//            throw new RuntimeException(e);
//        } catch (TransformerException e) {
//            throw new RuntimeException(e);
//        }
//        //删除预览生成的文件
//        Boolean flag = DeleteFolder(fileName2);
//        System.out.println(flag);
        Document document = new Document();
        //添加section
        Section sec = document.addSection();
        String htmlText = htmlStr2;
        //添加段落并写入HTML文本
        sec.addParagraph().appendHTML(htmlText);
        document.saveToFile("D:\\SBasinSys\\word\\"+"测试html" +".html", FileFormat.Html);
        document.saveToFile("D:\\SBasinSys\\word\\"+"测试doc" +".doc", FileFormat.Doc);
        document.dispose();
//        String outLocation = environment.getProperty("outLocation");
//        String mapLocation = environment.getProperty("mapLocation");
//        String fileName = outLocation + File.separator + "word" + File.separator + "yl" + dfcn.format(LocalDateTime.now()) + ".doc";
//
//        String url =
        return htmlStr3;
    }


    /**
     * 获取公报雨情生成条件
     *
     * @param stTime    开始时间
     * @param endTime   结束时间
     * @param situation 情况
     * @param bsId      流域id
     * @param keywords  关键字
     * @return GazetteRainParamsView
     */
    @Override
    public GazetteRainParamsView getGazetteRainParamsView(LocalDateTime stTime, LocalDateTime endTime, String situation, Integer bsId, String keywords) {
        // 符合条件的雨情站点列表
        List<MeteStationInfo> meteStationInfoList;
        meteStationInfoList = meteStationInfoMapper.getGazetteRainParams(stTime, endTime, bsId, keywords, situation);
        // 符合条件的雨情地图服务数据列表
        List<ThematicMap> thematicMapList;
        thematicMapList = thematicMapMapper.selectListByParams("雨情", stTime, endTime, bsId);
        GazetteRainParamsView view = new GazetteRainParamsView();
        view.setMeteStationInfos(meteStationInfoList);
        view.setThematicMapList(thematicMapList);
        return view;
    }

    /**
     * 获取公报水情生成条件
     *
     * @param stTime    开始时间
     * @param endTime   结束时间
     * @param situation 情况
     * @param bsId      流域id
     * @param keywords  关键字
     * @return GazetteRegimenParamsView
     */
    @Override
    public GazetteRegimenParamsView getGazetteRegimenParamsView(LocalDateTime stTime, LocalDateTime endTime, String situation, Integer bsId, String keywords) {
        // 符合条件的水情站点列表
        List<HydroStationInfo> hydroStationInfoList;
        hydroStationInfoList = hydroStationInfoMapper.getGazetteRegimenParams(stTime, endTime, bsId, keywords, situation);
        // 符合条件的水情地图服务数据列表
        List<ThematicMap> thematicMapList;
        thematicMapList = thematicMapMapper.selectListByParams("水情", stTime, endTime, bsId);
        GazetteRegimenParamsView view = new GazetteRegimenParamsView();
        view.setHydroStationInfoList(hydroStationInfoList);
        view.setThematicMapList(thematicMapList);
        return view;
    }

    /**
     * 获取模型预报公报生成条件
     *
     * @param stTime    开始时间
     * @param endTime   结束时间
     * @param situation 情况
     * @param bsId      流域id
     * @param keywords  关键字
     * @return GazetteModeForecastParamsView
     */
    @Override
    public GazetteModeForecastParamsView getGazetteModeForecastParamsView(LocalDateTime stTime, LocalDateTime endTime, String situation, Integer bsId, String keywords) {
        // 符合条件的模型预报站点列表
        List<HydroStationInfo> hydroStationInfoList;
        hydroStationInfoList = hydroStationInfoMapper.getGazetteModeForecastParams(stTime, endTime, bsId, keywords, situation);
        // 符合条件的模型预报地图服务数据列表
        List<ThematicMap> thematicMapList;
        thematicMapList = thematicMapMapper.selectListByParams("模型预报", stTime, endTime, bsId);
        GazetteModeForecastParamsView view = new GazetteModeForecastParamsView();
        view.setHydroStationInfoList(hydroStationInfoList);
        view.setThematicMapList(thematicMapList);
        return view;
    }

    /**
     * 获取特征值字典
     *
     * @return EigenvalueView
     */
    @Override
    public EigenvalueView getDicEigenvalue() {
        List<DicEigenvalue> eigenvalues = eigenvalueMapper.selectList(new LambdaQueryWrapper<>());
        List<DicEigenvalue> eigenvalueList1 = new ArrayList<>();
        List<DicEigenvalue> eigenvalueList2 = new ArrayList<>();
        List<DicEigenvalue> eigenvalueList3 = new ArrayList<>();
        //雨情特征值列表
        RainEigenvalue rainEigenvalue = new RainEigenvalue();
        //水情特征值列表
        WaterEigenvalue waterEigenvalue = new WaterEigenvalue();
        //模型预报特征值列表
        ModeEigenvalue modeEigenvalue = new ModeEigenvalue();
        for (DicEigenvalue each : eigenvalues) {
            if ("水情".equals(each.getEigenvalueType())) {
                eigenvalueList1.add(each);
            } else if ("雨情".equals(each.getEigenvalueType())) {
                eigenvalueList2.add(each);
            } else {
                eigenvalueList3.add(each);
            }
        }
        rainEigenvalue.setEigenvalues(eigenvalueList2);
        waterEigenvalue.setEigenvalues(eigenvalueList1);
        modeEigenvalue.setEigenvalues(eigenvalueList3);
        EigenvalueView view = new EigenvalueView();
        view.setRainEigenvalue(rainEigenvalue);
        view.setWaterEigenvalue(waterEigenvalue);
        view.setModeEigenvalue(modeEigenvalue);
        return view;
    }

    @Override
    public List<DicBulletinType> getGazetteType() {
        List<DicBulletinType> types = typeMapper.selectList(new LambdaQueryWrapper<>());
        return types;
    }

    @Override
    public List<DicBulletinStutus> getGazetteStutus() {
        List<DicBulletinStutus> stutuses = stutusMapper.selectList(new LambdaQueryWrapper<>());
        return stutuses;
    }


    /**
     * 审核公报
     *
     * @param bId
     * @param stutusId
     * @return Boolean
     */
    @Override
    public Boolean auditGazette(String bId, Integer stutusId) {
        int ret = 0;
        HydPiBulletinP hydPiBulletinP = hydPiBulletinPMapper.selectById(bId);
        hydPiBulletinP.setAuditStutus(stutusId);
        ret = hydPiBulletinPMapper.updateById(hydPiBulletinP);
        return ret == 1;
    }

    @Override
    public List<DicBSData> getDicBSDataList() {
        List<DicBSData> bsDataList = stutusMapper.selectbsDataList();
        return bsDataList;
    }

    /**
     * 批量下载
     *
     * @param bIds List<String>
     * @return List<String>
     */
    @Override
    public List<String> batchDownLoadGazetteByIds(String bIds) {
        String ids = bIds.replaceAll("，", ",");
        List<HydPiBulletinP> hydPiBulletinPS = hydPiBulletinPMapper.selectList(new LambdaQueryWrapper<HydPiBulletinP>()
                .apply("FIND_IN_SET(BID, '" + ids + "')"));
        List<String> urls = hydPiBulletinPS.stream().map(HydPiBulletinP::getWordUrl).collect(Collectors.toList());
        return urls;
    }


    /**
     * 根据路径删除指定的目录或文件，无论存在与否
     *
     * @param sPath 要删除的目录或文件
     * @return 删除成功返回 true，否则返回 false。
     */
    public boolean DeleteFolder(String sPath) {
        Boolean flag = false;
        File file = new File(sPath);
        // 判断目录或文件是否存在
        if (!file.exists()) {  // 不存在返回 false
            return flag;
        } else {
            // 判断是否为文件
            if (file.isFile()) {  // 为文件时调用删除文件方法
                flag = deleteFile(sPath);
            }
        }
        return flag;
    }


    /**
     * 删除单个文件
     *
     * @param sPath 被删除文件的文件名
     * @return 单个文件删除成功返回true，否则返回false
     */
    public boolean deleteFile(String sPath) {
        Boolean flag = false;
        File file = new File(sPath);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            file.delete();
            flag = true;
        }
        return flag;
    }

    /**
     * 获取时间区间的所有整点时间
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 整点时间集合
     */
    public List<String> getHourList(String startTime, String endTime) {
        List<String> hourList = new ArrayList<>();
        SimpleDateFormat format2 = new SimpleDateFormat("yyyy-MM-dd HH");
        SimpleDateFormat format3 = new SimpleDateFormat("yyyy-MM-dd HH:00:00");
        try {
            Date startTime1 = format2.parse(startTime);
            Date endTime1 = format2.parse(endTime);
            Calendar tempStart = Calendar.getInstance();
            tempStart.setTime(startTime1);
            while (endTime1.getTime() >= startTime1.getTime()) {
                hourList.add(format3.format(startTime1));
                //有多种模式可以增加或减去相应的时间
                tempStart.add(Calendar.HOUR_OF_DAY, 1);
                startTime1 = tempStart.getTime();
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return hourList;
    }

    /**
     * 拼接两个Document
     *
     * @param document1 前
     * @param document2 后
     */
    private void splicingDocument(Document document1, Document document2) {
        //在第二个文档中循环获取所有节
        for (Object sectionObj : (Iterable) document2.getSections()) {
            Section sec = (Section) sectionObj;
            //在所有节中循环获取所有子对象
            for (Object docObj : (Iterable) sec.getBody().getChildObjects()) {
                DocumentObject obj = (DocumentObject) docObj;
                //获取第一个文档的最后一节
                Section lastSection = document1.getLastSection();
                //将所有子对象添加到第一个文档的最后一节中
                Body body = lastSection.getBody();
                body.getChildObjects().add(obj.deepClone());
            }
        }
    }

    /**
     * 设置word段落格式
     *
     * @param document
     * @return
     */
    private Document setDocumentStyle(Document document) {
        //设置小标题格式
        ParagraphStyle style1 = new ParagraphStyle(document);
        style1.setName("secondTitleStyle");
        style1.getCharacterFormat().setBold(true);
        style1.getCharacterFormat().setTextColor(Color.BLACK);
        style1.getCharacterFormat().setFontName("宋体");
        style1.getCharacterFormat().setFontSize(10f);
        document.getStyles().add(style1);
        //设置正文段落的格式
        ParagraphStyle style2 = new ParagraphStyle(document);
        style2.setName("paraStyle");
        style2.getCharacterFormat().setFontName("宋体");
        style2.getCharacterFormat().setFontSize(9.5f);
        document.getStyles().add(style2);
        //设置表格标题格式
        ParagraphStyle style3 = new ParagraphStyle(document);
        style3.setName("tableStyle");
        style3.getCharacterFormat().setFontName("宋体");
        style3.getCharacterFormat().setFontSize(9f);
        document.getStyles().add(style3);
        //设置图片编码格式
        ParagraphStyle style4 = new ParagraphStyle(document);
        style4.setName("imageStyle");
        style4.getCharacterFormat().setFontName("Times New Roman");
        style4.getCharacterFormat().setFontSize(9.5f);
        document.getStyles().add(style4);
        return document;
    }

    /**
     * //获取段落
     *
     * @param stDate
     * @param endDate
     * @param title
     * @param content
     * @param thematicMapList
     * @param document
     * @param type
     * @return
     */
    private Document addSection(String stDate, String endDate, String title, String content, List<ThematicMap> thematicMapList, Document document, String type) {
        if (title != null && content != null) {
            //添加一个waterSection
            Section section = document.addSection();
            //添加两个段落至section
            Paragraph para1 = section.addParagraph();
            para1.appendText(title);
            Paragraph para2 = section.addParagraph();
            para2.appendText(content);
            //应用格式
            para1.applyStyle("secondTitleStyle");
            para2.applyStyle("paraStyle");
            //设置第一个段落的对齐方式
            para1.getFormat().setHorizontalAlignment(HorizontalAlignment.Left);
            //设置第二段和第三段的段首缩进
            para2.getFormat().setFirstLineIndent(25f);
            //设置第一段和第二段的段后间距
            para1.getFormat().setBeforeSpacing(10f);
            //设置行间距
            para2.getFormat().setLineSpacing(10f);
            if (!ObjectUtils.isEmpty(thematicMapList)) {
                for (int i = 0; i < thematicMapList.size(); i++) {
                    //从地址加载图片，并转化为Base64
//                    String image = Base64Util.imageUrlToBase64(thematicMapList.get(i).getMapUrl());
//                    String imageName = thematicMapList.get(i).getMapName();
//                    String base64 = "data:image/" + imageName + ";base64," + image;
                    if (!ObjectUtils.isEmpty(thematicMapList.get(i).getMapUrl())) {
                        String imageName = stDate + "-" + endDate + thematicMapList.get(i).getMapName() + "专题图";
                        String img = "<img src= \"" + thematicMapList.get(i).getMapUrl() + "\" alt=\"" + imageName + "\">";
                        Paragraph para4 = section.addParagraph();
                        para4.appendText(img);
                        para4.getFormat().setHorizontalAlignment(HorizontalAlignment.Center);
                        para4.applyStyle("imageStyle");
                    }
//                    Paragraph para3 = section.addParagraph();
//                    para3.appendText( stDate + "-" + endDate + imageName + "专题图");
//                    para3.getFormat().setHorizontalAlignment(HorizontalAlignment.Center);
//                    para3.applyStyle("tableStyle");
                }
            }

        }

        return document;
    }

    /**
     * 获取特征值名称对应特征值
     *
     * @param eigenvalueName
     * @param eigenvalue
     * @return
     */
    private String getGazetteEigenvalue(String eigenvalueName, GazetteEigenvalue eigenvalue, String type) {
        String value = null;
        if (!type.equals("雨情")) {
            switch (eigenvalueName) {
                case ("最大水位"):
                    value = eigenvalue.getMaxz() + "米";
                    break;
                case ("最小水位"):
                    value = eigenvalue.getMinz() + "米";
                    break;
                case ("平均水位"):
                    value = eigenvalue.getAvgz() + "米";
                    break;
                case ("最大流量"):
                    value = eigenvalue.getMaxq() + "立方米/秒";
                    break;
                case ("最小流量"):
                    value = eigenvalue.getMinq() + "立方米/秒";
                    break;
                case ("平均流量"):
                    value = eigenvalue.getAvgq() + "立方米/秒";
                    break;
                default:
            }
        } else {
            switch (eigenvalueName) {
                case ("1小时最大降雨量"):
                    value = eigenvalue.getMaxz() + "毫米";
                    break;
                case ("1小时最小降雨量"):
                    value = eigenvalue.getMinz() + "毫米";
                    break;
                case ("1小时平均降雨量"):
                    value = eigenvalue.getAvgz() + "毫米";
                    break;
                case ("10分钟最大降雨量"):
                    value = eigenvalue.getMaxq() + "毫米";
                    break;
                case ("10分钟最小降雨量"):
                    value = eigenvalue.getMinq() + "毫米";
                    break;
                case ("10分钟平均降雨量"):
                    value = eigenvalue.getAvgq() + "毫米";
                    break;
                default:
            }
        }
        return value;
    }

    /**
     * 获取二级标题
     *
     * @param secondTitleNum
     * @param type
     * @return
     */
    private String getSecondTitleNum(int secondTitleNum, String type) {
        String secondTitle = "";
        switch (secondTitleNum) {
            case (1):
                secondTitle = "一、" + type;
                break;
            case (2):
                secondTitle = "二、" + type;
                break;
            case (3):
                secondTitle = "三、" + type;
                break;
            default:
        }
        return secondTitle;
    }

    public static String readTextFromFile(String fileName) {
        StringBuffer sb = new StringBuffer();
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(fileName));
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
        String content;
        while (true) {
            try {
                if (!((content = br.readLine()) != null)) {
                    break;
                }
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            sb.append(content);
        }
        return sb.toString();
    }

}
