package com.cuit.basin.base.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.basin.base.biz.dao.*;
import com.cuit.basin.base.biz.entity.HydFiFittedD;
import com.cuit.basin.base.biz.entity.HydSiForeaccuS;
import com.cuit.basin.base.biz.pojo.*;
import com.cuit.basin.base.biz.service.HydrologicalCorrectionService;
import com.cuit.basin.base.biz.vo.HydroForeCurrDataView;
import com.cuit.basin.base.biz.vo.HydroForeCurrReferView;
import com.cuit.basin.base.biz.vo.ManualCorrectionCurveView;
import com.cuit.basin.base.biz.vo.ModuleFittedDataView;
import com.cuit.basin.framework.common.exception.ParamsException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

@Service
public class HydrologicalCorrectionServiceImpl implements HydrologicalCorrectionService {
    @Autowired
    private HydFiStdataFMapper hydFiStdataFMapper;
    @Autowired
    private HydSiForeaccuSMapper hydSiForeaccuSMapper;
    @Autowired
    private HydroStationInfoMapper stationInfoMapper;
    @Autowired
    private HydSiForeaccuSMapper foreaccuSMapper;
    @Autowired
    private HydFiFittedDMapper hydFiFittedDMapper;
    @Autowired
    private HydroStationDataMapper stationDataMapper;


    /**
     * 查询模型参考曲线
     *
     * @param stationNo  站号
     * @param validTime  预报时效
     * @param timePeriod 时间周期
     * @return HydroForeCurrReferView
     */
    @Override
    public HydroForeCurrReferView queryForecastReferenceCurve(String stationNo, Integer validTime, Integer timePeriod) {
        HydroForeCurrReferView view = new HydroForeCurrReferView();
        List<ModelForeReferData> modelForeReferDataList = new ArrayList<>();
        List<HydroStationForeDataInfo> hydroStationForeDataInfoList1 = new ArrayList<>();
        List<HydroStationForeDataInfo> hydroStationForeDataInfoList2 = new ArrayList<>();
        //获取精度评估数据
        List<HydSiForeaccuS> hydSiForeaccuSList = foreaccuSMapper.selectList(new LambdaQueryWrapper<HydSiForeaccuS>()
                .eq(HydSiForeaccuS::getSTCD, stationNo)
                .eq(HydSiForeaccuS::getDataTime, "2022-11-17 00:00:00"));
        //查询水文站点预测数据列表-前半段
        List<HydroStationForeDataInfo> hydroStationForeDataInfos1 = hydFiStdataFMapper.selectHydroStationForeDataInfoH(stationNo, validTime, timePeriod);
        //查询水文站点预测数据列表-后半段
        List<HydroStationForeDataInfo> hydroStationForeDataInfos2 = hydFiStdataFMapper.selectHydroStationForeDataInfoF(stationNo, timePeriod);
        //拼接返回类
        for (HydSiForeaccuS each : hydSiForeaccuSList) {
            ModelForeReferData modelForeReferData = ModelForeReferData.builder().mid(each.getMID()).stcd(each.getSTCD())
                    .nse(each.getNSE()).r2(each.getR2()).factor(each.getFACTOR()).build();
            for (HydroStationForeDataInfo each1 : hydroStationForeDataInfos1) {
                if (each1.getMid().equals(each.getMID())) {
                    hydroStationForeDataInfoList1.add(each1);
                }
            }
            for (HydroStationForeDataInfo each2 : hydroStationForeDataInfos2) {
                if (each2.getMid().equals(each.getMID())) {
                    hydroStationForeDataInfoList2.add(each2);
                }
            }
            modelForeReferData.setHydroStationForeDataInfoList1(hydroStationForeDataInfoList1);
            modelForeReferData.setHydroStationForeDataInfoList2(hydroStationForeDataInfoList2);
            modelForeReferDataList.add(modelForeReferData);
        }
        view.setModelForeReferDataList(modelForeReferDataList);
        view.setHisForeAccuracy(hydSiForeaccuSList.stream().mapToDouble(HydSiForeaccuS::getFACTOR).max().getAsDouble());
        return view;
    }

    /**
     * 预报订正曲线初始化
     *
     * @param stationNo   站号
     * @param correctType 订正类型(快速订正/公式订正)
     * @param correctStr  订正参数(最大、最小、平均/公式字符)
     * @return HydroForeCurrDataView
     */
    @Override
    public HydroForeCurrDataView queryHydrologicalCorrectionByParam(String stationNo, String correctType, String correctStr) {
        HydroForeCurrDataView view = new HydroForeCurrDataView();
        List<CurrData> currDataList = new ArrayList<>();

        if ("快速订正".equals(correctType)) {
            //获取快速订正后的数据
            currDataList = getCurrDataListByFastCorrect(correctStr, stationNo);
        } else {
            //获取公式订正后的数据
            currDataList = getCurrDataListByFormulaCorrect(correctStr, stationNo);
        }
        view.setCurrDataList(currDataList);
        return view;
    }

    /**
     * 获取公式订正后的数据
     *
     * @param formulaCorrect 公式订正
     * @param stationNo      站号
     * @return
     */
    private List<CurrData> getCurrDataListByFormulaCorrect(String formulaCorrect, String stationNo) {
        List<CurrData> currDataList = new ArrayList<>();
        List<HydSiForeaccuS> hydSiForeaccuSList = hydSiForeaccuSMapper.selectByNewDateTime(stationNo);

        //将字符串根据模型拆分
        String[] str = formulaCorrect.split("\\+");
        List<FormulaAfterSplit> formulaAfterSplits = new ArrayList<>();
        for (String s : str) {
            String[] mStr = s.split(",");
            FormulaAfterSplit formulaAfterSplit = FormulaAfterSplit.builder().mid(Integer.valueOf(mStr[0]))
                    .weight(Double.valueOf(mStr[1])).build();
            formulaAfterSplits.add(formulaAfterSplit);
        }
        //查询改起报时间的所有预报数据
        List<HydFiFittedDInfo> hydFiFittedDList = new ArrayList<>();
        for (HydSiForeaccuS hydSiForeaccuS : hydSiForeaccuSList) {
            for (FormulaAfterSplit each2 : formulaAfterSplits) {
                if (hydSiForeaccuS.getMID().equals(each2.getMid())) {
                    List<HydFiFittedDInfo> hydFiFittedDS = hydFiFittedDMapper.selectHydFiFittedDInfoList(hydSiForeaccuS.getID());
                    hydFiFittedDList.addAll(hydFiFittedDS);
                }
            }

        }
        for (HydFiFittedDInfo each1 : hydFiFittedDList) {
            for (FormulaAfterSplit each2 : formulaAfterSplits) {
                if (each2.getMid().equals(each1.getMid())) {
                    each1.setZ(each1.getZ().multiply(BigDecimal.valueOf(each2.getWeight())));
                    each1.setQ(each1.getQ().multiply(BigDecimal.valueOf(each2.getWeight())));
                }
            }
        }
        //根据dt字段进行分组后，对应值累加
        Map<LocalDateTime, HydFiFittedDInfo> groupByDt = hydFiFittedDList.stream()
                .collect(Collectors.toMap(HydFiFittedDInfo::getDt,
                        Function.identity(),
                        (n, o) -> {
                            n.setZ(n.getZ().add(o.getZ()));
                            n.setQ(n.getQ().add(o.getQ()));
                            return n;
                        }));
        groupByDt.forEach((k, v) -> {
            CurrData currData = CurrData.builder().dateTime(hydSiForeaccuSList.get(0).getDataTime()).foretm(k)
                    .currz(v.getZ()).currq(v.getQ()).build();
            currDataList.add(currData);
        });

        return currDataList.stream().sorted(Comparator.comparing(CurrData::getForetm, Comparator.nullsLast(LocalDateTime::compareTo))).collect(toList());
    }

    /**
     * 获取快速订正后的数据
     *
     * @param fastCorrect 快速订正
     * @param stationNo   站号
     * @return
     */
    private List<CurrData> getCurrDataListByFastCorrect(String fastCorrect, String stationNo) {
        List<CurrData> currDataList = new ArrayList<>();
        //获取统计数据
        List<FastCorrectData> fastCorrectData = hydFiFittedDMapper.queryfastCorrectData(stationNo);
        CurrData currData = new CurrData();
        for (FastCorrectData each : fastCorrectData) {
            switch (fastCorrect) {
                case ("平均"):
                    currData = CurrData.builder().dateTime(each.getDateTime()).foretm(each.getDt())
                            .currz(each.getAvgFZ()).currq(each.getAvgFQ()).build();
                    break;
                case ("最大"):
                    currData = CurrData.builder().dateTime(each.getDateTime()).foretm(each.getDt())
                            .currz(each.getMaxFZ()).currq(each.getMaxFQ()).build();
                    break;
                case ("最小"):
                    currData = CurrData.builder().dateTime(each.getDateTime()).foretm(each.getDt())
                            .currz(each.getMinFZ()).currq(each.getMinFQ()).build();
                    break;
                default:
                    throw new ParamsException("请输入正确的类型，类型有：" + " <平均><最大><最小>");
            }
            currDataList.add(currData);
        }
        return currDataList;
    }

    /**
     * 保存手动订正预报曲线
     *
     * @param view 订正数据
     * @return Boolean
     */
    @Override
    public Boolean saveManualCorrectionForecastCurve(ManualCorrectionCurveView view) {
        int ref = 0;
//        //获取对应站点的标准信息
//        QueryWrapper<HydroStationInfo> queryWrapper = new QueryWrapper<>();
//        HydroStationInfo info = stationInfoMapper.selectOne(queryWrapper.eq("STCD", view.getStationNo()));
        //用订正数据与标准信息计算
        List<HydFiFittedD> hydFiFittedDList = new ArrayList<>();
        //查询是否有改时刻，的精度评估记录
        HydSiForeaccuS hydSiForeaccuSQuery = hydSiForeaccuSMapper.selectByMultiField(view.getStationNo(), 4, view.getCurrDataList()
                .get(0).getDateTime());
        HydSiForeaccuS hydSiForeaccuS = new HydSiForeaccuS();
        if (ObjectUtils.isEmpty(hydSiForeaccuSQuery)) {
            //新增手动订正精度记录
            hydSiForeaccuS = HydSiForeaccuS.builder().STCD(view.getStationNo()).MID(4).dataTime(view.getCurrDataList()
                            .get(0).getDateTime()).STDataTime(view.getCurrDataList().get(0).getDateTime())
                    .EDDataTime(view.getCurrDataList().get(view.getCurrDataList().size() - 1).getForetm())
                    .NSE(null)
                    .r2(null)
                    .FACTOR(null)
                    .build();
            hydSiForeaccuSMapper.insert(hydSiForeaccuS);
            //新增数据
            for (int i = 0; i < view.getCurrDataList().size(); i++) {
                HydFiFittedD hydFiFittedD = HydFiFittedD.builder().foreaccuId(hydSiForeaccuS.getID())
                        .dt(view.getCurrDataList().get(i).getForetm())
                        .z(view.getCurrDataList().get(i).getCurrz())
                        .q(view.getCurrDataList().get(i).getCurrq()).build();
                hydFiFittedDList.add(hydFiFittedD);
            }
            //数据入库
            ref = hydFiFittedDMapper.insertBatch(hydFiFittedDList);
        } else {
            hydSiForeaccuS = hydSiForeaccuSQuery;
            //数据更新
            List<HydFiFittedD> hydFiFittedDListBefore = hydFiFittedDMapper.selectList(new LambdaQueryWrapper<HydFiFittedD>()
                    .eq(HydFiFittedD::getForeaccuId, hydSiForeaccuS.getID()));
            List<HydFiFittedD> hydFiFittedDList1 = new ArrayList<>();
            for (int i = 0; i < view.getCurrDataList().size(); i++) {
                HydFiFittedD hydFiFittedD = HydFiFittedD.builder().foreaccuId(hydSiForeaccuS.getID())
                        .dt(view.getCurrDataList().get(i).getForetm())
                        .z(view.getCurrDataList().get(i).getCurrz())
                        .q(view.getCurrDataList().get(i).getCurrq()).build();
                for (HydFiFittedD each : hydFiFittedDListBefore) {
                    if (each.getForeaccuId().equals(hydFiFittedD.getForeaccuId()) && each.getDt().equals(hydFiFittedD.getDt())) {
                        each.setForeaccuId(hydFiFittedD.getForeaccuId());
                        each.setDt(hydFiFittedD.getDt());
                        each.setZ(hydFiFittedD.getZ());
                        each.setQ(hydFiFittedD.getQ());
                        hydFiFittedDList.add(each);
                    }
                }

            }
            //数据更新
            ref = hydFiFittedDMapper.updateBatch(hydFiFittedDList);
        }
        return (ref == 1);
    }

    /**
     * 查询模型拟合曲线
     *
     * @param stationNo  站号
     * @param timePeriod 模型id
     * @return List<ModuleFittedDataView>
     */
    @Override
    public ModuleFittedDataView queryModuleFittedDataCurve(String stationNo, Integer timePeriod) {
        ModuleFittedDataView view = new ModuleFittedDataView();
        List<HydSiForeaccuS> hydSiForeaccuSList = hydSiForeaccuSMapper.selectByNewDateTime(stationNo);
        //获取实况数据
        List<RealDataInfo> realDataInfoList = new ArrayList<>();
        realDataInfoList = stationDataMapper.selectRealDataInfoList(stationNo, hydSiForeaccuSList.get(0).getDataTime(), timePeriod);
        //获取模型拟合数据
        List<ModuleFittedDataInfo> moduleFittedDataInfos = new ArrayList<>();
        if (!ObjectUtils.isEmpty(hydSiForeaccuSList)) {
            for (HydSiForeaccuS each : hydSiForeaccuSList) {
                List<HydFiFittedD> hydFiFittedDS = hydFiFittedDMapper.selectModuleFittedList(each.getDataTime(), each.getID(), timePeriod);
                //排序
                List<HydFiFittedD> list = hydFiFittedDS.stream().sorted(Comparator.comparing(HydFiFittedD::getDt, Comparator.nullsLast(LocalDateTime::compareTo))).collect(toList());
                ModuleFittedDataInfo moduleFittedDataInfo = ModuleFittedDataInfo.builder().dataTime(each.getDataTime()).stcd(each.getSTCD()).mid(each.getMID()).nse(each.getNSE())
                        .r2(each.getR2()).factor(each.getFACTOR()).hydFiFittedDList(list).build();
                moduleFittedDataInfos.add(moduleFittedDataInfo);
            }
        }
        //拼接返回类
        view = ModuleFittedDataView.builder().realDataInfoList(realDataInfoList).moduleFittedDataInfoList(moduleFittedDataInfos).build();
        return view;
    }
}
