package com.cuit.basin.base.biz.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.basin.base.biz.dao.*;
import com.cuit.basin.base.biz.entity.*;
import com.cuit.basin.base.biz.pojo.HydroForecastDetail;
import com.cuit.basin.base.biz.pojo.HydroStationForeDataInfo;
import com.cuit.basin.base.biz.pojo.ModelForeData;
import com.cuit.basin.base.biz.query.HydrologicalForecastListQuery;
import com.cuit.basin.base.biz.query.HydrologicalForecastQuery;
import com.cuit.basin.base.biz.query.RiverCrossSectionQuery;
import com.cuit.basin.base.biz.service.HydrologicalForecastService;
import com.cuit.basin.base.biz.vo.HydroForeSectionDetailView;
import com.cuit.basin.base.biz.vo.HydroForecastDetailView;
import com.cuit.basin.base.biz.vo.HydroStationForeView;
import com.cuit.basin.base.biz.vo.SingleStationForeDataView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class HydrologicalForecastServiceIpml implements HydrologicalForecastService {

    @Resource
    private HydroStationHistoryMapper hydroStationHistoryMapper;
    @Resource
    private HydroStationInfoMapper hydroStationInfoMapper;
    @Resource
    private SectionDataMapper sectionDataMapper;
    @Autowired
    private HydFiStdataFMapper hydFiStdataFMapper;
    @Autowired
    private ModelDicMapper modelDicMapper;

    /**
     * 条件查询水文预测列表信息
     *
     * @param query 查询参数类
     * @return HydroStationForeView
     */
    @Override
    public HydroStationForeView queryHydrologicalForecastListByParam(HydrologicalForecastListQuery query) {
        /**
         * 查询
         */
        List<HydroStationForeDataInfo> hydroStationForeDataInfos = hydFiStdataFMapper.queryHydrologicalForecastListByParam(query);
        /**
         * 赋值返回
         */
        return HydroStationForeView.builder().hydroStationForeDataInfoList(hydroStationForeDataInfos).build();
    }


    /**
     * @author: llw
     * @date: 2022/11/17 23:34
     * @return: HydroForeSectionDetailViewList
     * @description:2. 点击展开站点详情
     */
    @Override
    public SingleStationForeDataView queryHydroSingleStationModelForeInfo(String stcd, LocalDateTime dataTime, Integer time) {
        /**
         * 查询
         */
        List<ModelForeData> modelForeDataList = hydFiStdataFMapper.queryHydroSingleStationModelForeInfo(stcd, dataTime, time);
        /**
         * 赋值返回
         */
        return SingleStationForeDataView.builder().modelForeDataList(modelForeDataList).build();
    }


    /**
     * @author: llw
     * @date: 2022/11/17 23:34
     * @return: hydroForecastDetailView
     * @description: 3.条件查询站点水文预报详情
     */
    @Override
    public HydroForecastDetailView queryHydrologicalForecastDetails(HydrologicalForecastQuery query) {
        LambdaQueryWrapper<HydFiStdataF> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        /**
         * 设置查询条件
         */
        lambdaQueryWrapper.eq(HydFiStdataF::getDt, query.getDataTime()).eq(HydFiStdataF::getStcd, query.getStationNum());
        /**
         * 查询预报数据列表
         */
        List<HydFiStdataF> hydFiStdataFS = hydFiStdataFMapper.selectList(lambdaQueryWrapper);
        List<HydroForecastDetail> hydroForecastDetails = new LinkedList<>();
        /**
         * 按模型把数据分组
         */
        Map<Integer, List<HydFiStdataF>> collect1 = hydFiStdataFS.stream().collect(Collectors.groupingBy(HydFiStdataF::getMid));

        /**
         * 封装List<HydroForecastDetail>对象
         */
        for (Integer key : collect1.keySet()) {
            LambdaQueryWrapper<ModelDic> lambdaQueryWrapper1 = new LambdaQueryWrapper<>();
            lambdaQueryWrapper1.eq(ModelDic::getMid, key);
            ModelDic modelDic = modelDicMapper.selectOne(lambdaQueryWrapper1);
            hydroForecastDetails.add(HydroForecastDetail.builder().mid(key).mnm(modelDic.getMname()).hydroStationForeDataInfoList(collect1.get(key)).build());
        }
        /**
         * 将封装的List<HydroForecastDetail>对象赋值返回类并返回
         */
        return HydroForecastDetailView.builder().hydroForecastDetailList(hydroForecastDetails).build();
    }

    /**
     * @author: llw
     * @date: 2022/11/17 23:36
     * @return:
     * @description: 条件查询河流断面详情
     */
    @Override
    public HydroForeSectionDetailView queryRiverCrossSectionForeDetails(RiverCrossSectionQuery query) {
        /**
         * 查询站点历史统计
         */
        HydroStationHistory history = hydroStationHistoryMapper.selectOne(new LambdaQueryWrapper<HydroStationHistory>()
                .eq(HydroStationHistory::getStcd, query.getStationNum()));
        /**
         * 查询站点基础信息
         */
        HydroStationInfo stationInfo = hydroStationInfoMapper.selectOne(new LambdaQueryWrapper<HydroStationInfo>()
                .eq(HydroStationInfo::getStcd, query.getStationNum()));
        /**
         * 查询站点断面数据列表
         */
        List<SectionData> sectionDataList = sectionDataMapper.selectList(new LambdaQueryWrapper<SectionData>()
                .eq(SectionData::getStcd, query.getStationNum()));
        /**
         * 设置查询条件
         */
        LambdaQueryWrapper<HydFiStdataF> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        /**
         * 查询预报数据列表
         */
        List<HydFiStdataF> modelForeDataList = hydFiStdataFMapper.selectList(lambdaQueryWrapper.eq(HydFiStdataF::getMid, query.getModelId()).eq(HydFiStdataF::getDt, query.getDataTime())
                .eq(HydFiStdataF::getStcd, query.getStationNum()));
        /**
         * 给返回类赋值并返回
         */
        return HydroForeSectionDetailView.builder()
                .stationInfo(stationInfo)
                .hydroStationHistory(history)
                .sectionDataList(sectionDataList)
                .hydroStationForeDataInfoList(modelForeDataList).build();
    }
}

