package com.cuit.basin.base.biz.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.cuit.basin.base.biz.dao.LoginUserMapper;
import com.cuit.basin.base.biz.entity.sys.AppidToKen;
import com.cuit.basin.base.biz.entity.sys.LoginUser;
import com.cuit.basin.base.biz.entity.sys.User;
import com.cuit.basin.base.biz.service.LoginUserService;
import com.cuit.basin.framework.common.constant.Constant;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.utils.other.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;


/**
 * @author wzw
 */
@Service
@Slf4j
public class LoginUserServiceImpl implements LoginUserService {

    Md5TokenGenerator tokenGenerator = new Md5TokenGenerator();
    @Resource
    private LoginUserMapper userMapper;

    private RedisUtil redisUtil = new RedisUtil();

    /**
     * 获取app id的token
     *
     * @param appid
     * @return
     */
    @Override
    public String getAppIdToken(String appid) {
        AppidToKen appidToKen = userMapper.getAppidToKen(appid);
        if (appidToKen != null) {
            String token = tokenGenerator.generate(appid, appidToKen.getTk());

            try {
                //存入redis
                redisUtil.set(appid, token);
                redisUtil.expire(appid, token, Constant.TOKEN_EXPIRE_TIME);
                redisUtil.set(token, appid);
                redisUtil.expire(token, appid, Constant.TOKEN_EXPIRE_TIME);
                Long currentTime = System.currentTimeMillis();
                redisUtil.set(token + appid, currentTime.toString());
            } catch (Exception e) {
                log.error("取token时Redis错误：" + e.getMessage());
                throw new ErrorMessage("服务器故障！");
            }
            return token;
        } else {
            //应用不存在
            throw new ErrorMessage("拒绝访问！");
        }
    }

    /**
     * 根据账号和appid 获取token
     *
     * @param appid
     * @param account
     * @return
     */
    @Override
    public String getAppIdToken(String appid, String account) {
        AppidToKen appidToKen = userMapper.getAppidToKen(appid);
        if (appidToKen != null) {
            String token = tokenGenerator.generate(appid, account, appidToKen.getTk());
            try {
                //存入redis
                redisUtil.set(appid, token);
                redisUtil.set(token + "ACCOUNT", account);
                redisUtil.expire(appid, token, Constant.TOKEN_EXPIRE_TIME);
                redisUtil.set(token, appid);
                redisUtil.expire(token, appid, Constant.TOKEN_EXPIRE_TIME);
                Long currentTime = System.currentTimeMillis();
                redisUtil.set(token + appid, currentTime.toString());
            } catch (Exception e) {
                log.error("取token时Redis错误：" + e.getMessage());
                throw new ErrorMessage("服务器故障！");
            }
            return token;
        } else {
            //应用不存在
            throw new ErrorMessage("拒绝访问！");
        }
    }


    /**
     * 获取缓存中的所有用户信息
     *
     * @return
     */
    @Override
    public List<LoginUser> getAll() {
        //从redis中取
        List<LoginUser> users = (List<LoginUser>) redisUtil.get("users");
        //判断是否有数据
        if (null == users) {
            //从数据库中读取
            users = userMapper.getAll();
            //存入redis
            redisUtil.set("users", users);
        }
        return users;
    }

    @Override
    public User checkUser(String longinname, String password) {
        String retcode = "200";
        User user = null;
        //从数据库中读取
        LoginUser longinuser = userMapper.getUserByLoginName(longinname);
        if (longinuser != null) {
            if (longinuser.getLogerrnum() <= Constant.LOGINERRORMAXNUM) {
                MD5 md = new MD5();
                if (longinuser.getPassword().equalsIgnoreCase(md.encryption(password))) {
                    if (longinuser.getFlag() == 0) {
                        if (DateUtil.getDaysDifference(longinuser.getPasschangdate(), new Date()) < Constant.CHANGPASSWORDDAYS) {
                            //清空错误次数
                            user = new User();
                            user.setLoginname(longinuser.getLoginname());
                            user.setDptName(userMapper.getDptId(longinuser.getDptId()));
                            user.setPermissionLevelName(userMapper.getPermissionName(longinuser.getPermissionId()));
                            user.setPositionName(userMapper.getPositionId(longinuser.getPositionId()));
                            user.setName(longinuser.getName());
                            user.setTel(longinuser.getTel());
                            userMapper.clearLoginError(longinname);
                        } else {
                            throw new ErrorMessage("请尽快修改密码，密码已超过有效期！");
                        }
                    } else {
                        throw new ErrorMessage("用户已被锁定！");
                    }
                } else {
                    //错误次数加1
                    userMapper.setLoginError(longinname);
                    throw new ErrorMessage("用户密码错误！");
                }
            } else {
                //错误次数超过5次
                throw new ErrorMessage("用户密码超过" + Constant.LOGINERRORMAXNUM + "次！");
            }
        } else {
            //用户不存在
            throw new ErrorMessage("用户不存在！");
        }
        return user;
    }

    @Override
    public boolean setUserPassword(String loginname, String password) {
        //密码校验
        // 判断长度
        AssertUtil.isTrue(PwdCheckUtil.checkPasswordLength(password, "8", null), "密码位数至少为8位！");
        // 判断密码是否包含数字、字母（不区分大小写）、特殊字符
        AssertUtil.isTrue(PwdCheckUtil.checkContainDigit(password) || PwdCheckUtil.checkContainSpecialChar(password) || PwdCheckUtil.checkContainCase(password)
                , "密码必须包含数字、字母（不区分大小写）及特殊字符！");

        MD5 md = new MD5();
        String pass = md.encryption(password);
        int ret = userMapper.setUserPassword(loginname, pass);
        return ret == 1;
    }

    @Override
    public boolean deleteUser(String loginname) {
        int ret = userMapper.deleteUser(loginname);
        return ret == 1;
    }

    @Override
    public boolean lockUser(String loginname, boolean lockflag) {
        if (lockflag) {
            return userMapper.setUserLock(loginname) == 1;
        } else {
            return userMapper.setUserUnLock(loginname) == 1;
        }
    }

    @Override
    public boolean addUser(User user) {
        LoginUser loginUser = new LoginUser();
        //判断账户是否存在
        AssertUtil.isTrue(userMapper.getUserByLoginName(user.getLoginname()) != null, "用户名已存在，请重新输入");
        loginUser.setLoginname(user.getLoginname());
        loginUser.setName(user.getName());
        loginUser.setDptId(userMapper.queryDptIdByName(user.getDptName()));
        loginUser.setPositionId(userMapper.queryPositionIdByName(user.getPositionName()));
        loginUser.setPermissionId(userMapper.queryPermissionIdByName(user.getPermissionLevelName()));
        loginUser.setTel(user.getTel());
        MD5 md = new MD5();
        String pass = md.encryption(user.getPassword());
        loginUser.setPassword(pass);
        loginUser.setFlag(0);
        loginUser.setLogerrnum(0);
        loginUser.setPasschangdate(new Date());
        int ret = 0;
        try {
            ret = userMapper.addLoginUser(loginUser);
        } catch (DuplicateKeyException e) {
            log.error("插入用户时出错，有相同的登录名，用户信息" + loginUser.toString() + "错误信息：" + e.getMessage(), e);
            throw new ErrorMessage("有相同的登录名！");
        } catch (Exception e) {
            log.error("插入用户时出错！" + e.getMessage(), e);
            throw new ErrorMessage("插入用户时出错！");
        }
        return (ret == 1);
    }

    @Override
    public boolean isValidToken(String token) {
        if (!StringUtils.isEmpty(token)) {
            try {
                String appid = "";

                //根据token从Redis中去找有没有该用户Appid
                appid = (String) redisUtil.get(token);

                if (!StringUtils.isEmpty(appid)) {
                    //表示token有效
                    log.debug("根据token查找到有效APPID {}", appid);
                    Long tokeBirthTime = Long.valueOf((String) redisUtil.get(token + appid));
                    log.debug("token产生的时间是: {}", tokeBirthTime);
                    Long diff = System.currentTimeMillis() - tokeBirthTime;
                    log.debug("token已生成了: {} ms", diff);
                    if (diff > Constant.TOKEN_RESET_TIME) {
                        redisUtil.expire(appid, token, Constant.TOKEN_EXPIRE_TIME);
                        redisUtil.expire(token, appid, Constant.TOKEN_EXPIRE_TIME);
                        log.debug("重置token时间成功！");
                        Long newBirthTime = System.currentTimeMillis();
                        redisUtil.set(token + appid, newBirthTime.toString());
                    }
                    return true;
                } else {
                    //表示token无效
                    return false;
                }
            } catch (Exception e) {
                throw new ErrorMessage("校验token时故障！");
            }
        } else {
            return false;
        }
    }

    @Override
    public boolean updateUser(User user) {
        LoginUser loginUser = new LoginUser();
        loginUser.setLoginname(user.getLoginname());
        loginUser.setName(user.getName());
        loginUser.setDptId(userMapper.queryDptIdByName(user.getDptName()));
        loginUser.setPositionId(userMapper.queryPositionIdByName(user.getPositionName()));
        loginUser.setPermissionId(userMapper.queryPermissionIdByName(user.getPermissionLevelName()));
        loginUser.setTel(user.getTel());
        MD5 md = new MD5();
        String pass = md.encryption(user.getPassword());
        loginUser.setPassword(pass);
        loginUser.setPassword(pass);
        loginUser.setFlag(0);
        loginUser.setLogerrnum(0);
        loginUser.setPasschangdate(new Date());
        int ret = 0;
        try {
            ret = userMapper.updateLoginUser(loginUser);
        } catch (Exception e) {
            log.error("更新用户时出错！" + e.getMessage(), e);
            throw new ErrorMessage("更新用户时出错！");
        }
        return (ret == 1);
    }


    public boolean deleteAppidToken(String token) {
        try {
            String appid = (String) redisUtil.get(token);
            redisUtil.remove(token + "ACCOUNT");
            redisUtil.remove(token);
            redisUtil.remove(token + appid);
            return true;
        } catch (Exception e) {
            throw new ErrorMessage("服务器故障！");
        }
    }

    @Override
    public boolean loginOut(String token) {
        boolean b = deleteAppidToken(token);
        return b;
    }
}
