package com.cuit.basin.base.biz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.basin.base.biz.dao.HydSiForeaccuSMapper;
import com.cuit.basin.base.biz.dao.ModelDicMapper;
import com.cuit.basin.base.biz.dao.ModelLogMapper;
import com.cuit.basin.base.biz.dao.ModelParamMapper;
import com.cuit.basin.base.biz.entity.HydSiForeaccuS;
import com.cuit.basin.base.biz.entity.ModelDic;
import com.cuit.basin.base.biz.entity.ModelLog;
import com.cuit.basin.base.biz.entity.ModelParam;
import com.cuit.basin.base.biz.pojo.GBHMCalibration;
import com.cuit.basin.base.biz.service.ModelManagerService;
import com.cuit.basin.base.biz.vo.CronHourMinView;
import com.cuit.basin.base.biz.vo.ModelDicView;
import com.cuit.basin.base.biz.vo.ModelLogView;
import com.cuit.basin.base.biz.vo.ModelParamView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.ret.PageVo;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 模型管理 Service
 *
 * @author Xiao Yinrui
 */
@Slf4j
@Service
public class ModelManagerServiceImpl implements ModelManagerService {

    @Resource
    private ModelDicMapper modelDicMapper;
    @Resource
    private ModelLogMapper modelLogMapper;
    @Resource
    private ModelParamMapper modelParamMapper;
    @Resource
    private HydSiForeaccuSMapper hydSiForeaccuSMapper;

    /**
     * 查询模型列表信息
     *
     * @param status 运行状态
     * @return ModelDicView
     */
    @Override
    public ModelDicView getModelDicList(Integer status) {
        List<ModelDic> modelDic = modelDicMapper.selectList(new LambdaQueryWrapper<ModelDic>()
                .eq(!ObjectUtils.isEmpty(status), ModelDic::getStatus, status));
        return ModelDicView.builder().modelDicList(modelDic).build();
    }

    /**
     * 新增模型基础信息
     *
     * @param mname  模型名
     * @param mdes   模型描述
     * @param path   模型路径
     * @param dfproc 数据预报处理
     * @param mfdata 模型前置处理
     * @param mprog 模型程序
     * @param mfprog 模型前置程序
     * @param daproc 数据后处理
     * @return boolean
     */
    @Override
    public boolean addModelInfo(String mname, String mdes, String path, String dfproc, String mfdata, String mprog, String mfprog, String daproc) {
        return modelDicMapper.insert(ModelDic.builder()
                .mname(mname).mdes(mdes).path(path)
                .dfproc(dfproc).mfdata(mfdata).mprog(mprog).mfprog(mfprog).daproc(daproc).status(2)
                .build()) > 0;
    }

    /**
     * 编辑模型基础信息
     *
     * @param mid    模型id
     * @param mname  模型名
     * @param mdes   模型描述
     * @param path   模型路径
     * @param dfproc 数据预报处理
     * @param mfdata 模型前置处理
     * @param mprog 模型程序
     * @param mfprog 模型前置程序
     * @param daproc 数据后处理
     * @return boolean
     */
    @Override
    public boolean editModelInfo(Integer mid, String mname, String mdes, String path, String dfproc, String mfdata, String mprog, String mfprog, String daproc) {
        return modelDicMapper.updateById(ModelDic.builder()
                .mid(mid).mname(mname).mdes(mdes).path(path)
                .dfproc(dfproc).mfdata(mfdata).mprog(mprog).mfprog(mfprog).daproc(daproc)
                .build()) > 0;
    }

    /**
     * 批量删除模型信息
     *
     * @param mids 模型id字符串
     * @return boolean
     */
    @Override
    public boolean deleteModelInfo(String mids) {
        return modelDicMapper.deleteBatchIds(Arrays.stream(mids.replaceAll("，", ",").split(",")).collect(Collectors.toList())) > 0;
    }

    /**
     * 手动启动某一个模型
     *
     * @param mid 模型id
     * @return boolean
     */
    @Override
    public boolean startModelByMid(Integer mid) {
        return true;
    }

    /**
     * 停止某一个模型
     *
     * @param mid 模型uid
     * @return boolean
     */
    @Override
    public boolean stopModelByMid(Integer mid) {
        return true;
    }

    /**
     * 查询模型定时运行参数
     *
     * @param mid 模型id
     * @return CronHourMinView
     */
    @Override
    public CronHourMinView getModelScheduler(Integer mid) {
        CronHourMinView retCron = null;
        String cronString = modelDicMapper.selectById(mid).getCron();
        if (ObjectUtils.isEmpty(cronString)) {
            retCron = CronHourMinView.builder().type(-1).build();
        } else {
            if (cronString.contains("/")) {
                // 周期 格式例如：0 5/10 18/1 * * ?
                int firstBlank = cronString.indexOf(" ");
                int firstSlash = cronString.indexOf("/");
                int secondBlank = cronString.indexOf(" ", firstSlash + 1);
                int secondSlash = cronString.indexOf("/", secondBlank + 1);
                int thirdBlank = cronString.indexOf(" ", secondSlash + 1);

                String minute = cronString.substring(firstBlank + 1, firstSlash);
                String periodMin = cronString.substring(firstSlash + 1, secondBlank);
                String hour = cronString.substring(secondBlank + 1, secondSlash);
                String periodHour = cronString.substring(secondSlash + 1, thirdBlank);
                log.info("分 周期 小时 周期：" + minute + " " + periodMin + " " + hour + " " + periodHour);

                retCron = CronHourMinView.builder()
                        .mid(mid).type(1).hours(hour).minutes(minute).period(Integer.valueOf(periodMin))
                        .build();
            } else {
                //指定 格式例如：0 12 1,2 * * ?
                int firstBlank = cronString.indexOf(" ");
                int secondBlank = cronString.indexOf(" ", firstBlank + 1);
                int thirdBlank = cronString.indexOf(" ", secondBlank + 1);

                String minutes = cronString.substring(firstBlank + 1, secondBlank);
                String hours = cronString.substring(secondBlank + 1, thirdBlank);
                log.info("分 小时：" + minutes + " " + hours);

                retCron = CronHourMinView.builder()
                        .mid(mid).type(2).hours(hours).minutes(minutes).build();
            }
        }

        return retCron;
    }

    /**
     * 设置模型定时运行参数
     *
     * @param cronV 定时类
     * @return String
     */
    @Override
    public String scheduleModel(CronHourMinView cronV) {
        String cronStrStart = "0 ", cronStrEnd = " * * ? ", cronString = "", hour = "", minute = "";
        if (cronV.getType() == -1) {
            cronString = null;
        } else if (cronV.getType() == 1) {
            // 周期 格式例如：0 5/10 18/1 * * ?
            minute = cronV.getMinutes() + "/" + cronV.getPeriod();
            hour = cronV.getHours() + "/1";
            cronString = cronStrStart + minute + " " + hour + cronStrEnd;
        } else if (cronV.getType() == 2){
            //指定 格式例如：0 12 1,2 * * ?
            minute = cronV.getMinutes().replaceAll("，", ",");
            hour = cronV.getHours().replaceAll("，", ",");
            cronString = cronStrStart + minute + " " + hour + cronStrEnd;
        }
        log.info("新的cron表达式为：" + cronString);
        boolean flag = modelDicMapper.updateById(ModelDic.builder().mid(cronV.getMid()).cron(cronString).build()) > 0;
        return flag ? "修改成功" : "修改失败";
    }

    /**
     * 多条件查询模型运行日志
     *
     * @param stTime  开始时间
     * @param endTime 结束时间
     * @param mids    模型id字符串列表
     * @param method  运行方式
     * @param query   查询类
     * @return ModelLogView
     */
    @Override
    public ModelLogView getModelLogList(String stTime, String endTime, String mids, String method, BasePageQuery query) {
        Page<ModelLog> page = new Page<>(query.getCurrent(), query.getSize());
        String midString = ObjectUtils.isEmpty(mids) ? null : mids.replaceAll("，", ",");
        modelLogMapper.selectPage(page, new LambdaQueryWrapper<ModelLog>()
                .ge(!ObjectUtils.isEmpty(stTime) && !ObjectUtils.isEmpty(endTime), ModelLog::getSt, stTime)
                .le(!ObjectUtils.isEmpty(stTime) && !ObjectUtils.isEmpty(endTime), ModelLog::getEt, endTime)
                .eq(!ObjectUtils.isEmpty(method), ModelLog::getRm, method)
                .apply(!ObjectUtils.isEmpty(midString), "FIND_IN_SET(MID, '" + midString + "')"));
        return ModelLogView.builder().modelLogList(new PageVo<>(page)).build();
    }

    /**
     * 批量删除模型运行日志
     *
     * @param ids id字符串列表
     * @return boolean
     */
    @Override
    public boolean deleteModelLogs(String ids) {
        return modelLogMapper.deleteBatchIds(Arrays.stream(ids.replaceAll("，", ",").split(",")).collect(Collectors.toList())) > 0;
    }

    /**
     * 多条件查询模型参数和校准信息
     *
     * @param mid   模型id
     * @param query 查询类
     * @return ModelParamView
     */
    @Override
    public ModelParamView getModelParamList(Integer mid, BasePageQuery query) throws NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
        // 查询某个模型的参数
        List<ModelParam> listParam = modelParamMapper.selectModelParamList(mid);
        // 查询历史校准信息
        Page<HydSiForeaccuS> page = new Page<>(query.getCurrent(), query.getSize());
        hydSiForeaccuSMapper.selectPage(page, new LambdaQueryWrapper<HydSiForeaccuS>()
                .eq(HydSiForeaccuS::getMID, mid)
                .orderByAsc(HydSiForeaccuS::getDataTime));
        // 参数params字段变换
        Page page1 = getModelCalibration(page);
        return ModelParamView.builder().paramList(listParam).calibrationList(new PageVo<>(page1)).build();
    }


    /**
     * 模型参数运行校准（同时编辑某一个模型的参数）
     *
     * @param mParam 模型参数列表
     * @return boolean
     */
    @Override
    public boolean editModelParamAndCalibration(List<ModelParam> mParam) {
        // 1、编辑模型最新的参数
        Boolean ret1 = editModelParams(mParam);
        // 2、开始运行校准（挖坑）
        Boolean ret2 = doCalibration(mParam.get(0).getMid());
        return ret1 && ret2;
    }

    /**
     * 批量删除模型参数和校准信息
     *
     * @param ids id字符串列表
     * @return boolean
     */
    @Override
    public boolean deleteModelParam(String ids) {
        return hydSiForeaccuSMapper.deleteBatchIds(Arrays.stream(ids.replaceAll("，", ",").split(",")).collect(Collectors.toList())) > 0;
    }


    /**
     * 生成前台需要的模型拟合展示类
     *
     * @param page entity的分页类
     * @return Page 前台的分页类
     * @throws IllegalAccessException 异常
     */
    @NotNull
    private static Page getModelCalibration(Page<HydSiForeaccuS> page) throws IllegalAccessException {
        Field[] fields = GBHMCalibration.class.getDeclaredFields();
        List<GBHMCalibration> caList = new ArrayList<>();

        for (HydSiForeaccuS record : page.getRecords()) {
            GBHMCalibration gbhmCalibration = GBHMCalibration.builder()
                    .id(record.getID()).mid(record.getMID()).dataTime(record.getDataTime())
                    .stcd(record.getSTCD()).nse(record.getNSE()).r2(record.getR2())
                    .me(record.getME()).pbias(record.getPBIAS()).factor(record.getFACTOR())
                    .stDataTime(record.getSTDataTime()).edDataTime(record.getEDDataTime())
                    .optimal(record.getOptimal())
                    .build();
            // params不为空
            if (!ObjectUtils.isEmpty(record.getParams())) {
                String[] paramList = record.getParams().split(";");
                for (String each : paramList) {
                    String[] nameValList = each.split(":");
                    nameValList[1] = nameValList[1].replaceAll("[\\[\\]]", "");
                    // 匹配类字段，并赋值
                    for (Field field : fields) {
                        field.setAccessible(true);
                        String name = field.getName();
                        if (nameValList[0].equals(name)) {
                            field.set(gbhmCalibration, nameValList[1]);
                        }
                    }
                }
            }
            caList.add(gbhmCalibration);
        }
        Page page1 = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        page1.setRecords(caList);
        return page1;
    }

    /**
     * 编辑某一个模型的最新参数
     *
     * @param mParam 模型参数列表
     * @return Boolean
     */
    private Boolean editModelParams(List<ModelParam> mParam) {
        // 批量更新
        return modelParamMapper.updateBatch(mParam) > 0;
    }

    /**
     * 开始运行校准（挖坑）（待实现）
     *
     * @param mid 模型id
     * @return Boolean
     */
    private Boolean doCalibration(Integer mid) {
        return true;
    }
}
