package com.cuit.basin.base.biz.service.impl;


import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.basin.base.biz.dao.*;
import com.cuit.basin.base.biz.entity.GeoserverData;
import com.cuit.basin.base.biz.entity.HydroStationInfo;
import com.cuit.basin.base.biz.entity.WatershedData;
import com.cuit.basin.base.biz.pojo.GlacierDataUnion;
import com.cuit.basin.base.biz.pojo.HydroLocation;
import com.cuit.basin.base.biz.pojo.LakeDataUnion;
import com.cuit.basin.base.biz.pojo.SnowDataUnion;
import com.cuit.basin.base.biz.service.RTMonitoringBaseService;
import com.cuit.basin.base.biz.vo.GlacierView;
import com.cuit.basin.base.biz.vo.SnowView;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 实时监测基础数据接口实现类
 */
@Service
public class RTMonitoringBaseServiceImpl implements RTMonitoringBaseService {
    @Resource
    GeoserverDataMapper geoserverDataMapper;
    @Resource
    WatershedDataMapper watershedDataMapper;
    @Resource
    HydroStationInfoMapper hydroStationInfoMapper;
    @Resource
    GlacierDataMapper glacierDataMapper;
    @Resource
    SnowDataMapper snowDataMapper;
    @Resource
    LakeDataMapper lakeDataMapper;

    @Override
    public Object queryKeyWords(String key) {
        /**
         * 根据输入字符，查询流域名称、水文站号、水文站名等关键字列表
         */
        LambdaQueryWrapper<HydroStationInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        /**
         * 先查站号列表
         */
        lambdaQueryWrapper.select(HydroStationInfo::getStcd).likeRight(HydroStationInfo::getStcd, key);
        List<HydroStationInfo> hydroStationInfos = hydroStationInfoMapper.selectList(lambdaQueryWrapper);
        if (hydroStationInfos == null || hydroStationInfos.size() == 0) {
            /**
             * 站号没有，就查站名列表
             */
            LambdaQueryWrapper<HydroStationInfo> lambdaQueryWrapper2 = new LambdaQueryWrapper<>();
            lambdaQueryWrapper2.select(HydroStationInfo::getStnm).likeRight(HydroStationInfo::getStnm, key);
            hydroStationInfos = hydroStationInfoMapper.selectList(lambdaQueryWrapper2);
            if (hydroStationInfos == null || hydroStationInfos.size() == 0) {
                /**
                 * 站号，站名都不是，就查流域
                 */
                LambdaQueryWrapper<WatershedData> lambdaQueryWrapper3 = new LambdaQueryWrapper<>();
                lambdaQueryWrapper3.select(WatershedData::getBsnm).likeRight(WatershedData::getBsnm, key);
                List<WatershedData> watershedDataList = watershedDataMapper.selectList(lambdaQueryWrapper3);
                /**
                 * 为了通用返回，转json格式字符串。这里注意处理String类型的转义字符
                 */
                return watershedDataList;
            } else {
                return hydroStationInfos;
            }
        } else {
            return hydroStationInfos;
        }
    }

    @Override
    public HydroLocation queryResultByKeyWord(String keyword) {
        /**
         * 根据关键字，查询出某个流域或某个站点
         */
        JSONObject jsonObject = new JSONObject();
        LambdaQueryWrapper<HydroStationInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        /**
         * 先按站号查水文站点信息
         */
        lambdaQueryWrapper.select(HydroStationInfo::getStcd, HydroStationInfo::getStnm, HydroStationInfo::getBsid, HydroStationInfo::getLgtd, HydroStationInfo::getLttd).eq(HydroStationInfo::getStcd, keyword);
        HydroStationInfo hydroStationInfo = hydroStationInfoMapper.selectOne(lambdaQueryWrapper);
        if (hydroStationInfo == null) {
            /**
             * 再按站名查水文站点信息
             */
            LambdaQueryWrapper<HydroStationInfo> lambdaQueryWrapper2 = new LambdaQueryWrapper<>();
            lambdaQueryWrapper2.select(HydroStationInfo::getStcd, HydroStationInfo::getStnm, HydroStationInfo::getBsid, HydroStationInfo::getLgtd, HydroStationInfo::getLttd).eq(HydroStationInfo::getStnm, keyword);
            hydroStationInfo = hydroStationInfoMapper.selectOne(lambdaQueryWrapper2);
            if (hydroStationInfo == null) {
                /**
                 * 不是站点，就查流域
                 */
                LambdaQueryWrapper<WatershedData> lambdaQueryWrapper3 = new LambdaQueryWrapper<>();
                lambdaQueryWrapper3.eq(WatershedData::getBsnm, keyword);
                WatershedData watershedData = watershedDataMapper.selectOne(lambdaQueryWrapper3);
                /**
                 * 流域涉及地图服务，查询对应地图服务信息
                 */
                LambdaQueryWrapper<GeoserverData> lambdaQueryWrapper4 = new LambdaQueryWrapper<>();
                lambdaQueryWrapper4.eq(GeoserverData::getGeoid, watershedData.getGeoid());
                GeoserverData geoserverData = geoserverDataMapper.selectOne(lambdaQueryWrapper4);
                /**
                 * 返回的类是流域信息字段+地图服务数据字段拼接而成
                 * 转义字符处理
                 */
                return HydroLocation.builder()
                        .bsnm(watershedData.getBsnm()).lgtd(watershedData.getLgtd())
                        .lttd(watershedData.getLttd())
                        .bbox(geoserverData.getBbox()).build();
            } else {
                return HydroLocation.builder().stcd(hydroStationInfo.getStcd()).
                        stnm(hydroStationInfo.getStnm()).
                        lgtd(hydroStationInfo.getLgtd())
                        .lttd(hydroStationInfo.getLttd()).build();
            }
        } else {
            return HydroLocation.builder().stcd(hydroStationInfo.getStcd()).
                    stnm(hydroStationInfo.getStnm()).
                    lgtd(hydroStationInfo.getLgtd())
                    .lttd(hydroStationInfo.getLttd()).build();
        }
    }

    @Override
    public LakeDataUnion queryLakeLayer() {
        /**
         * 查询湖泊图层
         */
        LakeDataUnion lakeDataUnion = lakeDataMapper.selectLakeLayerInfo();

        return lakeDataUnion;
    }

    @Override
    public GlacierView queryLatestGlacierLayer() {
        /**
         *查询最新冰川图层列表
         */
        List<GlacierDataUnion> glacierDataUnions = glacierDataMapper.selectLatestGlacierLayerList();
        return GlacierView.builder().glacierLayerList(glacierDataUnions).build();
    }

    @Override
    public SnowView queryLatestSnowLayer() {
        /**
         *查询最新积雪图层列表
         */
        List<SnowDataUnion> snowDataUnions = snowDataMapper.selectLatestSnowLayerList();
        return SnowView.builder().snowLayerList(snowDataUnions).build();
    }
}
