package com.cuit.basin.base.biz.service.impl;


import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cuit.basin.base.biz.dao.TbSysDatauserMapper;
import com.cuit.basin.base.biz.dao.TbSysLogMapper;
import com.cuit.basin.base.biz.dao.UserInfoMapper;
import com.cuit.basin.base.biz.entity.SysLog;
import com.cuit.basin.base.biz.entity.TbSysDatauser;
import com.cuit.basin.base.biz.pojo.*;
import com.cuit.basin.base.biz.query.SysLogQuery;
import com.cuit.basin.base.biz.query.UserQuery;
import com.cuit.basin.base.biz.service.UserManagerService;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.utils.other.AssertUtil;
import com.cuit.basin.framework.common.utils.other.MD5;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author llw
 * @description 针对表【tb_sys_log】的数据库操作Service实现
 * @createDate 2022-12-01 16:35:24
 */
@Service
public class UserManagerServiceImpl extends ServiceImpl<TbSysLogMapper, SysLog> implements UserManagerService {
    @Autowired
    TbSysDatauserMapper tbSysDatauserMapper;
    @Autowired
    private TbSysLogMapper tbSysLogMapper;
    @Autowired
    private UserInfoMapper userInfoMapper;

    /*
     * @author: llw
     * @date: 2022/12/2 11:32
     * @param null
     * @return:
     * @description: 分页查询所有日志
     */
    @Override
    public Map queryAllSysLog() {

        Map refMap = new HashMap<>();
        List<SysLogPojo> sysLogPojoList;
        try {
            PageHelper.startPage(1, 10);
            PageInfo<SysLogPojo> pageInfo = new PageInfo<>(tbSysLogMapper.queryAllSysLog());
            refMap.put("data", pageInfo.getList());
            refMap.put("count", pageInfo.getTotal());
            return refMap;
        } catch (Exception e) {
            log.error("分页查询日志失败...");
            e.printStackTrace();
            throw new ErrorMessage("查询数据失败！");
        }
    }

    /*
     * @author: llw
     * @date: 2022/12/2 11:32
     * @param null
     * @return:
     * @description: 按条件查询所有日志
     */
    @Override
    public Map querySysLogByParams(SysLogQuery sysLogQuery) {
        Map refMap = new HashMap<>();
        try {
            PageHelper.startPage(sysLogQuery.getCurrent(), sysLogQuery.getSize());
            PageInfo<SysLogPojo> pageInfo = new PageInfo<>(tbSysLogMapper.querySysLogByParams(sysLogQuery));
            refMap.put("data", pageInfo.getList());
            refMap.put("count", pageInfo.getTotal());
            return refMap;
        } catch (Exception e) {
            log.error("分页查询失败...");
            e.printStackTrace();
            throw new ErrorMessage("查询数据失败！");
        }
    }

    /*
     * @author: llw
     * @date: 2022/12/2 11:32
     * @param null
     * @return:
     * @description: 批量删除日志
     */
    @Override
    public Boolean batchDeleteSysLogByQuery(String ids) {
        Boolean isDelete;
        AssertUtil.isTrue(ObjectUtil.isEmpty(ids), "删除消息时，id串为空！");
        isDelete = tbSysLogMapper.batchDeleteSysLogByQuery(ids);
        return isDelete;
    }

    /*
     * @author: llw
     * @date: 2022/12/2 11:32
     * @param null
     * @return:
     * @description: 多条件查询用户信息列表
     */
    @Override
    public Map querySysUserByParam(UserQuery userQuery) {
        Map refMap = new HashMap<>();
        List<UserInfo> userInfoList;
        try {
            PageHelper.startPage(userQuery.getCurrent(), userQuery.getSize());
            PageInfo<UserInfo> pageInfo = new PageInfo<>(userInfoMapper.querySysUserByParam(userQuery));
            refMap.put("count", pageInfo.getTotal());
            refMap.put("data", pageInfo.getList());
            return refMap;
        } catch (Exception e) {
            log.error("多条件查询系统用户列表失败！");
            e.printStackTrace();
            throw new ParamsException("多条件查询系统用户列表时错误......");
        }
    }


    /*
     * @author: llw
     * @date: 2022/12/4 20:48
     * @param null
     * @return:
     * @description: 批量删除用户
     */
    @Override
    public boolean batchDelectUserByIds(String ids) {
        AssertUtil.isTrue(ObjectUtil.isEmpty(ids), "删除用户时，id串为空！");
        return userInfoMapper.batchDelectUserByIds(ids);
    }

    /*
     * @author: llw
     * @date: 2022/12/4 22:07
     * @param id
     * @param status
     * @return: void
     * @description:启用/禁用用户
     */
    @Override
    public void enableOrDisable(String id, Integer status) {
        userInfoMapper.enableOrDisable(id, status);
    }


    /*
     * @author: llw
     * @date: 2022/12/5 14:38
     * @param UserDto
     * @return: isSuccess
     * @description: 编辑用户信息
     */
    @Override
    public Boolean editUserByParams(UserDto userDto, Integer id) {
        Boolean isSuccess;
        String loginName = null;
        //校验用户名信息是否已经存在
        loginName = userInfoMapper.checkUser(userDto.getLoginname());
        //用户名不存在,执行操作
        if (loginName == null) {
            MD5 md = new MD5();
            userDto.setPassword(md.encryption(userDto.getPassword()));
            isSuccess = userInfoMapper.editUserByParams(userDto, id);
            return isSuccess;
        } else {
            throw new ErrorMessage("用户名重复...");
        }
    }


    /**
     * @param
     * @return
     * @description : 查询个人中心信息
     * @author tianjunbiao
     * @date 2022/12/4 15:15
     */
    @Override
    public QueryPersonalCenterInfo queryPersonalCenterInfo(String account) {
        return tbSysDatauserMapper.queryPersonalCenterInfo(account);
    }

    /**
     * @param
     * @return
     * @description : 修改个人中心信息
     * @author tianjunbiao
     * @date 2022/12/4 16:03
     */
    @Override
    public void updatePersonalCenterInfo(String account, UpdatePersonalCenterInfo updatePersonalCenterInfo) {
        tbSysDatauserMapper.updatePersonalCenterInfo(account, updatePersonalCenterInfo);
    }

    /**
     * @param
     * @return
     * @description : 修改密码
     * @author tianjunbiao
     * @date 2022/12/4 17:19
     */
    @Override
    public void updatePassword(String account, String newPassword) {
        MD5 md5 = new MD5();
        String password = md5.encryption(newPassword);
        tbSysDatauserMapper.updatePassword(account, password);
    }

    /**
     * @param
     * @return
     * @description : 05按条件分页查询个人日志
     * @author tianjunbiao
     * @date 2022/12/4 20:11
     */
    @Override
    public Map querySysLogByParams(String account, SysLogQuery sysLogQuery) {
        Map refMap = new HashMap<>();
        try {
            PageHelper.startPage(sysLogQuery.getCurrent(), sysLogQuery.getSize());
            PageInfo<SysLogPojo> PageInfo = new PageInfo<>(tbSysDatauserMapper.querySysLogByParams(account, sysLogQuery));
            refMap.put("data", PageInfo.getList());
            refMap.put("total", PageInfo.getTotal());
            return refMap;
        } catch (Exception e) {
            log.error("分页查询失败...");
            e.printStackTrace();
            throw new ErrorMessage("查询数据失败！");
        }
    }

    /**
     * @param
     * @return
     * @description : 展示个人日志
     * @author tianjunbiao
     * @date 2022/12/4 20:36
     */
    @Override
    public List<SysLogPojo> queryPersonalLogs(String account) {
        try {
            return tbSysDatauserMapper.queryPersonalLogs(account);
        } catch (Exception e) {
            log.error("查询个人日志失败...");
            e.printStackTrace();
            throw new ErrorMessage("查询数据失败！");
        }
    }

    /**
     * @param
     * @return
     * @description : 查询所有用户信息
     * @author tianjunbiao
     * @date 2022/12/3 20:40
     */
    @Override
    public Map queryAllUsersInfo() {
        Map refMap = new HashMap<>();
        try {
            PageHelper.startPage(1, 10);
            PageInfo<TbSysDatauser> PageInfo = new PageInfo<>(tbSysDatauserMapper.queryAllUsersInfo());
            refMap.put("data", PageInfo.getList());
            refMap.put("total", PageInfo.getTotal());
            return refMap;
        } catch (Exception e) {
            log.error("查询所有用户信息失败...");
            e.printStackTrace();
            throw new ErrorMessage("查询所有用户信息失败！");
        }
    }

    /*
     * @description: 下拉展示菜单接口(部门或单位)
     * @author: LLW
     * @date: 2022/12/8 16:13
     * @param:
     * @return:
     **/

    @Override
    public List<String> DropdownMenuOfDptName() {
        return tbSysDatauserMapper.DropdownMenuOfDptName();
    }

    /*
     * @description: 下拉展示菜单接口(所任职务)
     * @author: LLW
     * @date: 2022/12/8 17:01
     * @param:
     * @return: list：职务列表
     **/

    @Override
    public List<String> DropdownMenuOfPositionName() {
        return tbSysDatauserMapper.DropdownMenuOfPositionName();
    }


    /*
     * @description: 下拉展示菜单接口(权限等级)
     * @author: LLW
     * @date: 2022/12/8 17:06
     * @param:
     * @return: list：权限等级列表
     **/


    @Override
    public List<String> DropdownMenuOfPermissonName() {
        return tbSysDatauserMapper.DropdownMenuOfPermissonName();
    }

    /**
     * @param
     * @return
     * @description : 新增用户信息
     * @author tianjunbiao
     * @date 2022/12/5 11:30
     */
//    @Override
//    public void addUser(UserDto userDto) {
//        tbSysDatauserMapper.addUser(userDto);
//    }
}




