package com.cuit.basin.base.biz.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.basin.base.biz.dao.HydBiGlacierDMapper;
import com.cuit.basin.base.biz.dao.HydBiGlacierSMapper;
import com.cuit.basin.base.biz.dao.HydBiSnowBMapper;
import com.cuit.basin.base.biz.dao.TfReportMapper;
import com.cuit.basin.base.biz.entity.HydBiGlacierD;
import com.cuit.basin.base.biz.entity.TfReport;
import com.cuit.basin.base.biz.service.WaterResourceDBService;
import com.cuit.basin.base.biz.vo.GlacierDatasetView;
import com.cuit.basin.base.biz.vo.SnowDatasetView;
import com.cuit.basin.framework.common.base.BasePageQuery;
import com.cuit.basin.framework.common.exception.ParamsException;
import com.cuit.basin.framework.common.ret.PageVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class WaterResourceDBServiceImpl implements WaterResourceDBService {
    @Autowired
    private HydBiGlacierSMapper hydBiGlacierSMapper;
    @Autowired
    private HydBiGlacierDMapper hydBiGlacierDMapper;
    @Autowired
    private HydBiSnowBMapper hydBiSnowBMapper;
    @Autowired
    private TfReportMapper tfReportMapper;

    /**
     * 条件分页查询冰川数据集
     *
     * @param startYear 开始年份
     * @param endYear   结束年份
     * @param keyword   关键字
     * @param pageQuery 分页参数
     * @return PageVo<GlacierDatasetView>
     */
    @Override
    public PageVo<GlacierDatasetView> queryGlacierDataSetByParam(Integer startYear, Integer endYear, String keyword, BasePageQuery pageQuery) {
        //分页对象，当前面及每页条数
        Page<GlacierDatasetView> page = new Page<>(pageQuery.getCurrent(), pageQuery.getSize());
        hydBiGlacierSMapper.queryGlacierDataSetByParam(page, startYear, endYear, keyword);
        //将Page转换为PageVO
        return new PageVo<>(page);
    }

    /**
     * 查询数据集对应冰川数据
     *
     * @param setId      数据集Id
     * @param glacierIds 冰川Id
     * @return List<HydBiGlacierD>
     */
    @Override
    public List<HydBiGlacierD> queryGlacierDataByGlacierIds(Integer setId, String glacierIds) {
        String ids = glacierIds.replaceAll("，", ",");
        List<HydBiGlacierD> list = new ArrayList<>();
        list = hydBiGlacierDMapper.queryGlacierDataByGlacierIds(setId, glacierIds);
        if (ObjectUtils.isEmpty(list)){
            throw new ParamsException("无编号为："+glacierIds+"的数据！");
        }
        return list;
    }

    /**
     * 条件查询冰川的变化曲线
     *
     * @param glacierId 冰川Id
     * @param startYear 开始年份
     * @param endYear   结束年份
     * @return List<HydBiGlacierD>
     */
    @Override
    public List<HydBiGlacierD> queryGlacierDataChangeCurveByGlacierId(String glacierId, Integer startYear, Integer endYear) {
        List<HydBiGlacierD> list = hydBiGlacierDMapper.queryGlacierDataChangeCurveByGlacierId(glacierId, startYear, endYear);
        return list;
    }

    /**
     * 条件分页查询研究报告
     *
     * @param year      年份
     * @param dataType  数据类型
     * @param keyword   关键字
     * @param pageQuery 分页参数
     * @return PageVo<TfReport>
     */
    @Override
    public PageVo<TfReport> queryReportByParam(Integer year, String dataType, String keyword, BasePageQuery pageQuery) {
        //分页对象，当前面及每页条数
        Page<TfReport> page = new Page<>(pageQuery.getCurrent(), pageQuery.getSize());
        tfReportMapper.queryReportByParam(page, year, dataType, keyword);
        //将Page转换为PageVO
        return new PageVo<>(page);
    }

    /**
     * 下载研究报告
     *
     * @param reportIds 报告ids
     * @return List<String>
     */
    @Override
    public List<String> downloadReportByIds(String reportIds) {
        List<String> urls = new ArrayList<>();
        String ids = reportIds.replaceAll("，", ",");
        List<TfReport> tfReports = tfReportMapper.selectByIds(ids);
        urls = tfReports.stream().map(TfReport::getUrl).collect(Collectors.toList());
        return urls;
    }

    /**
     * 条件分页查询积雪数据集
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param keyword   关键字
     * @param pageQuery 分页参数
     * @return PageVo<SnowDatasetView>
     */
    @Override
    public PageVo<SnowDatasetView> querySnowDataSettByParam(LocalDateTime startTime, LocalDateTime endTime, String keyword, BasePageQuery pageQuery) {
        //分页对象，当前面及每页条数
        Page<SnowDatasetView> page = new Page<>(pageQuery.getCurrent(), pageQuery.getSize());
        hydBiSnowBMapper.querySnowDataSettByParam(page, startTime, endTime, keyword);
        //将Page转换为PageVO
        return new PageVo<>(page);
    }
}
