package com.cuit.basin.base.biz.service.model;


import com.cuit.basin.base.biz.controller.model.vo.InitTaskParams;
import com.cuit.basin.base.biz.controller.model.vo.ModelTaskInfo;
import com.cuit.basin.base.biz.controller.model.vo.SuccessTaskParams;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.sriver.sriversys.service.model
 * @Author: HouPJ
 * @Date: 2022/12/23 9:54
 * @Description: 模型运行服务
 */
public interface ModelOperationTaskService {

    /**
     * 初始化模型任务
     * @param initTaskParams 模型初始化参数
     * @return 模型任务关键信息
     */
    ModelTaskInfo initModelTask(InitTaskParams initTaskParams);

    /**
     * 当模型运行过程中发生异常时调用
     * @param modelTaskId 模型日志id
     */
    void stopModelWithError(String modelTaskId);

    /**
     * 获取模型运行进度
     * @param modelTaskId 模型任务id
     * @return 进度
     */
    ModelTaskInfo obtainTaskProgress(String modelTaskId);

    /**
     * 模型任务有更新时调用
     * @param successTaskParams 模型成功参数
     */
    void modelTaskRefreshHandler(SuccessTaskParams successTaskParams);

    /**
     * 模型任务发生异常时调用
     * @param apiErrorLog 模型异常参数
     */
    void modelTaskExceptionHandler(String apiErrorLog);

    /**
     * 模型任务成功时调用
     * @param successTaskParams
     */
    void modelTaskSuccessHandler(SuccessTaskParams successTaskParams);

}
