package com.cuit.basin.base.biz.service.monitor;

import cn.hutool.core.util.ObjectUtil;
import com.cuit.basin.base.biz.dao.DicModuleMapper;
import com.cuit.basin.base.biz.dao.LoginUserMapper;
import com.cuit.basin.base.biz.dao.TbSysErrorLogMapper;
import com.cuit.basin.base.biz.dao.TbSysLogMapper;
import com.cuit.basin.base.biz.entity.SysLog;
import com.cuit.basin.base.biz.entity.TbSysErrorLog;
import com.cuit.basin.base.biz.pojo.monitor.BaseAttributeReq;
import com.cuit.basin.base.biz.pojo.monitor.MonitorLogDisplayReq;
import com.cuit.basin.base.biz.service.monitor.factory.MonitorHandlerFactory;
import com.cuit.basin.base.biz.service.monitor.handler.DataAttributionHandler;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.utils.other.AssertUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.basin.base.biz.service.monitor
 * @Author: HouPJ
 * @Date: 2022/12/22 11:12
 * @Description: TODO
 */
@Slf4j
@Service
public class RTMonitorScreenServiceImpl implements RTMonitorScreenService {

    @Resource
    private LoginUserMapper loginUserMapper;
    @Resource
    private TbSysErrorLogMapper errorLogMapper;
    @Resource
    private TbSysLogMapper tbSysLogMapper;
    @Resource
    private DicModuleMapper moduleMapper;
    @Resource
    private MonitorHandlerFactory factory;
    private DataAttributionHandler dataAttributionHandler;

    @Override
    public List<MonitorLogDisplayReq> obtainRTMonitorLogs(Integer moduleId) {
        AssertUtil.isTrue(ObjectUtil.isNull(moduleId), "请输入模块id");
        List<MonitorLogDisplayReq> monitorLogDisplayReqs = new ArrayList<>();
        String moduleName= moduleMapper.obtainModuleDic(moduleId).getTypeName();
        // 查询并封装操作日志表
        try {
            createOperationLog(moduleId, monitorLogDisplayReqs, moduleName);
        } catch (Exception e) {
            throw new ErrorMessage("查询并封装操作日志出错");
        }
        try{
            // 查询并封装异常日志表
            createErrorLog(moduleId, monitorLogDisplayReqs, moduleName);
        }catch (Exception e) {
            throw new ErrorMessage("查询并封装异常日志出错");
        }

        return monitorLogDisplayReqs;
    }

    @Override
    public BaseAttributeReq getServerRTAttribute(Integer moduleId) {
        // 根据模块id选择对应处理器
        dataAttributionHandler = factory.createDictionaryFetch(moduleMapper.obtainModuleDic(moduleId).getTypeName());
        // 执行获取数据操作
        AssertUtil.isTrue(ObjectUtil.isNull(dataAttributionHandler),"请输入正确的模块id[7，8，9]");
        // 返回
        return dataAttributionHandler.getServerRTAttribute();
    }

    /**
     * 查询并封装异常日志表
     * @param moduleId
     * @param monitorLogDisplayReqs
     */
    private void createErrorLog(Integer moduleId, List<MonitorLogDisplayReq> monitorLogDisplayReqs,String module) {
        List<TbSysErrorLog> tbSysErrorLogs = errorLogMapper.selectListByModuleId(moduleId);
        tbSysErrorLogs.forEach(term -> {
            monitorLogDisplayReqs.add(MonitorLogDisplayReq.builder().module(module)
                    .createTime(term.getExceptionTime())
                    .describe(initializeErrorDescribe(term)).build());
        });

    }

    /**
     * 查询并封装操作日志表
     * @param moduleId
     * @param monitorLogDisplayReqs
     */
    private void createOperationLog(Integer moduleId, List<MonitorLogDisplayReq> monitorLogDisplayReqs,String module) {
        List<SysLog> sysLogs = tbSysLogMapper.selectListByModuleId(moduleId);
        sysLogs.stream().forEach(term -> {
            monitorLogDisplayReqs.add(MonitorLogDisplayReq.builder().module(module)
                    .createTime(term.getVisitTime())
                    .describe(initializeOpDescribe(term)).build());
        });
    }

    /**
     * 初始化操作日志描述
     *
     * @param term
     * @return
     */
    private String initializeOpDescribe(SysLog term) {
        return "用户" + term.getUserName() + "进行了[" + term.getOpration() + "]操作，时长为[" + term.getExecutionTime() + "]";
    }

    /**
     * 初始化异常日志描述
     * @param term
     * @return
     */
    private String initializeErrorDescribe(TbSysErrorLog term) {
        return "用户" + loginUserMapper.getUserAccountById(term.getUserId())+"在执行操作["+term.getExceptionMethodName()+"]时发生异常：["+term.getExceptionName()+"："+term.getExceptionMessage()+"]";
    }

}
