package com.cuit.basin.base.biz.service.monitor.handler;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cuit.basin.base.biz.dao.ModelLogMapper;
import com.cuit.basin.base.biz.dao.TbSysLogMapper;
import com.cuit.basin.base.biz.entity.ModelLog;
import com.cuit.basin.base.biz.pojo.monitor.ModelAttributeReq;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.basin.base.biz.service.monitor.handler
 * @Author: HouPJ
 * @Date: 2022/12/22 15:06
 * @Description: TODO
 */
@Service(value = "模型管理")
public class ModelDataAttributionHandler implements DataAttributionHandler {

    @Resource
    private TbSysLogMapper tbSysLogMapper;
    @Resource
    private ModelLogMapper modelLogMapper;

    @Override
    public ModelAttributeReq getServerRTAttribute() {
        // 获取模型日志
        List<ModelLog> modelLogs = modelLogMapper.selectList(new QueryWrapper<ModelLog>());
        List<Integer> excutionTime = getExcutionTimeList(modelLogs);
        // 封装模型属性信息列表
        ModelAttributeReq modelAttributeReq = ModelAttributeReq.builder()
                .modelRunTotal(modelLogs.size())
                .timeAvg(formatDable(excutionTime.stream().mapToDouble(Double::valueOf).average().getAsDouble()) + "分钟")
                .timeMin(formatDable(excutionTime.stream().mapToDouble(Double::valueOf).min().getAsDouble()) + "分钟")
                .timeMax(formatDable(excutionTime.stream().mapToDouble(Double::valueOf).max().getAsDouble()) + "分钟")
                .build();
        modelAttributeReq.setRecentTermTime(modelLogMapper.obtainRecentModelTime());
        return modelAttributeReq;
    }

    private double formatDable(double num) {
        return BigDecimal.valueOf(num).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    private List<Integer> getExcutionTimeList(List<ModelLog> sysLogs) {
        // 获取时长字段列表
        List<Integer> excutionTime = new ArrayList<>();
        sysLogs.stream().forEach(item -> {
            if (item.getRt() != null) {
                excutionTime.add(item.getRt());
            }
        });
        return excutionTime;
    }

}
