package com.cuit.basin.framework.common.utils.other;

import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author hpj
 * @describe 构建树工具
 */
@Slf4j
public class BuildTreeUtil {

    public static <T> List<T> buildAreaTree(List<T> dataList) {
        List<T> tree = new ArrayList<>();
        for (Iterator<T> it = dataList.iterator(); it.hasNext(); ) {
            T data = (T) it.next();
            try {
                if ("0".equals(String.valueOf(ReflectUtil.getGetMethod(data, "pid")))) {
                    recursionFn(dataList, data);
                    tree.add(data);
                }
            } catch (Exception e) {
                e.printStackTrace();
                AssertUtil.isTrue(true, "获取顶层等级数据列表时发生错误...");
            }
        }
        return tree;
    }

    /**
     * 递归列表
     *
     * @param dataList 地区列表
     * @param data     当前等级地区类
     */
    private static <T> void recursionFn(List<T> dataList, T data) {
        // 得到子节点列表
        List<T> childList = getChildList(dataList, data);
        try {
            ReflectUtil.getSetMethod(data, "children", childList);
        } catch (Exception e) {
            AssertUtil.isTrue(true, "设置子列表时发生错误...");
        }
        for (T child : childList) {
            if (hasChild(dataList, child)) {
                //判断是否有子节点
                Iterator<T> it = childList.iterator();
                while (it.hasNext()) {
                    T n = (T) it.next();
                    recursionFn(dataList, n);
                }
            }
        }
    }

    /**
     * 得到子节点列表
     *
     * @param list
     * @param data
     * @return
     */
    private static <T> List<T> getChildList(List<T> list, T data) {
        List<T> tlist = new ArrayList<T>();
        Iterator<T> it = list.iterator();
        while (it.hasNext()) {
            T dataTemp = (T) it.next();
            try {
                if (ReflectUtil.getGetMethod(dataTemp, "pid") == ReflectUtil.getGetMethod(data, "id")) {
                    tlist.add(dataTemp);
                }
            } catch (Exception e) {
                e.printStackTrace();
                AssertUtil.isTrue(true, "获取子列表时发生错误...");
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     *
     * @param list
     * @param data
     * @return
     */
    private static <T> boolean hasChild(List<T> list, T data) {
        return getChildList(list, data).size() > 0 ? true : false;
    }


}
