package com.cuit.basin.framework.common.utils.word;

import fr.opensagres.poi.xwpf.converter.xhtml.Base64EmbedImgManager;
import fr.opensagres.poi.xwpf.converter.xhtml.XHTMLConverter;
import fr.opensagres.poi.xwpf.converter.xhtml.XHTMLOptions;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.poi.hwpf.HWPFDocumentCore;
import org.apache.poi.hwpf.converter.PicturesManager;
import org.apache.poi.hwpf.converter.WordToHtmlConverter;
import org.apache.poi.hwpf.converter.WordToHtmlUtils;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.*;

@Slf4j
public class WordHtmlUtil {
    //doc转换html
    public static String docToHtml(File file) throws IOException, ParserConfigurationException, TransformerException {
        HWPFDocumentCore wordDocument = WordToHtmlUtils.loadDoc(new FileInputStream(file));
        WordToHtmlConverter wordToHtmlConverter = new WordToHtmlConverter(
                DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument()
        );
        PicturesManager pictureRunMapper = (bytes, pictureType, s, v, v1) -> "data:image/png;base64,"+Base64.encodeBase64String(bytes);
        wordToHtmlConverter.setPicturesManager(pictureRunMapper);
        wordToHtmlConverter.processDocument(wordDocument);
        Document htmlDocument = wordToHtmlConverter.getDocument();
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DOMSource domSource = new DOMSource(htmlDocument);
        StreamResult streamResult = new StreamResult(out);
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer serializer = transformerFactory.newTransformer();
        serializer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
        serializer.setOutputProperty(OutputKeys.METHOD, "html");
        serializer.transform(domSource, streamResult);
        out.close();
        return new String(out.toByteArray());
    }

    //docx转换html
    public static String docxToHtml(File file) throws IOException {
        XWPFDocument docxDocument = new XWPFDocument(new FileInputStream(file));
        XHTMLOptions options = XHTMLOptions.create();
        //图片转base64
        options.setImageManager(new Base64EmbedImgManager());
        // 转换htm11
        ByteArrayOutputStream htmlStream = new ByteArrayOutputStream();
        XHTMLConverter.getInstance().convert(docxDocument, htmlStream, options);
        htmlStream.close();
        return new String(htmlStream.toByteArray());
    }
    public static boolean addHtml(String content, String path){
        //用于存储html字符串
        StringBuilder stringHtml = new StringBuilder();
        //输入HTML文件内容
        stringHtml.append("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
        stringHtml.append("<head>");
        stringHtml.append("<meta http-equiv='Content-Type' content='text/html;  charset=UTF-8'>");
        stringHtml.append("<title></title>");
        stringHtml.append("</head>");
        stringHtml.append("<body style=\"pagewidth:595.3pt;pageheight:841.9pt;docgridtype:lines;docgridlinepitch:15.6pt;>");
        stringHtml.append("<div class=\"Section0\"><div style=\"min-height:72pt\" />");
        stringHtml.append(content);
        stringHtml.append("</div></body></html>");
        try{
            File f = new File(path);
            if (f.getParentFile() != null || !f.getParentFile().isDirectory()) {
                // 创建文件
                f.getParentFile().mkdirs();
            }
            //将HTML文件内容写入文件中
            FileOutputStream fileOutputStream=new FileOutputStream(path);
            PrintStream printStream=new PrintStream(fileOutputStream);
            // 转码，转化为utf-8
            String htmls = new String(stringHtml.toString().getBytes("utf-8"),"utf-8");
            printStream.println(htmls);
            fileOutputStream.flush();
            printStream.flush();
            fileOutputStream.close();
            printStream.close();
            return true;
        }catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static void main(String[] args) {
//        String a = "{\"html\":\"<p>saasafdfdsadasd</p>\\n<p><strong>sdfsdfsdf</strong></p>\\n<p><strong>打开解放军官兵的时间赶不上的肌肤比较失败的不过深度不够撒旦解放大师傅士大夫士大夫士大夫士大夫士大夫看见电脑打开发给你的口水歌看你可怜的那个锻炼方法的联合国的法律后果费德勒换个地方；陆海空地方管理框架和 规范化风格风格化风格化分隔花粉管花粉管和法规和风格化法国和法国恢复回复给梵蒂冈豆腐干反对恢复共和国建国和话剧规划局肉汤羊肉汤预亏里哦蕾哈娜贴吧软弱和任天堂如果退货有高速带宽不够数量的可能过来的咖啡馆上的困境老师的可能各类考试的 <span id=\\\"attachment_mce_2\\\" class=\\\"attachment\\\" contenteditable=\\\"false\\\"><img src=\\\"http://172.16.80.138:8917/iccMeeting/tinymce/fileIcons//file_type_image.svg\\\" /><a href=\\\"http://172.16.80.145//fileServer\\\\upload\\\\2022/06/27\\\\8ac4290caa5f0a88930d80d2f4b608f5.jpeg\\\">8ac4290caa5f0a88930d80d2f4b608f5.jpeg (8.35 K)</a></span></strong></p>\\n<p><img src=\\\"http://172.16.80.145//fileServer\\\\upload\\\\2022/06/27\\\\cut-6546-1656312210599.png\\\" alt=\\\"\\\" /><img src=\\\"http://172.16.80.145//fileServer\\\\upload\\\\2022/06/27\\\\cut-6546-1656312781796.png\\\" alt=\\\"\\\" /></p>\\n<p><strong>sdfsdf</strong></p>\\n<p><strong><span style=\\\"color: #e03e2d;\\\">yrdfsdf</span></strong></p>\\n<p><strong><span style=\\\"color: #e03e2d;\\\"><span style=\\\"font-size: 24px;\\\">dsfsdfsdfsfsdfsdfsdfsxxxdfvbvscaskdbsjhcbsddjfcbsdjdbfcsjdbfcshdbfcjskzhbvdfafvajkshbdfvjhoasdbvjsadbvkjdsbvkjdfhbfvkjdbhfvkjdhbfvkjdzbhfvjjdhbvvdf你好山东福建河南山东快解封你是困难的时刻发你的空间概念的设计风格百度是打开空间概念是的看法呢广东省快解封那个时代开发功能速度快给你是打开开关你手动开关你是大哥</span></span></strong></p>\\n<p><span style=\\\"text-decoration: underline;\\\"><strong><span style=\\\"color: #e03e2d; text-decoration: underline;\\\"><span style=\\\"font-size: 24px;\\\">dsfsfsd<span id=\\\"attachment_mce_1\\\" class=\\\"attachment\\\" contenteditable=\\\"false\\\"><img src=\\\"http://172.16.80.138:8917/iccMeeting/tinymce/fileIcons//file_type_image.svg\\\" /><a href=\\\"http://172.16.80.145//fileServer\\\\upload\\\\2022/06/27\\\\8ac4290caa5f0a88930d80d2f4b608f5.jpeg\\\">8ac4290caa5f0a88930d80d2f4b608f5.jpeg (8.35 K)</a></span></span></span></strong></span></p>\",\"imgUrl\":\"http://172.16.80.138:8917\"}";

        String a = "<p class=\"Normal\" style=\"text-align:center;background-color:#FFAFAF;line-height:0.95;margin-top:13.95pt;margin-left:0.3pt;text-indent:-0.3pt;border-bottom-color:#000000;border-bottom-style:solid;border-bottom-width:0.67px;\"><span style=\"letter-spacing:6.1pt;color:#000000;font-size:43.0pt;font-family:楷体;mso-fareast-font-family:楷体;mso-bidi-font-family:楷体;lang:EN-US;mso-fareast-language:ZH-CN;font-weight:bold;\">水情公报</span><a id=\"_GoBack\" /></p><p class=\"Normal\" style=\"text-align:center;line-height:0.95;margin-top:13.95pt;border-bottom-color:#000000;border-bottom-style:solid;border-bottom-width:0.67px;\"><span style=\"letter-spacing:0.3pt;color:#000000;font-size:10.0pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;lang:EN-US;mso-fareast-language:ZH-CN;font-weight:bold;\">2020年09月04日17时-2022年12月17日00时</span></p><p class=\"Normal\" style=\"font-size: 10.5pt;\"><span style=\"color:#000000;font-size:10.5pt;font-family:Arial;mso-fareast-font-family:宋体;mso-bidi-font-family:Arial;lang:EN-US;mso-fareast-language:ZH-CN;\">&#xa0;</span></p><p class=\"secondTitleStyle\" style=\"margin-top:10.0pt;margin-bottom:10.0pt;\"><span style=\"color:#000000;font-size:10.0pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;font-weight:bold;\">一、雨情</span></p><p class=\"paraStyle\" style=\"line-height:0.83;text-indent:25.0pt;\"><span style=\"color:#000000;font-size:9.5pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;\">2020年09月04日17时至2022年12月17日00时,子流域1流域内。特征值统计：978056（）10分钟平均降雨量为0.37毫米,10分钟最大降雨量为1.9毫米,10分钟最小降雨量为0.0毫米,1小时平均降雨量为0.0毫米,1小时最大降雨量为0.0毫米,1小时最小降雨量为0.0毫米。超警情况：978056（）10分钟平均降雨量超过阈值0.37毫米,10分钟最大降雨量超过阈值1.9毫米,10分钟最小降雨量超过阈值0.0毫米,1小时平均降雨量超过阈值0.0毫米,1小时最大降雨量超过阈值0.0毫米,1小时最小降雨量超过阈值0.0毫米。</span></p><p style=\"text-align: center; \"><img src= \"http://localhost:8086/file/png/qwerty.png\" alt=\"2020年09月04日17时-2022年12月17日00时西藏年降水量专题图专题图></p><p class=\"secondTitleStyle\" style=\"margin-top:10.0pt;margin-bottom:10.0pt;\"><span style=\"color:#000000;font-size:10.0pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;font-weight:bold;\">二、水情</span></p><p class=\"paraStyle\" style=\"line-height:0.83;text-indent:25.0pt;\"><span style=\"color:#000000;font-size:9.5pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;\">2020年09月04日17时至2022年12月17日00时,子流域1流域内。特征值统计：S1001（测试站1），小时平均水位为52.1米，小时平均流量为17.57立方米/秒，小时最大水位为52.5米，小时最大流量为19.25立方米/秒，小时最小水位为51.7米。S1002（测试站2），小时平均水位为47.0米，小时平均流量为18.96立方米/秒，小时最大水位为49.7米，小时最大流量为20.3立方米/秒，小时最小水位为44.3米。超警情况：S1001（测试站1）10分钟平均降雨量超过警戒值32.5米,10分钟最大降雨量超过警戒值-7.5立方米/秒,10分钟最小降雨量超过警戒值42.5米,1小时平均降雨量超过警戒值-7.5立方米/秒,1小时最大降雨量超过警戒值32.5米。S1002（测试站2）10分钟平均降雨量超过警戒值33.7米,10分钟最大降雨量超过警戒值-58.3立方米/秒,10分钟最小降雨量超过警戒值41.7米,1小时平均降雨量超过警戒值-58.3立方米/秒,1小时最大降雨量超过警戒值33.7米。</span></p><p class=\"secondTitleStyle\" style=\"margin-top:10.0pt;margin-bottom:10.0pt;\"><span style=\"color:#000000;font-size:10.0pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;font-weight:bold;\">三、模型预报</span></p><p class=\"paraStyle\" style=\"line-height:0.83;text-indent:25.0pt;\"><span style=\"color:#000000;font-size:9.5pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;\">2020年09月04日17时至2022年12月17日00时,子流域1流域内。特征值统计：S1001（测试站1），GBHM模型，预测小时平均流量为2.93立方米/秒，预测小时最大水位为18.0米，预测小时最大流量为15.0立方米/秒，预测小时最小水位为3.0米，预测小时最小流量为8.0立方米/秒。S1001（测试站1），M2模型，预测小时平均流量为0.47立方米/秒，预测小时最大水位为12.0米，预测小时最大流量为6.0立方米/秒，预测小时最小水位为11.0米，预测小时最小流量为1.0立方米/秒。S1001（测试站1），M3模型，预测小时平均流量为2.01立方米/秒，预测小时最大水位为18.0米，预测小时最大流量为15.0立方米/秒，预测小时最小水位为2.0米，预测小时最小流量为1.1立方米/秒。S1001（测试站1），Mannual模型，预测小时平均流量为2.07立方米/秒，预测小时最大水位为17.0米，预测小时最大流量为13.0立方米/秒，预测小时最小水位为2.0米，预测小时最小流量为1.0立方米/秒。超警情况：S1001（测试站1），GBHM模型10分钟平均降雨量超过警戒值-42.0立方米/秒,10分钟最大降雨量超过警戒值8.0米,10分钟最小降雨量超过警戒值-42.0立方米/秒,1小时平均降雨量超过警戒值-2.0米,1小时最大降雨量超过警戒值-42.0立方米/秒。S1001（测试站1），M2模型10分钟平均降雨量超过警戒值-48.0立方米/秒,10分钟最大降雨量超过警戒值2.0米,10分钟最小降雨量超过警戒值-48.0立方米/秒,1小时平均降雨量超过警戒值-8.0米,1小时最大降雨量超过警戒值-48.0立方米/秒。S1001（测试站1），M3模型10分钟平均降雨量超过警戒值-42.0立方米/秒,10分钟最大降雨量超过警戒值8.0米,10分钟最小降雨量超过警戒值-42.0立方米/秒,1小时平均降雨量超过警戒值-2.0米,1小时最大降雨量超过警戒值-42.0立方米/秒。S1001（测试站1），Mannual模型10分钟平均降雨量超过警戒值-43.0立方米/秒,10分钟最大降雨量超过警戒值7.0米,10分钟最小降雨量超过警戒值-43.0立方米/秒,1小时平均降雨量超过警戒值-3.0米,1小时最大降雨量超过警戒值-43.0立方米/秒。</span></p><p class=\"paraStyle\" style=\"text-align:right;margin-top:10.0pt;margin-bottom:10.0pt;\"><span style=\"color:#000000;font-size:9.5pt;font-family:宋体;mso-fareast-font-family:宋体;mso-bidi-font-family:宋体;\">发布时间:2022年12月26日</span></p>";
//        String b = "<html><body>" + JSONObject.parseObject(a).get("html").toString() + "</body></html>";
        addHtml(a,"D:/test.pdf");
    }
    /**
     *  读取本地html文件里的html代码
     * @param file  File file=new File("文件的绝对路径")
     * @return
     */
    public static String toHtmlString(File file) {
        // 获取HTML文件流
        StringBuffer htmlSb = new StringBuffer();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(
                    new FileInputStream(file), "utf-8"));
            while (br.ready()) {
                htmlSb.append(br.readLine());
            }
            br.close();
            // 删除临时文件
            //file.delete();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        // HTML文件字符串
        String htmlStr = htmlSb.toString();
        // 返回经过清洁的html文本
        return htmlStr;
    }

    /**
     * 去除第一行水印
     * @param filePath 原路径
     * @param filePathS 新路径
     */
    public static void fileZh(String filePath,String filePathS) throws IOException {

        //将文档输入input流
        InputStream is = new FileInputStream(filePath);
        XWPFDocument document = new XWPFDocument(is);
        //Spire的产品 生成的文件会自带同步警告信息，这里来删除Spire产品的的警告信息
        document.removeBodyElement(0);
        //将文件流put到新的文件中
        OutputStream os=new FileOutputStream(filePathS);
        try {
            document.write(os);
            log.info("水印去除完成，文档生成成功");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
