package com.cuit.basin.framework.kafka.core.api;


import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.kafka.core.util.KafkaProducerUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.support.SendResult;

import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.util.concurrent.ListenableFutureCallback;

import javax.annotation.Resource;

/**
 * @BelongsProject: kafka-module
 * @BelongsPackage: com.cuit.kafka.service
 * @Author: HouPJ
 * @Date: 2022/11/12 21:20
 * @Description: TODO
 */
@Slf4j
public class KafkaProducerServerApiImpl implements KafkaProducerApi {


    private  KafkaProducerUtil kafkaProducerUtil = new KafkaProducerUtil();

    @Override
    public void sendMessage(String topicName, String data) {
        kafkaProducerUtil.sendMessage(topicName,data);
    }

    @Override
    public void sendMessage(String topicName, String key, String data) {
        kafkaProducerUtil.sendMessage(topicName,key,data);
    }

    @Override
    public void sendMessage(String topicName, Integer partition, String key, Object data) {
        kafkaProducerUtil.sendMessage(topicName,partition,key,data);
    }

    /**
     * producer 异步方式发送数据(带回调)
     *
     * @param topic   topic名称
     * @param message producer发送的数据
     */
    @Override
    public void sendMessageWithCallBack(String topic, String message) {
        kafkaProducerUtil.sendMessage(topic, message ,new ListenableFutureCallback<SendResult<String, Object>>(){
            @Override
            public void onSuccess(SendResult<String, Object> sendResult) {
                log.info("发送消息成功！topic:{}, partition:{}, offset:{}, key:{}, value:{}",
                        sendResult.getRecordMetadata().topic(),
                        sendResult.getRecordMetadata().partition(),
                        sendResult.getRecordMetadata().offset(),
                        sendResult.getProducerRecord().key(),
                        sendResult.getProducerRecord().value());
            }
            @Override
            public void onFailure(Throwable throwable) {
                log.error("发送消息失败！errMsg:{}", throwable.getCause() ,throwable);
                throw new ErrorMessage("消息发送失败！");
            }
        });
    }

}
