package com.cuit.basin.base.redismq.config;

import com.cuit.basin.base.redismq.core.RedisMQTemplate;
import com.cuit.basin.base.redismq.core.pubsub.AbstractChannelMessageListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.listener.ChannelTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.scheduling.annotation.Async;

import java.util.List;


/**
 * @BelongsProject: steward
 * @BelongsPackage: com.yhxc.server.framework.redismq.config
 * @Author: HouPJ
 * @Date: 2022/12/21 10:29
 * @Description: TODO
 */
@Configuration(proxyBeanMethods = false)
@AutoConfigureAfter(RedisMQAutoConfiguration.class)
@EnableCaching
@Slf4j
public class RedisMQAutoConfiguration {

    /**
     * 获取redis消息模板类（此处构建的时StringRedisTemplate）
     *
     * @param redisTemplate RedisTemplate的一个子类
     * @return RedisMQTemplate
     */
    @Bean
    public RedisMQTemplate redisMQTemplate(StringRedisTemplate redisTemplate) {
        RedisMQTemplate redisMQTemplate = new RedisMQTemplate(redisTemplate);
        return redisMQTemplate;
    }

    // ========== 消费者相关 ==========

    /**
     * 创建 Redis Pub/Sub 广播消费的容器
     * @param redisMQTemplate redis mq 模板类（上步注入的模板）
     * @param listeners 可以有多个，AbstractChannelMessageListener的子类，且加了Component注解的listener都会被注册
     * @return RedisMessageListenerContainer
     */
    @Bean
    @Async // 异步化，可降低 2 秒左右的启动时间
    public RedisMessageListenerContainer redisMessageListenerContainer(
            RedisMQTemplate redisMQTemplate, List<AbstractChannelMessageListener<?>> listeners) {
        // 创建 RedisMessageListenerContainer 对象
        RedisMessageListenerContainer container = new RedisMessageListenerContainer();
        // 设置 RedisConnection 工厂。
        container.setConnectionFactory(redisMQTemplate.getRedisTemplate().getRequiredConnectionFactory());
        // 添加监听器
        listeners.forEach(listener -> {
            listener.setRedisMQTemplate(redisMQTemplate);
            container.addMessageListener(listener, new ChannelTopic(listener.getChannel()));
            log.info("[redisMessageListenerContainer][注册 Channel({}) 对应的监听器({})]",
                    listener.getChannel(), listener.getClass().getName());
        });
        return container;
    }


}
