package com.cuit.basin.base.websocket.core;

import com.cuit.basin.base.websocket.util.WebSocketUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;

/**
 * @BelongsProject: sriversys
 * @BelongsPackage: com.cuit.sriver.sriversys.server.websocket.core
 * @Author: HouPJ
 * @Date: 2022/12/17 14:28
 * @Description: TODO
 */
@ServerEndpoint(value = "/websocket")
@Component
@Slf4j
public class PgsWebSocket {

    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;
    /**
     * @Description: 连接建立成功调用的方法
     * @Author: HouPJ
     * @Date: 2022/12/17 14:28
     * @Param: [session]
     * @return: void
     **/
    @OnOpen
    public void onOpen(Session session) {
        this.session = session;
        WebSocketUtil.addSocket(this);
        log.info("有新连接{}加入！当前{}人在线", session.getId(), WebSocketUtil.getOnlineCount());
        try {
            sendMessage("欢迎加入！当前在线人数为" + WebSocketUtil.getOnlineCount());
        } catch (IOException e) {
            log.info("IO异常");
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        WebSocketUtil.removeSocket(this);
        log.info("有一连接关闭！当前还有{}人在线", WebSocketUtil.getOnlineCount());
    }

    /**
     * @Description: 收到客户端消息后调用的方法
     * @Author: HouPJ
     * @Date: 2022/12/17 14:28
     * @Param: [message, sessionCurrent]
     * @return: void
     **/
    @OnMessage
    public void onMessage(String message, Session sessionCurrent) {
        log.info("来自客户端{}的消息:{}", sessionCurrent.getId(), message);
        /**
         * 向客户端群发消息
         */
        for (PgsWebSocket item : WebSocketUtil.getWebSocketSet()) {
            try {
                //向发件人单独回应
                if (item.session.getId().equals(sessionCurrent.getId())) {
                    item.sendMessage("我是服务器，我已经收到你的信息---" + message);
                    // 对来自客户端的message做业务逻辑处理

                    return;
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * @Author: HouPJ
     * @Date: 2022/12/17 14:28
     * @Description: 发生错误时调用
     * @Param: [session, error]
     * @return: void
     **/
    @OnError
    public void onError(Session session, Throwable error) {
        log.info("关闭{}发生错误", session.getId());
        error.printStackTrace();
    }

    public void sendMessage(String message) throws IOException {
        //this.session.getBasicRemote().sendText(message);
        this.session.getAsyncRemote().sendText(message);
    }
}
