package com.cuit.sriver.sriversys.datadeal.dataetl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.cuit.sriver.sriversys.dao.BdTableB2Mapper;
import com.cuit.sriver.sriversys.dao.HydroStationDataMapper;
import com.cuit.sriver.sriversys.dao.HydroStationHistoryMapper;
import com.cuit.sriver.sriversys.dao.HydroStationInfoMapper;
import com.cuit.sriver.sriversys.datadeal.domain.BdTableB2;
import com.cuit.sriver.sriversys.entity.HydroStationData;
import com.cuit.sriver.sriversys.entity.HydroStationHistory;
import com.cuit.sriver.sriversys.entity.HydroStationInfo;
import lombok.Builder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @Description:
 * @Author LLW
 * @BelongsProject: sbasinsys1
 * @BelongsPackage: com.cuit.sriver.sriversys.datadeal.dataetl
 * @CreateTime: 2022/12/24 12:08:14
 * @Version 1.0
 */

@Slf4j
@Builder
@Component
public class GetProcessingData {
    @Autowired
    private BdTableB2Mapper bdTableB2Mapper;
    @Autowired
    private HydroStationHistoryMapper hydroStationHistoryMapper;
    @Autowired
    private HydroStationInfoMapper hydroStationInfoMapper;
    @Autowired
    private HydroStationDataMapper hydroStationDataMapper;

    /*
     * @description: 取出源数据表里相应数据
     * @author: LLW
     * @date: 2022/12/24 13:19
     * @param:
     * @return: list：源数据表里相应数据
     **/
    public Integer GetDataOut(String stCd,LocalDateTime sTime,LocalDateTime edTime) {
//        String stCd = "S1001";
//        LocalDateTime sTime = LocalDateTime.of(2020, 7, 1, 0, 0, 0);
//        LocalDateTime edTime = LocalDateTime.of(2020, 7, 15, 23, 0, 0);
        LambdaQueryWrapper<BdTableB2> queryWrapper = new LambdaQueryWrapper<>();
        /*将源数据按照时间升序排列封装成list：bdTableB2List*/
        queryWrapper.between(BdTableB2::getOdate, sTime, edTime).orderByAsc(BdTableB2::getOdate);
        List<BdTableB2> bdTableB2List = bdTableB2Mapper.selectList(queryWrapper);
        log.info("抓取数据条数："+bdTableB2List.size());
        /*取出对应站点历史数据：hydroStationHistory*/
        LambdaQueryWrapper<HydroStationHistory> queryWrapper2 = new LambdaQueryWrapper<>();
        queryWrapper2.eq(HydroStationHistory::getStcd, stCd);
        HydroStationHistory hydroStationHistory = hydroStationHistoryMapper.selectOne(queryWrapper2);
        /*修改历史最大值条件构造器*/
        LambdaUpdateWrapper<HydroStationHistory> updateWrapper = new LambdaUpdateWrapper<>();
        LambdaUpdateWrapper<HydroStationHistory> updateWrapper1 = new LambdaUpdateWrapper<>();
        /*取出对应站点的警戒、保证水位流量标准值*/
        LambdaQueryWrapper<HydroStationInfo> queryWrapper3 = new LambdaQueryWrapper<>();
        queryWrapper3.eq(HydroStationInfo::getStcd, stCd);
        HydroStationInfo hydroStationInfo = hydroStationInfoMapper.selectOne(queryWrapper3);
        //水位变化
        Double zc = 0.0;
        //流量变化
        Double qc = 0.0;
        //超警戒水位值
        Double bywrz = 0.0;
        //超警戒流量值
        Double bywrq = 0.0;
        //超保证水位值
        Double bygrz = 0.0;
        //超保证流量值
        Double bygrq = 0.0 ;
        //超历史水位值
        Double byhizs = 0.0;
        //超历史流量值
        Double byhisq = 0.0;
        //水位数据变化状态标志：（2：上升异常，1：上升，0：无变化，-1 下降， -2 下降异常）
        Integer ZCS = 0;
        //流量数据变化状态标志：（2：上升异常，1：上升，0：无变化，-1 下降， -2 下降异常）
        Integer QCS = 0;
        //超警戒水位状态标志：超警戒水位状态标志（2：超限，1：预警，0：安全）
        Integer BYWRZS = 0;
        //超警戒流量状态标志：超警戒水位状态标志（2：超限，1：预警，0：安全）
        Integer BYWRQS = 0;
        //超保证水位状态标志：超警戒水位状态标志（2：超限，1：预警，0：安全）
        Integer BYGRZS = 0;
        //超保证流量状态标志：超警戒水位状态标志（2：超限，1：预警，0：安全）
        Integer BYGRQS = 0;
        //超历史水位状态标志：超警戒水位状态标志（2：超限，1：预警，0：安全）
        Integer BYHISZS = 0;
        //超历史流量状态标志：超警戒水位状态标志（2：超限，1：预警，0：安全）
        Integer BYHISQS = 0;
        // 最终水位状态标志
        Integer ZS  = 0;
        // 最终流量状态标志
        Integer QS  = 0;
        int total = 0;
        for (int i = 1; i < bdTableB2List.size(); i++) {
            /*对相差1小时数据进行处理*/
            Duration duration = Duration.between(bdTableB2List.get(i - 1).getOdate(), bdTableB2List.get(i).getOdate());
            System.out.println(duration.toHours());
            if (duration.toHours() == 1) {
                /*计算水位变化*/
                zc = bdTableB2List.get(i).getWaterl() - bdTableB2List.get(i - 1).getWaterl();
                ZCS = judgmentStatusByZ(zc, hydroStationInfo.getZct());
                /*计算流量变化*/
                qc = bdTableB2List.get(i).getWaterf() - bdTableB2List.get(i - 1).getWaterf();
                QCS = judgmentStatusByZ(qc, hydroStationInfo.getQct());
            }
            /*计算超警戒水位值*/
            bywrz = bdTableB2List.get(i).getWaterl() - hydroStationInfo.getWrz();
            BYWRZS = judgmentStatusByWr(bywrz, 0.1);
            /*计算超警戒流量值*/
            bywrq = bdTableB2List.get(i).getWaterf() - hydroStationInfo.getWrq();
            BYWRQS = judgmentStatusByWr(bywrq, 10.0);
            /*计算超保证水位值*/
            bygrz = bdTableB2List.get(i).getWaterl() - hydroStationInfo.getGrz();
            BYGRZS = judgmentStatusByWr(bygrz, 0.1);
            /*计算超保证流量值*/
            bygrq = bdTableB2List.get(i).getWaterf() - hydroStationInfo.getGrq();
            BYGRQS = judgmentStatusByWr(bygrq, 10.0);
            /*计算超历史水位值*/
            byhizs = bdTableB2List.get(i).getWaterl() - hydroStationHistory.getHhtz();
            BYHISZS = judgmentStatusByWr(byhizs, 0.1);
            /*更新最新历史水位值*/
            if (byhizs > 0) {
                updateWrapper.eq(HydroStationHistory::getStcd, stCd).set(HydroStationHistory::getHltz, bdTableB2List.get(i).getWaterl())
                        .set(HydroStationHistory::getHhtztm, bdTableB2List.get(i).getOdate());

            }
            /*计算超历史流量值*/
            byhisq = bdTableB2List.get(i).getWaterf() - hydroStationHistory.getHmxq();
            BYHISQS = judgmentStatusByWr(byhisq, 10.0);
            /*更新最新历史流量值*/
            if (byhisq > 0) {
                updateWrapper1.eq(HydroStationHistory::getStcd, stCd).set(HydroStationHistory::getHmxq, bdTableB2List.get(i).getWaterf())
                        .set(HydroStationHistory::getHmxqtm, bdTableB2List.get(i).getOdate());
            }
            ZS = zOrqWarnStatusValueStrategy(ZCS,BYWRZS,BYGRZS,BYHISZS);
            QS = zOrqWarnStatusValueStrategy(QCS,BYWRQS,BYGRQS,BYHISQS);
            HydroStationData hydroStationData = HydroStationData.builder().dt(bdTableB2List.get(i).getOdate())
                    .stcd(stCd)
                    .z(bdTableB2List.get(i).getWaterl())
                    .q(bdTableB2List.get(i).getWaterf())
                    .zc(zc)
                    .qc(qc)
                    .bywrz(bywrz)
                    .bywrq(bywrq)
                    .bygrz(bygrz)
                    .bygrq(bygrq)
                    .byhisz(byhizs)
                    .byhisq(byhisq)
                    .zs(ZS)
                    .qs(QS)
                    .zcs(ZCS)
                    .qcs(QCS)
                    .bywrzs(BYWRZS)
                    .bywrqs(BYWRQS)
                    .bygrzs(BYGRZS)
                    .bygrqs(BYGRQS)
                    .byhiszs(BYHISZS)
                    .byhisqs(BYHISQS)
                    .build();
            hydroStationDataMapper.insert(hydroStationData);
            total++;
        }
        log.info("入库成功记录条数："+total);
        return total;
    }

    /**
     * 综合判定水位或流量的预警状态码  预警：超保证>超警戒>超历史>超变限>正常
     *
     * @param cs     变化状态
     * @param bywrs  超警戒状态
     * @param bygrs  超保证状态
     * @param byhiss 超历史状态
     * @return 水位或流量预警状态码 0：正常，1：超警戒，2：超保证，3：超历史，4：超变限
     */
    public static Integer zOrqWarnStatusValueStrategy(Integer cs, Integer bywrs, Integer bygrs, Integer byhiss) {
        if (bygrs == 2) {
            return 2;
        } else if (bywrs == 2) {
            return 1;
        } else if (byhiss == 2) {
            return 3;
        } else if (cs == 2) {
            return 4;
        } else {
            return 0;
        }
    }

    /*
     * @description: 判断水位、流量变化状态标志
     * @author: LLW
     * @date: 2022/12/25 11:58
     * @param: byct 阈值 /  byvalue 水位、流量变化值
     * @return:
     **/
    public int judgmentStatusByZ(Double byvalue, Double byct) {
        if (byvalue >= byct) {
            return 2;
        } else if (byvalue > 0) {
            return 1;
        } else if (byvalue == 0) {
            return 0;
        } else if (byvalue < 0) {
            return -1;
        } else {
            return -2;
        }
    }

    /*
     * @description: 判断超警戒、保证、历史状态标志
     * @author: LLW
     * @date: 2022/12/25 12:15
     * @param: bywarn 超限值 0.05 / bypledge 接近阈值 0.1
     * @return:
     **/
    public int judgmentStatusByWr(Double bywarn, Double bypledge) {

        if (bywarn > 0) {
            return 2;
        } else if (Math.abs(bywarn) - bypledge < 0 && Math.abs(bywarn) - bypledge > -0.1) {
            return 1;
        } else {
            return 0;
        }
    }
}

