package com.cuit.sriver.sriversys.datadeal.datastat;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cuit.sriver.sriversys.dao.HydBiStinfoBMapper;
import com.cuit.sriver.sriversys.dao.HydroStationDataMapper;
import com.cuit.sriver.sriversys.entity.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: asus
 * @date: 2022/11/16 15:32
 * @description:获取站点特征值信息
 */
@Component
public class GetStationsSpecialInfo {

    @Autowired
    HydroStationDataMapper hydroStationDataMapper;
    @Autowired
    HydBiStinfoBMapper hydBiStinfoBMapper;

    /**
     * @param list: 按指定条件筛选出的list
     * @return htz： list中的最高水位值
     * @Description: 返回传入list中最高水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getDayHtz(List<HydroStationData> list) {
        double htz = list.get(0).getZ();
        for (int i = 1; i < list.size(); i++) {
            if (htz < list.get(i).getZ()) {
                htz = list.get(i).getZ();
            }
        }
        return htz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthHtz： list中的最高水位值
     * @Description: 返回传入list中最高水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getMonthHtz(List<HydroDayCharacteristicValue> list) {
        double monthHtz = list.get(0).getHtz();
        for (int i = 1; i < list.size(); i++) {
            if (monthHtz < list.get(i).getHtz()) {
                monthHtz = list.get(i).getHtz();
            }
        }
        return monthHtz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return yearHtz： list中的最高水位值
     * @Description: 返回传入list中最高水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getYearHtz(List<HydroMonthCharacterLevelValue> list) {
        double yearHtz = list.get(0).getHtz();
        for (int i = 1; i < list.size(); i++) {
            if (yearHtz < list.get(i).getHtz()) {
                yearHtz = list.get(i).getHtz();
            }
        }
        return yearHtz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return htzTm： list中的最高水位值出现的时间
     * @Description: 返回传入list中最高水位值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getDayHtzTime(List<HydroStationData> list) {
        double dayHtz = list.get(0).getZ();
        LocalDateTime dayHtzTm = list.get(0).getDt();
        for (int i = 1; i < list.size(); i++) {
            if (dayHtz < list.get(i).getZ()) {
                dayHtz = list.get(i).getZ();
                dayHtzTm = list.get(i).getDt();
            }
        }
        return dayHtzTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthHtzTm： list中的最高水位值出现的时间
     * @Description: 返回传入list中最高水位值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getMonthHtzTime(List<HydroDayCharacteristicValue> list) {
        double monthHtz = list.get(0).getHtz();
        LocalDateTime monthHtzTm = list.get(0).getHtztm();
        for (int i = 1; i < list.size(); i++) {
            if (monthHtz < list.get(i).getHtz()) {
                monthHtz = list.get(i).getHtz();
                monthHtzTm = list.get(i).getHtztm();
            }
        }
        return monthHtzTm;
    }


    /**
     * @param list: 按指定条件筛选出的list
     * @return yearHtzTm： list中的最高水位值出现的时间
     * @Description: 返回传入list中最高水位值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public LocalDateTime getYearHtzTime(List<HydroMonthCharacterLevelValue> list) {
        double yearHtz = list.get(0).getHtz();
        LocalDateTime yearHtzTm = list.get(0).getHtztm();
        for (int i = 1; i < list.size(); i++) {
            if (yearHtz < list.get(i).getHtz()) {
                yearHtz = list.get(i).getHtz();
                yearHtzTm = list.get(i).getHtztm();
            }
        }
        return yearHtzTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getAvz： list中的平均水位值
     * @Description: 返回传入list中的平均水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getDayAvz(List<HydroStationData> list) {
        double getAvz = 0;
        for (int i = 0; i < list.size(); i++) {
            getAvz += list.get(i).getZ();
        }
        getAvz = getAvz / list.size();
        return getAvz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getMonthAvz： list中的平均水位值
     * @Description: 返回传入list中的平均水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getMonthAvz(List<HydroDayCharacteristicValue> list) {
        double getMonthAvz = 0;
        for (int i = 0; i < list.size(); i++) {
            getMonthAvz += list.get(i).getAvz();
        }
        getMonthAvz = getMonthAvz / list.size();
        return getMonthAvz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getYearAvz： list中的平均水位值
     * @Description: 返回传入list中的平均水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public double getYearAvz(List<HydroMonthCharacterLevelValue> list) {
        double getYearAvz = 0;
        for (int i = 0; i < list.size(); i++) {
            getYearAvz += list.get(i).getAvz();
        }
        getYearAvz = getYearAvz / list.size();
        return getYearAvz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return ltz： list中的最低水位值
     * @Description: 返回传入list中的最低水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getDayLtz(List<HydroStationData> list) {
        double ltz = list.get(0).getZ();
        for (int i = 1; i < list.size(); i++) {
            if (ltz > list.get(i).getZ()) {
                ltz = list.get(i).getZ();
            }
        }
        return ltz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthLtz： list中的最低水位值
     * @Description: 返回传入list中的最低水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getMonthLtz(List<HydroDayCharacteristicValue> list) {
        double monthLtz = list.get(0).getLtz();
        for (int i = 1; i < list.size(); i++) {
            if (monthLtz > list.get(i).getLtz()) {
                monthLtz = list.get(i).getLtz();
            }
        }
        return monthLtz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return yearLtz： list中的最低水位值
     * @Description: 返回传入list中的最低水位值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getYearLtz(List<HydroMonthCharacterLevelValue> list) {
        double yearLtz = list.get(0).getLtz();
        for (int i = 1; i < list.size(); i++) {
            if (yearLtz > list.get(i).getLtz()) {
                yearLtz = list.get(i).getLtz();
            }
        }
        return yearLtz;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return htzLm： list中的最低水位值出现的时间
     * @Description: 返回传入list中的最低水位值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public LocalDateTime getDayLtzTime(List<HydroStationData> list) {
        Double ltz = list.get(0).getZ();
        LocalDateTime htzLm = list.get(0).getDt();
        for (int i = 1; i < list.size(); i++) {
            if (ltz > list.get(i).getZ()) {
                ltz = list.get(i).getZ();
                htzLm = list.get(i).getDt();
            }
        }
        return htzLm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getHtzLm： list中的最低水位值出现的时间
     * @Description: 返回传入list中的最低水位值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public LocalDateTime getMonthLtzTime(List<HydroDayCharacteristicValue> list) {
        Double ltz = list.get(0).getLtz();
        LocalDateTime getHtzLm = list.get(0).getLtztm();
        for (int i = 1; i < list.size(); i++) {
            if (ltz > list.get(i).getLtz()) {
                ltz = list.get(i).getLtz();
                getHtzLm = list.get(i).getLtztm();
            }
        }
        return getHtzLm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return yearHtzLm： list中的最低水位值出现的时间
     * @Description: 返回传入list中的最低水位值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public LocalDateTime getYearLtzTime(List<HydroMonthCharacterLevelValue> list) {
        Double ltz = list.get(0).getLtz();
        LocalDateTime yearHtzLm = list.get(0).getLtztm();
        for (int i = 1; i < list.size(); i++) {
            if (ltz > list.get(i).getLtz()) {
                ltz = list.get(i).getLtz();
                yearHtzLm = list.get(i).getLtztm();
            }
        }
        return yearHtzLm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return mxq： list中的最大流量值
     * @Description: 返回传入list中的最大流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public double getDayFlowMxq(List<HydroStationData> list) {
        double mxq = list.get(0).getQ();
        for (int i = 1; i < list.size(); i++) {
            if (mxq < list.get(i).getQ()) {
                mxq = list.get(i).getQ();
            }
        }
        return mxq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthMxq： list中的最大流量值
     * @Description: 返回传入list中的最大流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public double getMonthFlowMxq(List<HydroDayCharacteristicValue> list) {
        double monthMxq = list.get(0).getMxq();
        for (int i = 1; i < list.size(); i++) {
            if (monthMxq < list.get(i).getMxq()) {
                monthMxq = list.get(i).getMxq();
            }
        }
        return monthMxq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return yearMxq： list中的最大流量值
     * @Description: 返回传入list中的最大流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getYearFlowMxq(List<HydroMonthCharacterFlowValue> list) {
        double yearMxq = list.get(0).getMxq();
        for (int i = 1; i < list.size(); i++) {
            if (yearMxq < list.get(i).getMxq()) {
                yearMxq = list.get(i).getMxq();
            }
        }
        return yearMxq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return mxqTm： list中的最大流量值出现的时间
     * @Description: 返回传入list中的最大流量值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getDayFlowMxqTm(List<HydroStationData> list) {
        Double mxq = list.get(0).getQ();
        LocalDateTime mxqTm = list.get(0).getDt();
        for (int i = 1; i < list.size(); i++) {
            if (mxq < list.get(i).getQ()) {
                mxq = list.get(i).getQ();
                mxqTm = list.get(i).getDt();
            }
        }
        return mxqTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthMxqTm： list中的最大流量值出现的时间
     * @Description: 返回传入list中的最大流量值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getMonthFlowMxqTm(List<HydroDayCharacteristicValue> list) {
        Double mxq = list.get(0).getMxq();
        LocalDateTime monthMxqTm = list.get(0).getMxqtm();
        for (int i = 1; i < list.size(); i++) {
            if (mxq < list.get(i).getMxq()) {
                mxq = list.get(i).getMxq();
                monthMxqTm = list.get(i).getMxqtm();
            }
        }
        return monthMxqTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return yearMxqTm： list中的最大流量值出现的时间
     * @Description: 返回传入list中的最大流量值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getYearFlowMxqTm(List<HydroMonthCharacterFlowValue> list) {
        Double mxq = list.get(0).getMxq();
        LocalDateTime yearMxqTm = list.get(0).getMxqtm();
        for (int i = 1; i < list.size(); i++) {
            if (mxq < list.get(i).getMxq()) {
                mxq = list.get(i).getMxq();
                yearMxqTm = list.get(i).getMxqtm();
            }
        }
        return yearMxqTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getAvq： list中的平均流量值
     * @Description: 返回传入list中的平均流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public double getDayFlowAvq(List<HydroStationData> list) {
        double getAvq = 0;
        for (int i = 0; i < list.size(); i++) {
            getAvq += list.get(i).getQ();
        }
        getAvq = getAvq / list.size();
        return getAvq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getMonthAvq： list中的平均流量值
     * @Description: 返回传入list中的平均流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */

    public double getMonthFlowAvq(List<HydroDayCharacteristicValue> list) {
        double getMonthAvq = 0;
        for (int i = 0; i < list.size(); i++) {
            getMonthAvq += list.get(i).getAvq();
        }
        getMonthAvq = getMonthAvq / list.size();
        return getMonthAvq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return getYearAvq： list中的平均流量值
     * @Description: 返回传入list中的平均流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getYearFlowAvq(List<HydroMonthCharacterFlowValue> list) {
        double getYearAvq = 0;
        for (int i = 0; i < list.size(); i++) {
            getYearAvq += list.get(i).getAvq();
        }
        getYearAvq = getYearAvq / list.size();
        return getYearAvq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return mnq： list中的最小流量值
     * @Description: 返回传入list中的最小流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getDayFlowMnq(List<HydroStationData> list) {
        double mnq = list.get(0).getQ();
        for (int i = 1; i < list.size(); i++) {
            if (mnq > list.get(i).getQ()) {
                mnq = list.get(i).getQ();
            }
        }
        return mnq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthMnq： list中的最小流量值
     * @Description: 返回传入list中的最小流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getMonthFlowMnq(List<HydroDayCharacteristicValue> list) {
        double monthMnq = list.get(0).getMnq();
        for (int i = 1; i < list.size(); i++) {
            if (monthMnq > list.get(i).getMnq()) {
                monthMnq = list.get(i).getMnq();
            }
        }
        return monthMnq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthMnq： list中的最小流量值
     * @Description: 返回传入list中的最小流量值
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public double getYearFlowMnq(List<HydroMonthCharacterFlowValue> list) {
        double yearMnq = list.get(0).getMnq();
        for (int i = 1; i < list.size(); i++) {
            if (yearMnq > list.get(i).getMnq()) {
                yearMnq = list.get(i).getMnq();
            }
        }
        return yearMnq;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return mnqTm： list中的最小流量值出现的时间
     * @Description: 返回传入list中的最小流量值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getDayFlowMnqTm(List<HydroStationData> list) {
        Double mnq = list.get(0).getQ();
        LocalDateTime mnqTm = list.get(0).getDt();
        for (int i = 1; i < list.size(); i++) {
            if (mnq > list.get(i).getQ()) {
                mnq = list.get(i).getQ();
                mnqTm = list.get(i).getDt();
            }
        }
        return mnqTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return monthMnqTm： list中的最小流量值出现的时间
     * @Description: 返回传入list中的最小流量值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getMonthFlowMnqTm(List<HydroDayCharacteristicValue> list) {
        Double mnq = list.get(0).getMnq();
        LocalDateTime monthMnqTm = list.get(0).getHtztm();
        for (int i = 1; i < list.size(); i++) {
            if (mnq > list.get(i).getHtz()) {
                mnq = list.get(i).getMnq();
                monthMnqTm = list.get(i).getHtztm();
            }
        }
        return monthMnqTm;
    }

    /**
     * @param list: 按指定条件筛选出的list
     * @return yearMnqTm： list中的最小流量值出现的时间
     * @Description: 返回传入list中的最小流量值出现的时间
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:10
     */
    public LocalDateTime getYearFlowMnqTm(List<HydroMonthCharacterFlowValue> list) {
        Double mnq = list.get(0).getMnq();
        LocalDateTime yearMnqTm = list.get(0).getMnqltm();
        for (int i = 1; i < list.size(); i++) {
            if (mnq > list.get(i).getMnql()) {
                mnq = list.get(i).getMnq();
                yearMnqTm = list.get(i).getMnqltm();
            }
        }
        return yearMnqTm;
    }

    /**
     * @param
     * @return
     * @Description: 获取不重复的所有站点名
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/18 9:27
     */
    public List<String> getAllStationsName() {
        QueryWrapper<HydBiStinfoB> queryWrapper = new QueryWrapper<>();
        List<String> differentStcd = hydBiStinfoBMapper.selectList(queryWrapper)
                .stream().map(HydBiStinfoB::getStcd).collect(Collectors.toList());
        differentStcd.forEach(System.out::println);
        return differentStcd;
    }
}
