package com.cuit.sriver.sriversys.datadeal.datastat;

import com.cuit.sriver.sriversys.entity.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @BelongsProject: sriver
 * @BelongsPackage: com.cuit.sriver.sriversys.utils
 * @Author: tianjunbiao
 * @CreateTime: 2022/11/16 23:10
 * @Description:获取满足指定年、月、日的站点特征值实体类
 * @Version: 1.0
 */

@Component

public class PackagingClass {

    @Autowired
    GetStationsSpecialInfo getStationsSpecialInfo;

    /**
     * @param list:按日筛选条件得到的list
     * @return hydroDayCharacteristicValue：一个日特征值实体类
     * @Description: 从筛选出来的list中调用相应的计算方法返回一个HydroDayCharacteristicValue实体类
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/16 23:19
     */
    public HydroDayCharacteristicValue getDaySpecialInfo(@NotNull List<HydroStationData> list) {
        HydroDayCharacteristicValue hydroDayCharacteristicValue = HydroDayCharacteristicValue.builder()
                .stcd(list.get(0).getStcd())
                .yr(list.get(0).getDt().getYear())
                .mnth(list.get(0).getDt().getMonthValue())
                .day(list.get(0).getDt().getDayOfMonth())
                .htz(getStationsSpecialInfo.getDayHtz(list))
                .htztm(getStationsSpecialInfo.getDayHtzTime(list))
                .avz(getStationsSpecialInfo.getDayAvz(list))
                .ltz(getStationsSpecialInfo.getDayLtz(list))
                .ltztm(getStationsSpecialInfo.getDayLtzTime(list))
                .mxq(getStationsSpecialInfo.getDayFlowMxq(list))
                .mxqtm(getStationsSpecialInfo.getDayFlowMxqTm(list))
                .avq(getStationsSpecialInfo.getDayFlowAvq(list))
                .mnq(getStationsSpecialInfo.getDayFlowMnq(list))
                .mnqtm(getStationsSpecialInfo.getDayFlowMnqTm(list))
                .build();
        return hydroDayCharacteristicValue;
    }

    /**
     * @param monthList:按月筛选条件得到的list；earlyList：上旬list；midList：中旬list；lateList：下旬list
     * @return hydroMonthCharacterLevelValue：一个月特征值水量实体类
     * @Description: 从筛选出来的list中调用相应的计算方法返回一个HydroMonthCharacterLevelValue实体类
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/16 23:19
     */
    public HydroMonthCharacterLevelValue getMonthSpecialInfo(@NotNull List<HydroDayCharacteristicValue> monthList, @NotNull List<HydroDayCharacteristicValue> earlyList, @NotNull List<HydroDayCharacteristicValue> midList, @NotNull List<HydroDayCharacteristicValue> lateList) {
        if (monthList.size() != 0) {
            HydroMonthCharacterLevelValue hydroMonthCharacterLevelValue = HydroMonthCharacterLevelValue.builder()
                    .stcd(monthList.get(monthList.size() - 1).getStcd())
                    .yr(monthList.get(monthList.size() - 1).getYr())
                    .mnth(monthList.get(monthList.size() - 1).getMnth())
                    .htz(getStationsSpecialInfo.getMonthHtz(monthList))
                    .htztm(getStationsSpecialInfo.getMonthHtzTime(monthList))
                    .avz(getStationsSpecialInfo.getMonthAvz(monthList))
                    .ltz(getStationsSpecialInfo.getMonthLtz(monthList))
                    .ltztm(getStationsSpecialInfo.getMonthLtzTime(monthList))
                    //上旬水位和时间
                    .htze(getStationsSpecialInfo.getMonthHtz(earlyList))
                    .htzetm(getStationsSpecialInfo.getMonthHtzTime(earlyList))
                    .avze(getStationsSpecialInfo.getMonthAvz(earlyList))
                    .ltze(getStationsSpecialInfo.getMonthLtz(earlyList))
                    .ltzetm(getStationsSpecialInfo.getMonthLtzTime(earlyList))
                    //中旬水位和时间
                    .htzm(getStationsSpecialInfo.getMonthHtz(midList))
                    .htzmtm(getStationsSpecialInfo.getMonthHtzTime(midList))
                    .avzm(getStationsSpecialInfo.getMonthAvz(midList))
                    .ltzm(getStationsSpecialInfo.getMonthLtz(midList))
                    .ltzmtm(getStationsSpecialInfo.getMonthLtzTime(midList))
                    //下旬水位和时间
                    .htzl(getStationsSpecialInfo.getMonthHtz(lateList))
                    .htzltm(getStationsSpecialInfo.getMonthHtzTime(lateList))
                    .avzl(getStationsSpecialInfo.getMonthAvz(lateList))
                    .ltzl(getStationsSpecialInfo.getMonthLtz(lateList))
                    .ltzltm(getStationsSpecialInfo.getMonthLtzTime(lateList))
                    .build();
            return hydroMonthCharacterLevelValue;
        }
        return null;
    }

    /**
     * @param monthList:按月筛选条件得到的list；earlyList：上旬list；midList：中旬list；lateList：下旬list
     * @return hydroMonthCharacterFlowValue：一个月特征值流量实体类
     * @Description: 从筛选出来的list中调用相应的计算方法返回一个HydroMonthCharacterFlowValue实体类
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/16 23:19
     */
    public HydroMonthCharacterFlowValue getMonthFlowSpecialInfo(@NotNull List<HydroDayCharacteristicValue> monthList, @NotNull List<HydroDayCharacteristicValue> earlyList, @NotNull List<HydroDayCharacteristicValue> midList, @NotNull List<HydroDayCharacteristicValue> lateList) {
        if (monthList.size() != 0) {
            HydroMonthCharacterFlowValue hydroMonthCharacterFlowValue = HydroMonthCharacterFlowValue.builder()
                    .stcd(monthList.get(monthList.size() - 1).getStcd())
                    .yr(monthList.get(monthList.size() - 1).getYr())
                    .mnth(monthList.get(monthList.size() - 1).getMnth())
                    .mxq(getStationsSpecialInfo.getMonthFlowMxq(monthList))
                    .mxqtm(getStationsSpecialInfo.getMonthFlowMxqTm(monthList))
                    .avq(getStationsSpecialInfo.getMonthFlowAvq(monthList))
                    .mnq(getStationsSpecialInfo.getMonthFlowMnq(monthList))
                    .mnqtm(getStationsSpecialInfo.getMonthFlowMnqTm(monthList))
                    //上旬流量和时间
                    .mxqe(getStationsSpecialInfo.getMonthFlowMnq(earlyList))
                    .mxqetm(getStationsSpecialInfo.getMonthFlowMxqTm(earlyList))
                    .avqe(getStationsSpecialInfo.getMonthFlowAvq(earlyList))
                    .mnqe(getStationsSpecialInfo.getMonthFlowMnq(earlyList))
                    .mnqetm(getStationsSpecialInfo.getMonthFlowMnqTm(earlyList))
                    //中旬流量和时间
                    .mxqm(getStationsSpecialInfo.getMonthFlowMnq(midList))
                    .mxqmtm(getStationsSpecialInfo.getMonthFlowMxqTm(midList))
                    .avqm(getStationsSpecialInfo.getMonthFlowAvq(midList))
                    .mnqm(getStationsSpecialInfo.getMonthFlowMnq(midList))
                    .mnqmtm(getStationsSpecialInfo.getMonthFlowMnqTm(midList))
                    //下旬流量和时间
                    .mxql(getStationsSpecialInfo.getMonthFlowMnq(lateList))
                    .mxqltm(getStationsSpecialInfo.getMonthFlowMxqTm(lateList))
                    .avql(getStationsSpecialInfo.getMonthFlowAvq(lateList))
                    .mnql(getStationsSpecialInfo.getMonthFlowMnq(lateList))
                    .mnqltm(getStationsSpecialInfo.getMonthFlowMnqTm(lateList))
                    .build();
            return hydroMonthCharacterFlowValue;
        }
        return null;
    }

    /**
     * @param levelValueList:按年筛选条件得到的水量list；flowValueList:按年筛选条件得到的流量list，
     * @return hydroYearCharacteristicValue：一年特征值实体类
     * @Description: 从筛选出来的list中调用相应的计算方法返回一个HydroYearCharacteristicValue实体类
     * @version v1.0
     * @author tianjunbiao
     * @date 2022/11/16 23:19
     */
    public HydroYearCharacteristicValue getYearSpecialInfo(@NotNull List<HydroMonthCharacterLevelValue> levelValueList, @NotNull List<HydroMonthCharacterFlowValue> flowValueList) {
        if (levelValueList.size() != 0) {
            HydroYearCharacteristicValue hydroYearCharacteristicValue = HydroYearCharacteristicValue.builder()
                    .stcd(levelValueList.get(levelValueList.size() - 1).getStcd())
                    .yr(levelValueList.get(levelValueList.size() - 1).getYr())
                    .htz(getStationsSpecialInfo.getYearHtz(levelValueList))
                    .htztm(getStationsSpecialInfo.getYearHtzTime(levelValueList))
                    .avz(getStationsSpecialInfo.getYearAvz(levelValueList))
                    .ltz(getStationsSpecialInfo.getYearLtz(levelValueList))
                    .ltztm(getStationsSpecialInfo.getYearLtzTime(levelValueList))
                    .mxq(getStationsSpecialInfo.getYearFlowMxq(flowValueList))
                    .mxqtm(getStationsSpecialInfo.getYearFlowMxqTm(flowValueList))
                    .avq(getStationsSpecialInfo.getYearFlowAvq(flowValueList))
                    .mnq(getStationsSpecialInfo.getYearFlowMnq(flowValueList))
                    .mnqtm(getStationsSpecialInfo.getYearFlowMnqTm(flowValueList))
                    .build();
            return hydroYearCharacteristicValue;
        }
        return null;
    }
}
