package com.cuit.sriver.sriversys.pojo;

import com.alibaba.fastjson2.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.github.jeffreyning.mybatisplus.anno.MppMultiId;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.time.LocalDateTime;

@Data
@ApiModel(value = "WaterGazetteData", description = "水情公报对应数据类")
/**
 * 为空属性、字段不返回
 */
@JsonInclude(value = JsonInclude.Include.NON_NULL)
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class WaterGazetteData {
    /**
     * 站号
     */
    @ApiModelProperty(value = "站号")
    @TableId
    @JSONField(jsonDirect = true)
    private String stcd;
    /**
     * 站名
     */
    @ApiModelProperty(value = "站名")
    @JSONField(jsonDirect = true)
    private String stnm;
    /**
     * 流域名称
     */
    @ApiModelProperty(value = "流域名称")
    private String bsnm;
    /**
     * 数据时间
     */
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "数据时间")
    @MppMultiId
    private LocalDateTime dt;

    /**
     * 水位
     */
    @ApiModelProperty(value = "水位")
    private Double z;
    /**
     * 流量
     */
    @ApiModelProperty(value = "流量")
    private Double q;

    /**
     * 水位数据状态标志
     */
    @ApiModelProperty(value = "水位数据状态标志：0正常，1超警戒，2超保证，3超历史，4超变限")
    private int zs;
    /**
     * 流量数据状态标志
     */
    @ApiModelProperty(value = "流量数据状态标志：0正常，1超警戒，2超保证，3超历史，4超变限")
    private int qs;
    /**
     * 超警戒水位
     */
    @ApiModelProperty(value = "超警戒水位数值")
    private int bywrz;
    /**
     * 超警戒流量
     */
    @ApiModelProperty(value = "超警戒流量数值")
    private int bywrq;

    @Override
    public String toString() {
        return "WaterGazetteData{" +
                "stcd='" + stcd  +
                ", stnm='" + stnm  +
                ", bsnm='" + bsnm  +
                ", dt=" + dt +
                ", z=" + z +
                ", q=" + q +
                ", zs=" + zs +
                ", qs=" + qs +
                ", bywrz=" + bywrz +
                ", bywrq=" + bywrq +
                '}';
    }
}
