package com.cuit.basin.model.entity; /***********************************************************************
 * Module:  HydroStarionData.java
 * Author:  PYJ
 * Purpose: Defines the Class HydroStarionData
 ***********************************************************************/

import com.baomidou.mybatisplus.annotation.TableName;
import com.cuit.basin.framework.common.constant.Constant;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.github.jeffreyning.mybatisplus.anno.MppMultiId;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDateTime;

/**
 * 水文站点数据
 *
 * @author pyj
 */
@Builder
@ApiModel(value = "水文站点数据", description = "")
@Data
@AllArgsConstructor
@NoArgsConstructor
@TableName("hyd_ri_stdatal_r")
public class HydroStationData {
    /**
     * 数据时间
     */
    @JsonFormat(pattern = Constant.PATTERN_DATETIME, timezone = Constant.TIMEZONE)
    @ApiModelProperty(value = "数据时间")
    @MppMultiId
    private LocalDateTime dt;

    /**
     * 站号
     */
    @ApiModelProperty(value = "站号")
    @MppMultiId
    private String stcd;
    /**
     * 水位
     */
    @ApiModelProperty(value = "水位")
    private Double z;
    /**
     * 流量
     */
    @ApiModelProperty(value = "流量")
    private Double q;
    /**
     * 水位变化
     */
    @ApiModelProperty(value = "水位变化数值")
    private Double zc;
    /**
     * 流量变化
     */
    @ApiModelProperty(value = "流量变化数值")
    private Double qc;
    /**
     * 水位数据状态标志
     */
    @ApiModelProperty(value = "水位数据状态标志：0正常，1超警戒，2超保证，3超历史，4超变限")
    private int zs;
    /**
     * 流量数据状态标志
     */
    @ApiModelProperty(value = "流量数据状态标志：0正常，1超警戒，2超保证，3超历史，4超变限")
    private int qs;
    /**
     * 水位数据变化状态标志
     */
    @ApiModelProperty(value = "水位数据变化状态标志:1是上升，2是上升超限，0：无变化,-1下降，-2下降异常")
    private int zcs;
    /**
     * 流量数据变化状态标志
     */
    @ApiModelProperty(value = "流量数据变化状态标志:1是上升，2是上升超限，0：无变化,-1下降，-2下降异常")
    private int qcs;
    /**
     * 超警戒水位
     */
    @ApiModelProperty(value = "超警戒水位数值")
    private int bywrz;
    /**
     * 超保证水位
     */
    @ApiModelProperty(value = "超保证水位数值")
    private int bygrz;
    /**
     * 超警戒流量
     */
    @ApiModelProperty(value = "超警戒流量数值")
    private int bywrq;
    /**
     * 超保证流量
     */
    @ApiModelProperty(value = "超保证流量数值")
    private int bygrq;
}