package com.cuit.basin.model.handler;

import cn.hutool.core.exceptions.ExceptionUtil;
import com.cuit.basin.base.api.log.ErrorLogApi;
import com.cuit.basin.base.api.log.dto.ApiErrorLog;
import com.cuit.basin.framework.common.constant.ModuleEnum;
import com.cuit.basin.framework.kafka.core.api.KafkaProducerApi;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * @BelongsProject: sbasinsys
 * @BelongsPackage: com.cuit.sriver.sriversys.base.handler
 * @Author: HouPJ
 * @Date: 2022/12/22 9:52
 * @Description: 全局异常处理器
 */
@Slf4j
@Component
@AllArgsConstructor
public class GlobalExceptionHandler {


    private final ErrorLogApi errorLogApi;

    @Resource
    private KafkaProducerApi kafkaProducerServer;

    public void createExceptionLog(Throwable e) {
        // 插入错误日志
        ApiErrorLog errorLog = new ApiErrorLog();
        try {
            log.error("[GlobalExceptionHandler]：receive: {}]", e.getMessage());
            // 初始化 errorLog
            initExceptionLog(errorLog, e);
            // 记录异常日志
            errorLogApi.createErrorLog(errorLog);
        } catch (Throwable th) {
            th.printStackTrace();
            log.error("[createExceptionLog] log({}) 发生异常]", errorLog, th);
        }
    }

    private void initExceptionLog(ApiErrorLog errorLog, Throwable e) {
        // 设置默认值
        Integer userId = 14;
        errorLog.setUserId(Long.valueOf(userId));
        StackTraceElement[] stackTraceElements = e.getStackTrace();
        StackTraceElement stackTraceElement = stackTraceElements[0];
        // 本部分异常均为模型运行相关
        errorLog.setModule(ModuleEnum.MODEL_MANAGE.getType());
        // 设置异常字段
        errorLog.setExceptionName(e.getClass().getName());
        errorLog.setExceptionMessage(ExceptionUtil.getMessage(e));
        errorLog.setExceptionRootCauseMessage(ExceptionUtil.getRootCauseMessage(e));
        errorLog.setExceptionStackTrace("待开发补充");
        Assert.notEmpty(stackTraceElements, "异常 stackTraceElements 不能为空");
        errorLog.setExceptionClassName(stackTraceElement.getClassName());
        errorLog.setExceptionFileName(stackTraceElement.getFileName());
        errorLog.setExceptionMethodName("模型任务消息消费");
        errorLog.setExceptionLineNumber(stackTraceElement.getLineNumber());
        // 设置其它字段
        errorLog.setApplicationName("model-server");
        errorLog.setUserAgent("待开发补充");
        errorLog.setUserIp("模型服务IP");
        errorLog.setExceptionTime(LocalDateTime.now());
    }

}
