package com.cuit.basin.model.job;

import com.cuit.basin.model.dao.ModelDicMapper;
import com.cuit.basin.model.entity.ModelDic;
import com.cuit.basin.model.utils.QuartzBean;
import com.cuit.basin.model.utils.QuartzUtils;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.Scheduler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;

@Service
@Slf4j
public class StartModelJob extends QuartzJobBean {
    @Autowired
    JobList jobList;                //将jobList自动注入进来

    //注入任务调度
    @Resource
    private Scheduler scheduler;
    @Resource
    private ModelDicMapper modelDicMapper;

    /**
     * 开始数据Job
     *
     * @param jobExecutionContext
     */
    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) {
        if (jobList.getJoblist() != null && jobList.getJoblist().size() > 0) {
            for (int i = 0; i < jobList.getJoblist().size(); i++) {
                try {
                    JobConfig jobConfig = jobList.getJoblist().get(i);
                    if (jobConfig.isActive()) {
                        // 从数据库里面取name和cron
                        ModelDic modelInfo = modelDicMapper.selectModelInfo(jobConfig.getName());
                        if (ObjectUtils.isEmpty(modelInfo) || ObjectUtils.isEmpty(modelInfo.getCron())) {
                            log.warn("数据库中模型名[" + jobConfig.getName() + "]不存在或定时表达式为空，将使用配置文件信息。");
                        } else {
                            jobConfig.setCron(modelInfo.getCron());
                        }
                        QuartzBean quartzBean = new QuartzBean();
                        quartzBean.setJobClass(jobConfig.getJob());
                        quartzBean.setJobName(jobConfig.getName());
                        quartzBean.setCronExpression(jobConfig.getCron());
                        QuartzUtils.createScheduleJob(scheduler, quartzBean);
                        log.info("已启动定时任务（" + (i + 1) + "）:" + jobConfig.toString());
                    }

                } catch (Exception e) {
                    log.debug("创建Job失败");
                }
            }
        } else {
            log.debug("Job定义为空！");
        }
    }
}