package com.cuit.basin.model.job.handler;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.basin.framework.common.exception.ErrorMessage;
import com.cuit.basin.framework.common.utils.file.SSHRemoteCall;
import com.cuit.basin.model.dao.*;
import com.cuit.basin.model.entity.*;
import com.cuit.basin.model.job.data.ClimateForcing;
import com.cuit.basin.model.job.data.DischargeObs;
import com.cuit.basin.model.job.data.FileCalibrationResult;
import com.cuit.basin.model.job.data.FileDataResult;
import com.cuit.basin.model.job.params.GBHMBasedata;
import com.cuit.basin.model.job.params.GBHMModelData;
import com.cuit.basin.model.job.params.GBHMOutputData;
import com.cuit.basin.model.parse.ModelTaskManager;
import com.cuit.basin.model.pojo.InitTaskParams;
import com.cuit.basin.model.service.ModelTaskService;
import lombok.extern.slf4j.Slf4j;
import org.rosuda.REngine.REXPMismatchException;
import org.rosuda.REngine.REngineException;
import org.rosuda.REngine.Rserve.RConnection;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.util.StopWatch;

import javax.annotation.Resource;
import java.io.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * GBHM模型数据处理
 *
 * @author ZengwuWang
 */
@Component(value = "GBHM")
@Slf4j
public class GBHMProcess implements ApiModelProcess {

    @Resource
    private ModelParamMapper modelParamMapper;
    @Resource
    private MeteStationDataMapper meteStationDataMapper;
    @Resource
    private HydroStationDataMapper hydroStationDataMapper;
    @Resource
    private HydroStationInfoMapper hydroStationInfoMapper;
    @Resource
    private MeteStationInfoMapper meteStationInfoMapper;
    @Resource
    private HydSiForeaccuSMapper hydSiForeaccuSMapper;
    @Resource
    private HydSiForeaccuDMapper hydSiForeaccuDMapper;
    @Resource
    private HydFiFittedDMapper hydFiFittedDMapper;
    @Resource
    private HydFiStdataFMapper hydFiStdataFMapper;
    @Resource
    private ModelTaskService taskService;

    String ipAddress = "123.60.45.144";// ip地址
    String userName = "root";// 账号
    String password = "Wzw@Wyq!123";// 密码
    SSHRemoteCall remote = SSHRemoteCall.getInstance();

    String pathUp = "src/main/resources/txt/upload/";
    String pathDown = "src/main/resources/txt/download/";

    /**
     * 运行GBHM模型
     */
    @Override
    @Async
    public void runModel(InitTaskParams params) {
        log.info("[GBHM-model]：[当前运行线程为：{},模型运行方式为：{}]",Thread.currentThread().getName(),params.getRm());
        launchGBHMModel(params);
        log.info("[GBHM-model]:[任务{}定时运行结束！]", params.getId());
        // 移除全局变量 map 中的当前任务
        ModelTaskManager.modelTaskRunFlag.remove(params.getId());
    }

    public void launchGBHMModel(InitTaskParams params) {
        try {
            //1、获取模型所需数据
            GBHMBasedata gbhmBasedata = null;
            if (ModelTaskManager.modelTaskRunFlag.get(params.getId())) {
                gbhmBasedata = getBaseDataFromDatabase();
                taskService.sendModelTaskProcess(params, "20.00");
            }
            //2、模型数据前置处理
            if (ModelTaskManager.modelTaskRunFlag.get(params.getId())) {
                getModelFrontData(gbhmBasedata);
                taskService.sendModelTaskProcess(params, "45.00");
            }
            //3、运行模型,得到输出数据
            if (ModelTaskManager.modelTaskRunFlag.get(params.getId())) {
                runModelAndGetData(GBHMModelData.builder()
                        .dataTime(gbhmBasedata.getDataTime()).stTime(gbhmBasedata.getStTime())
                        .endTime(gbhmBasedata.getEndTime()).build());
                taskService.sendModelTaskProcess(params, "80.00");
            }
            //4、处理模型输出数据
            if (ModelTaskManager.modelTaskRunFlag.get(params.getId())) {
                GBHMOutputData dataProcessed = processModelOutputData();
                taskService.sendModelTaskProcess(params, "85.00");
            }
            //5、处理后模型数据入库
            //saveProcessedModelOutputData(dataProcessed, gbhmBasedata);
            //6、数据统计
            if (ModelTaskManager.modelTaskRunFlag.get(params.getId())) {
                // 休眠10s
                Thread.sleep(10000);
                taskService.sendModelTaskProcess(params, "100.00");
            }
        } catch (IOException | REngineException | REXPMismatchException e) {
            e.printStackTrace();
            throw new ErrorMessage("model-exception:" + e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ErrorMessage("model:exception:" + e.getMessage());
        }
    }

    /**
     * 1、获取模型所需数据
     *
     * @return
     */
    public GBHMBasedata getBaseDataFromDatabase() {

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        log.info("【1】正在准备GBHM模型基础数据....");
        // 读取模型参数和时间
        List<ModelParam> modelParams = modelParamMapper.selectList(new LambdaQueryWrapper<ModelParam>()
                .eq(ModelParam::getMid, 1));
//        LocalDateTime dataTime = LocalDateTime.now();
//        LocalDateTime stTime = nowTime.minusDays(15);
//        LocalDateTime endTime = nowTime.plusDays(5);
        LocalDateTime dataTime = LocalDateTime.of(2021, 7, 15, 0, 0, 0);
        LocalDateTime stTime = LocalDateTime.of(2021, 7, 1, 0, 0, 0);
        LocalDateTime endTime = LocalDateTime.of(2021, 7, 31, 23, 59, 59);
        modelParams.forEach(e -> log.info("读取到GBHM模型参数：name=" + e.getName() + ", maxv=" + e.getMaxv() + "minv=" + e.getMinv() + ", value=" + e.getValue()));
        log.info("读取到GBHM模型开始时间：" + stTime + "，结束时间：" + endTime);
        // 读取模型所需的数据
        List<String> stcdMete = meteStationInfoMapper.selectList(new LambdaQueryWrapper<>()).stream().map(MeteStationInfo::getStcd).collect(Collectors.toList());
        List<String> stcdHydro = hydroStationInfoMapper.selectList(new LambdaQueryWrapper<>()).stream().map(HydroStationInfo::getStcd).collect(Collectors.toList());
        List<MeteStationData> meteStationData = meteStationDataMapper.selectList(new LambdaQueryWrapper<MeteStationData>()
                .ge(MeteStationData::getDt, stTime.format(df))
                .le(MeteStationData::getDt, endTime.format(df)));
        List<HydroStationData> hydroStationData = hydroStationDataMapper.selectList(new LambdaQueryWrapper<HydroStationData>()
                .ge(HydroStationData::getDt, stTime.format(df))
                .le(HydroStationData::getDt, endTime.format(df)));
        log.info("读取到GBHM模型气象数据" + meteStationData.size() + "条，水文数据" + hydroStationData.size() + "条。");
        // MQ发消息
        // ...

        return GBHMBasedata.builder()
                .params(modelParams).dataTime(dataTime).stTime(stTime).endTime(endTime)
                .stcdMeteList(stcdMete).stcdHydroList(stcdHydro)
                .meteStationDataList(meteStationData).hydroStationDataList(hydroStationData)
                .build();
    }

    /**
     * 2、模型数据前置处理
     *
     * @param data
     * @return
     */
    public void getModelFrontData(GBHMBasedata data) throws Exception {

        log.info("【2】正在开始GBHM模型前置处理....");
        DateTimeFormatter dfMete = DateTimeFormatter.ofPattern("yyyy/M/d H:mm");
        DateTimeFormatter dfHydro = DateTimeFormatter.ofPattern("yyyy/M/d H:mm");
        // 1、将GBHM模型的开始时间、结束时间以及模型参数存入配置文件中
        List<ModelParam> paramList = data.getParams();

        // 2、将水文数据和气象数据按照格式转为txt文件
        List<MeteStationData> meteStationData = data.getMeteStationDataList();
        List<HydroStationData> hydroStationData = data.getHydroStationDataList();

        // 气象数据
        for (String eachMS : data.getStcdMeteList()) {
            List<ClimateForcing> climateForcingList = new ArrayList<>();
            for (MeteStationData each : meteStationData) {
                if (each.getStcd().equals(eachMS)) {
                    climateForcingList.add(ClimateForcing.builder()
                            .time(each.getDt().format(dfMete)).wind(each.getWs2m())
                            .rain(each.getPre10min()).temperature(each.getTem()).relativeHumidity(each.getRh())
                            .build());
                }
            }
            saveDataToTxt(pathUp, "qixiangData", eachMS, climateForcingList,
                    e -> e.getTime() + "\t" + e.getWind() + "\t" + e.getRain() + "\t" + e.getTemperature() + "\t" + e.getRelativeHumidity());
        }
        log.info("GBHM模型气象数据TXT已生成。");
        // 水文数据
        for (String eachHS : data.getStcdHydroList()) {
            List<DischargeObs> dischargeObsList = new ArrayList<>();
            for (HydroStationData each : hydroStationData) {
                if (each.getStcd().equals(eachHS)) {
                    dischargeObsList.add(DischargeObs.builder()
                            .time(each.getDt().format(dfHydro)).flow(each.getQ())
                            .build());
                }
            }
//            saveDataToTxt(pathUp, "obs_hourly", eachHS, dischargeObsList,
//                    e -> e.getTime() + "\t" + e.getFlow());
        }
        log.info("GBHM模型水文数据TXT已生成。");

        // 1、首先远程连接ssh
        remote.sshRemoteCallLogin(ipAddress, userName, password);
        // 打印信息
        log.info("连接ip地址: " + ipAddress + ",账号: " + userName + ",连接成功.....");
        // 3、上传文件
        File file = new File(pathUp);
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            for (File eachF : files) {
                // 目标文件名
                String directory = "/root/docker/Rfunction/GBHM20221223/GBHM/data/" + eachF.getName();
                // 本地文件名
                String uploadFile = pathUp + eachF.getName();
                remote.uploadFile(directory, uploadFile);
            }
        }
        log.info("GBHM模型数据TXT已上传。");
    }

    /**
     * 运行模型,得到输出数据
     *
     * @param data
     * @return
     */
    public void runModelAndGetData(GBHMModelData data) throws REngineException, REXPMismatchException {

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-M-d");
        StopWatch watch = new StopWatch();
        watch.start("water");
        log.info("【3】正在运行GBHM模型....");
        // Rserve远程连接
        RConnection rc = new RConnection("123.60.45.144", 8088);
        // 运行模型
        rc.parseAndEval("setwd(dir = '/r_script/GBHM20221223/GBHM/')");
        rc.parseAndEval("source('step1_processClimateData.R')");
        log.info("step1 前置处理已完成...");
        rc.assign("startDate", df.format(data.getStTime()));
        rc.assign("endDate", df.format(data.getEndTime()));
        rc.parseAndEval("source('step2_runModel.R')");
        log.info("step2 模型运行已完成...");
        rc.parseAndEval("source('step3_postProcess.R')");
        log.info("step3 精度评估已完成...");
        // 关闭连接
        rc.close();
        log.info("GBHM模型运行结束....");
        watch.stop();
        log.info("运行模型耗时：" + String.format("%.2f", watch.getTotalTimeSeconds() / 60) + "分。");
    }

    /**
     * 4、处理模型输出数据
     *
     * @return
     */
    public GBHMOutputData processModelOutputData() throws Exception {

//        String srcOutput = "/root/docker/Rfunction/GBHM20221223/GBHM/GBHM_Jialewan_450/Output/";
        String srcCalibration = "/root/docker/Rfunction/GBHM20221223/GBHM/GBHM_Jialewan_450/Calibration/";
        log.info("【4】正在处理GBHM模型输出数据....");

        // 1、首先远程连接ssh
        remote.sshRemoteCallLogin(ipAddress, userName, password);

        String retOutput = remote.execCommand("ls " + srcCalibration);
        String retCalibration = remote.execCommand("ls " + srcCalibration);
        String regxDrw = "(DrwHourly[0-9]+\\.txt)|(DrwHourly[a-zA-Z]+[0-9]+\\.txt)";
        String regxDischarge = "(DischargeHourly[0-9]+\\.txt)|(DischargeHourly[a-zA-Z]+[0-9]+\\.txt)";
        String regxCalibration = "assess.txt";
        Pattern pDrw = Pattern.compile(regxDrw);
        Pattern pDischarge = Pattern.compile(regxDischarge);
        Pattern pCalibration = Pattern.compile(regxCalibration);
        Matcher mDrw = pDrw.matcher(retOutput);
        Matcher mDischarge = pDischarge.matcher(retOutput);
        Matcher mCalibration = pCalibration.matcher(retCalibration);

        while (mDrw.find()) {
            String fDrw = mDrw.group();
            remote.fileDownload(srcCalibration + fDrw, pathDown + fDrw);
        }
        while (mDischarge.find()) {
            String fDischarge = mDischarge.group();
            remote.fileDownload(srcCalibration + fDischarge, pathDown + fDischarge);
        }
        while (mCalibration.find()) {
            String fCalibration = mCalibration.group();
            remote.fileDownload(srcCalibration + fCalibration, pathDown + fCalibration);
        }
        // 关闭连接
        remote.closeSession();

        List<Map<String, Object>> discharge = readDataFromTxt(pathDown, "Discharge");
        List<Map<String, Object>> drw = readDataFromTxt(pathDown, "Drw");
        List<Map<String, Object>> calibration = readDataFromTxt(pathDown, "assess");

        return GBHMOutputData.builder()
                .dischargeList(discharge)
                .drwList(drw).calibration(calibration)
                .build();
    }

    /**
     * 5、处理后模型数据入库
     *
     * @param dataOut
     * @param dataBase
     */
    public void saveProcessedModelOutputData(GBHMOutputData dataOut, GBHMBasedata dataBase) {

        List<Map<String, Object>> dischargeList = dataOut.getDischargeList();
        List<Map<String, Object>> drwList = dataOut.getDrwList();
        FileCalibrationResult calibration = (FileCalibrationResult) dataOut.getCalibration().get(0).get("calibration");
        LocalDateTime dataTime = dataBase.getDataTime();
        LocalDateTime stTime = dataBase.getStTime();
        LocalDateTime endTime = dataBase.getEndTime();

        List<Map<String, Object>> dischargeDrwList = dischargeList.stream().peek(mDischarge -> drwList.stream()
                .filter(mDrw -> Objects.equals(mDischarge.get("stcd"), mDrw.get("stcd")))
                .forEach(each -> {
                    mDischarge.put("discharge", mDischarge.get("data"));
                    mDischarge.put("drw", each.get("data"));
                    mDischarge.remove("data");
                })).collect(Collectors.toList());

    }


    //6、数据统计


    /**
     * 从TXT文件读取处理后数据
     *
     * @param path     文件路径
     * @param fileName 文件名
     * @return List<Map < String, Object>>
     * @throws IOException IO异常
     */
    private List<Map<String, Object>> readDataFromTxt(String path, String fileName) throws IOException {

        List<Map<String, Object>> ret = new ArrayList<>();
        // 筛选符合条件的文件
        String regx = "";
        if ("Discharge".equals(fileName) || "Drw".equals(fileName)) {
            regx = "(" + fileName + "Hourly[0-9]+\\.txt)|(" + fileName + "Hourly[a-zA-Z]+[0-9]+\\.txt)";
        } else {
            regx = fileName + "\\.txt";
        }
        Pattern pattern = Pattern.compile(regx);
        File filePath = new File(path);
        List<String> fileN = new ArrayList<>();
        if (filePath.isDirectory()) {
            File[] files = filePath.listFiles();
            for (File eachF : files) {
                Matcher matcher = pattern.matcher(eachF.getName());
                if (matcher.find()) {
                    fileN.add(matcher.group());
                }
            }
        }

        if ("Discharge".equals(fileName) || "Drw".equals(fileName)) {
            // 读取输出的拟合数据
            // 读取文件
            for (String eachN : fileN) {
                Map<String, Object> map = new LinkedHashMap<>();
                File file = new File(path + eachN);
                // 筛选站号
                int end = eachN.indexOf(".txt");
                int start = end - 2;
                String stcd = eachN.substring(start, end);
                map.put("stcd", stcd);

                List<FileDataResult> fileDataResult = new ArrayList<>();
                BufferedReader bufReader = new BufferedReader(new FileReader(file));

                Integer i = 0;
                String line = "";
                while ((line = bufReader.readLine()) != null) {
                    String[] split = line.split("\\s+");
                    if (i > 23) {
                        i = 0;
                    }
                    fileDataResult.add(FileDataResult.builder()
                            .time(split[0] + " " + String.format("%02d", i) + ":00:00")
                            .value(split[1]).build());
                    i++;
                }
                map.put("data", fileDataResult);
                ret.add(map);
            }
            return ret;
        } else {
            // 读取校准参数
            Map<String, Object> map = new LinkedHashMap<>();
            File file = new File(path + fileN.get(0));

            List<FileDataResult> fileDataResult = new ArrayList<>();
            BufferedReader bufReader = new BufferedReader(new FileReader(file));
            // 获取最后一行
            String currLine = "", lastLine = "";
            while ((currLine = bufReader.readLine()) != null) {
                lastLine = currLine;
            }
            String[] split = lastLine.split(",");
            FileCalibrationResult result = FileCalibrationResult.builder()
                    .nash(split[0]).r2(split[1]).me(split[2]).pbias(split[3])
                    .build();
            map.put("calibration", result);
            ret.add(map);

            return ret;
        }
    }

    /**
     * 将数据保存为TXT文本
     *
     * @param path     保存路径
     * @param fileName 文件名
     * @param stcd     站点编号
     * @param dataList 数据列表
     * @param func     操作函数
     * @param <T>      列表类型
     * @param <R>      函数返回类型
     * @throws IOException IO异常
     */
    private <T, R> void saveDataToTxt(String path, String fileName, String stcd, List<T> dataList, Function<T, R> func) throws IOException {

        File file = new File(path + fileName + ".txt");
        if (!file.exists()) {
            file.createNewFile();
        }
        FileWriter fw = new FileWriter(file);
        fw.write("");
        fw.flush();

        BufferedWriter bw = new BufferedWriter(fw);
        dataList.stream()
                .map(func)
                .forEach(e -> {
                    try {
                        bw.write((String) e);
                        bw.newLine();
                    } catch (IOException ex) {
                        throw new RuntimeException(ex);
                    }
                });
        bw.flush();
        fw.close();
        bw.close();
    }
}
