package com.cuit.basin.model.job.handler;

import com.cuit.basin.framework.common.constant.ModleTaskEnum;
import com.cuit.basin.model.parse.ModelTaskManager;
import com.cuit.basin.model.pojo.InitTaskParams;
import com.cuit.basin.model.pojo.SuccessTaskParams;
import com.cuit.basin.model.service.ModelTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * @BelongsProject: sbasinModel
 * @BelongsPackage: com.cuit.sriver.model.job.handler
 * @Author: HouPJ
 * @Date: 2022/12/27 10:24
 * @Description: TODO
 */
@Component(value = "手动运行")
@Slf4j
public class ManualProcess implements ApiModelProcess{

    @Resource
    private ModelTaskService taskService;

    @Async
    @Override
    public void runModel(InitTaskParams params) {
        log.info("[Manual-model]：[当前运行线程为：{},模型运行方式为：{}]",Thread.currentThread().getName(),params.getRm());
        launchManualModel(params);
        log.info("[Manual-model]：[模型{}运行结束！]",params.getId());
        // 将当前任务从全局 map 中移除
        ModelTaskManager.modelTaskRunFlag.remove(params.getId());
    }

    public void launchManualModel(InitTaskParams initTaskParams) {
        String progress = "";
        log.info("模型开始运行......");
        int pro = 20;
        Boolean runFlag = null;
        runFlag = ModelTaskManager.modelTaskRunFlag.get(initTaskParams.getId());
        for (int i = 0; i < 5; i++) {
            log.info("任务{}的运行标识为：{}",initTaskParams.getId(),runFlag);
            if (runFlag) {
                progress = pro + "";
                try {
                    Thread.sleep(30000);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                if (initTaskParams.getId() != null) {
                    taskService.sendModelTaskProcess(initTaskParams, progress);
                    pro += 20;
                }
            }
        }
        if (runFlag) {
            taskService.sendModelTaskSuccess(SuccessTaskParams.builder()
                    .et(LocalDateTime.now())
                    .id(initTaskParams.getId())
                    .process(100.00)
                    .isFinish(ModleTaskEnum.TASK_FINISH.getType())
                    .build());
        }
    }

}
