package com.cuit.basin.model.parse;

import cn.hutool.core.util.ObjectUtil;
import com.cuit.basin.framework.common.utils.other.AssertUtil;
import com.cuit.basin.model.enums.ModelTypeEnums;
import com.cuit.basin.model.job.factories.ModelProcessFactory;
import com.cuit.basin.model.job.handler.ApiModelProcess;
import com.cuit.basin.model.pojo.InitTaskParams;
import com.cuit.basin.model.service.ModelTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @BelongsProject: sbasinModel
 * @BelongsPackage: com.cuit.sriver.model.parse
 * @Author: HouPJ
 * @Date: 2022/12/26 17:28
 * @Description: TODO
 */
@Component
@Slf4j
public class ModelTaskManager {

    private ApiModelProcess apiModel;

    @Resource
    private ModelTaskService modelTaskService;

    @Resource
    private ModelProcessFactory modelProcessFactory;



    /**
     * 存放各模型运行与否停止标志
     */
    public static Map<String,Boolean> modelTaskRunFlag = new ConcurrentHashMap<>();

    public void runModel(InitTaskParams params) {
        // 初始化当前任务的运行标识
        initCurrentModelRunFlag(params.getId());
        // 根据模型名称获取模型处理器
        apiModel = modelProcessFactory.createModelProcess(params.getMnmcn());
        AssertUtil.isTrue(ObjectUtil.isNull(apiModel),"模型：["+params.getMnmcn()+"]未在模型处理器工厂中注册！已注册模型列表为："+ Arrays.toString(ModelTypeEnums.values()));
        // 异步（多线程）调用模型
        apiModel.runModel(params);
    }

    /**
     * 设置当前任务的运行标识,默认为true
     * @param id 任务id
     */
    private void initCurrentModelRunFlag(String id) {
        modelTaskRunFlag.put(id,Boolean.TRUE);
    }

    /**
     * 停止运行中的模型任务
     * @param taskId 任务 id
     */
    public void stopModelTask(String taskId) {
        log.info("收到模型停止运行消息，要停止的模型id为：{}，当前运行中的任务列表为：{}",taskId,modelTaskRunFlag);
        if (modelTaskRunFlag.containsKey(taskId)) {
            modelTaskRunFlag.replace(taskId,Boolean.FALSE);
        }

    }


}
