package com.cuit.basin.model.service;

import cn.hutool.json.JSONUtil;
import com.cuit.basin.framework.common.constant.ModleTaskEnum;
import com.cuit.basin.framework.kafka.core.api.KafkaProducerApi;
import com.cuit.basin.framework.kafka.core.common.KafkaTopic;
import com.cuit.basin.model.convert.ModelLogConvert;
import com.cuit.basin.model.parse.ModelTaskManager;
import com.cuit.basin.model.pojo.InitTaskParams;
import com.cuit.basin.model.pojo.SuccessTaskParams;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;


/**
 * @Author: HouPJ
 * @Date: 2022/12/23 9:37
 * @Description: TODO
 */
@Service
@Slf4j
public class ModelTaskServiceImpl implements ModelTaskService {

    @Resource
    private KafkaProducerApi producer;

    @Override
    public void sendModelTaskProcess(InitTaskParams initTaskParams, String process) {
        SuccessTaskParams success = null;
        success = ModelLogConvert.INSTANCE.convert(initTaskParams);
        // 设置默认值
        if (null == success.getIsFinish()) {
            success.setIsFinish(ModleTaskEnum.TASK_RUNNING.getType());
        }
        success.setProcess(Double.valueOf(process));
        success.setEt(LocalDateTime.now());
        // 发送模型进度消息
        producer.sendMessageWithCallBack(KafkaTopic.TOPIC_MODEL_PROCESS, JSONUtil.toJsonStr(success));
    }

    @Override
    public void sendModelTaskSuccess(SuccessTaskParams success) {
        producer.sendMessageWithCallBack(KafkaTopic.TOPIC_MODEL_SUCCESS, JSONUtil.toJsonStr(success));
    }

    @Override
    public void sendModelTaskException(String taskId) {
        producer.sendMessageWithCallBack(KafkaTopic.TOPIC_MODEL_EXCEPTION, taskId);
    }

    @Override
    @Async
    public void runModel(InitTaskParams params) {
        log.info("当前运行线程为：{}",Thread.currentThread().getName());
        launchSHBModel(params);
    }

    public void launchSHBModel(InitTaskParams initTaskParams) {
        String progress = "";
        log.info("模型开始运行......");
        int pro = 20;
        Boolean runFlag = null;
        runFlag = ModelTaskManager.modelTaskRunFlag.get(initTaskParams.getId());
        for (int i = 0; i < 5; i++) {
            log.info("任务{}的运行标识为：{}",initTaskParams.getId(),runFlag);
            if (runFlag) {
                progress = pro + "";
                try {
                    Thread.sleep(30000);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                if (initTaskParams.getId() != null) {
                    sendModelTaskProcess(initTaskParams, progress);
                    pro += 20;
                }
            }
        }
        if (runFlag) {
            sendModelTaskSuccess(SuccessTaskParams.builder()
                    .et(LocalDateTime.now())
                    .id(initTaskParams.getId())
                    .process(100.00)
                    .isFinish(ModleTaskEnum.TASK_FINISH.getType())
                    .build());
        }
        // 将当前任务从全局 map 中移除
        ModelTaskManager.modelTaskRunFlag.remove(initTaskParams.getId());
    }





}
